// global variables definitions and global functions implementations
/* Date (yymmdd):               081215
 * Version:                     0.1
 * Written by:                  Hongqing Cao
 * Additional contributions by: 
				Dr. Francisco Jos Romero Campero
				Jonathan Blakes				
				Dr. Jamie Twycross
 *                              Dr. Natalio Krasnogor
 * Funded by:                   EPSRC grant EP/E017215/1
 * Website:                     http://www.infobiotic.org/
 * Contact:                     nxk@cs.nott.ac.uk
*/ 

#if HAVE_CONFIG_H
        #include <config.h>
#endif

#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <math.h>
#include <stdlib.h> 
#include <string.h>

#include <list>
#include <iostream>
#include <string>
#include <vector>
     
#include <signal.h>     // includes int kill(pid_t pid, int sig);
#include <unistd.h>     // includes fork()

#include <ecsb/ecsb.h>
#include <mcss/SimulationDatasets.h>

#include <gsl/gsl_statistics_double.h>
#include <gsl/gsl_cdf.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

#include "POptimizerCommon.h"
#include "POptimizerModule.h"

using namespace ECSB;
using namespace SimulationDatasets;

namespace POptimizer {


// global parameters
  int paraoptimethodno,methodno;
  int option_simu,option_mcheck;
  char multiinitfile[10][100],multiinputfile[10][100];

  float maxtime,interval;
  int maxmodulesno,maxbtypeno,maxmtypeno,maxrtypeno;
  unsigned long maxsteps;
  float scalevalue;
  int runs,totalruns;
  unsigned long seednum;
  bool show_progress;
  bool debug_mode;

  int popsize,maxgeno,maxop;
  int GApsize,GAmaxmu,GAmaxxo;
  int EDApsize,EDAmaxgeno;
  int DEpsize,DEmaxgeno;
  int CMAESmaxgeno;

 /* global variable*/
  char membranetype[3][20];
  int timesteps;
  float *normalizetime;
  int runno;
  float besttotalerror;
  float multiRMSE[10];
  int totalevanum;
  char allobjname[100][50];  
  float allobjinit[100];
  int allobjnum;
  int *allobjectsupperb;

  int targetobjnum;
  char **targetobjname;
  float ***targetobjvalue,**modelobjvalue;

  int varvaluenum;
  char varvaluename[10][50];

  int initsetnum;
  int nonzeroobjnum[10];
  char nonzeroobjname[10][50][50];
  float nonzeroobjinit[10][50];

  char testfile2[100];
  Module modulelib[10];	
  Module fixmodulelib[10];
  int fixmodulesno;

  gsl_rng *rng;
  int gapeevalnum;
	
  char fixlibrary_file[100];
  char library_file[100];
  int flagMDL;
  int flagfixlib;
  int varindex[5];

  
  float randfab(int a, int b);
  float get_value(int i);
  int randi(int i);
  float randf(void);
  int tournament_selection(int psize, int tsize, float *q);
  void read_target_file(int targetfileno, char *fname);
  float randfabwithprecision(float lowerb, float upperb, float precision);
  float ftruncate(float f, int sigdigit);
  float Gaussian_mutation(float para, float precision, int sigdigit, float fit, float lowerb, float upperb);
  float calculate_maximum(float *f, int num);
  int new_brinv(float **a, int n);
  int EDA_Bayesian_distribution(int paranum,float *indi, float **covarmatrix, float *means, float *bvariance, float *bmeans);
  int new_bchol(float **a,int n, float *det);
  float Gaussian_random_number(float mu, float sigma);
  void generate_random_normalized_weights(int num, float *weights);
  float calculate_minimum(float *f, int num);
  float cal_fitness_function(int methodno, int pointnum, int varnum, float **target, float **fitted);
  int precision_to_significant_digit(float precision);
  void write_parameter_xmlfile_for_simulator(char *parafilename, char *inputxmlfilename, char*outputh5filename);
  void formalize_simulation_number(int simuno, int maxsimuno, char *str);
  int get_real_object_alphabet_order(int objnum, char**allobjname, char *objname);
 // void write_parameter_xmlfile_for_modelcheker(int pno,char *commonname);
  //void write_file_combine_modelchecker_output(void);
  void get_allobjects_valuename(char *wholestring, char ch);
  void read_initialvalue_file(int initfileno, char *fname);
  //int call_simulator(int maxtime, char parafile[100]);
  void selectsampleno(int candino, int size, int *selectno);
  int new_cjcbi(float **a,int n, float **v,float eps,int jt);
  void new_brmul(float **a,float **b,int m,int n,int k,float **c);
  float my_pow(float base, int times);
  int my_int(float f);

// generate a random floating number less than integer i 
float get_value(int i)
{
	return((float)(i*1.0*(rand()%32767)/(32767+1.0)));
}

// generate a random floating number between integer a and integer b
float randfab(int a, int b)
{
	return((float)(get_value(b-a)+a));
}

/*int randi(int i)
{
	return((int)((float)i*(rand()%32767)/32768.0));
}
*/

// generate a random integer less than integer i using gsl function
int randi(int i)
{
	return((int)gsl_rng_uniform_int(rng,i));
}


/*float randf(void)
{
	return((float)(randi(32767)/32766.0));
}
*/

//generate a random floating number between 0 and 1 by using gsl function
float randf(void)
{
	return((float)gsl_rng_uniform(rng));
}

// select an individual from the population by using tournament selection with size tsize based on its fitness value
int tournament_selection(int psize, int tsize, float *q)
{
	int i,nn,candi[100];
	float tempq;
   
	for(i=0;i<tsize;i++)
		candi[i]=randi(psize);
	tempq=q[candi[0]]; 
	nn=candi[0];
	for(i=1; i<tsize; i++)
	{
		if(q[candi[i]]<tempq) 
	        {
			tempq=q[candi[i]];
	            nn=candi[i];
	        }
	}
	return(nn);
}	 

//read the time series data of all target objects from the input data file  
void read_target_file(int targetfileno, char *fname)
{

	char str[50];
	int i,j,k;
	FILE *fp;
	float time;
		
	//allocate space for names and values of tartget objects, values of model output 
	
	 
	if((fp=fopen(fname,"rt"))==NULL)
	{
		printf("Can not open target input file !\n");
		exit(1);
	}
	k=fscanf(fp,"%s",str);
	for(i=0;i<targetobjnum;i++)
		k=fscanf(fp,"%s",targetobjname[i]);
	i=0;
	while(!feof(fp))
	{
		k=fscanf(fp,"%f",&time);
		for(j=0;j<targetobjnum;j++)
			k=fscanf(fp,"%f",&targetobjvalue[targetfileno][j][i]);
		i++;
	}
}

//read the non-zero initial values of some objects in the file  
void read_initialvalue_file(int initfileno, char *fname)
{
	int i;
	FILE *fp;
	
	if((fp=fopen(fname,"rt"))==NULL)
	{
		printf("Can not open input file !\n");
		exit(1);
	}
	i=0;
	while(!feof(fp))
	{
		fscanf(fp,"%s",nonzeroobjname[initfileno][i]);
		fscanf(fp,"%f",&nonzeroobjinit[initfileno][i]);
		i++;
	}
	nonzeroobjnum[initfileno]=i-1;
	

}

//generate a random floating number with precision between lowerb and upperb 
float randfabwithprecision(float lowerb, float upperb, float precision)
{
	int totalnum;
	int rani;
	float tempf;

	totalnum=(int)((upperb-lowerb)/precision);
	rani=randi(totalnum)+1;
	tempf=lowerb+rani*precision;
	return(tempf);
}

float my_pow(float base, int times)
{
	float product;
	int i;

	product=1.0;
	for(i=0;i<times;i++)
		product=product*base;
	return(product);
}

int my_int(float f)
{
	char str[20];
	
	sprintf(str,"%f.0",f);
	return(atoi(str));
}
	

//truncate a floating number with assigned number of significant digit
float ftruncate(float f, int sigdigit)
{
	int tempi;
	float tempf;

	tempf=f*my_pow(10.0,sigdigit);
	tempi=my_int(tempf);
    tempf=tempi/my_pow(10.0,sigdigit);
	return(tempf);
}

//do Gaussion mutation to a parameter with assigned precision, significant digit, range(lower and upper bound) and fitness value 
/*float Gaussian_mutation(float para, float precision, int sigdigit, float fit, float lowerb, float upperb)
{
	float r1,r2,v1,v2,R,f,y1,tempp;
	
	do 
	{
		r1=randf();
		r2=randf();
		v1=2*r1-1;
		v2=2*r2-1;
		R=v1*v1+v2*v2;
	}while(R>=1||R==0);
	f=sqrt((-2.0)*log(R))/R;
	y1=v1*f;
	tempp=para+precision*fit*y1;
	tempp=ftruncate(tempp,sigdigit);
	if(tempp<=lowerb || tempp>upperb)
		tempp=randfabwithprecision(lowerb,upperb,precision);
	return(tempp);
}
*/
float Gaussian_mutation(float para, float precision, int sigdigit, float fit, float lowerb, float upperb)
{
	float r1,r2,v1,v2,R,f,y1,tempp;
	
	do 
	{
		r1=randf();
		r2=randf();
		v1=2*r1-1;
		v2=2*r2-1;
		R=v1*v1+v2*v2;
	}while(R>=1||R==0);
	f=sqrt((-2.0)*log(R))/R;
	y1=v1*f;
	tempp=para+0.1*(upperb-lowerb)*y1;
	tempp=ftruncate(tempp,sigdigit);
	if(tempp<=lowerb || tempp>upperb)
		tempp=randfabwithprecision(lowerb,upperb,precision);
	return(tempp);
}

/*float Gaussian_random_number(float mu, float sigma)
{
	float r1,r2,v1,v2,R,f,y1,temp;
	
	do 
	{
		//r1=gsl_rng_uniform(rng);
		//r2=gsl_rng_uniform(rng);
		r1=randf();
		r2=randf();
		v1=2*r1-1;
		v2=2*r2-1;
		R=v1*v1+v2*v2;
	}while(R>=1||R==0);
	f=sqrt((-2.0)*log(R))/R;
	y1=v1*f;
	temp=mu+sigma*y1;
	return temp;
}
*/

//generate a Gaussian random number with mean mu and variance sigma 
float Gaussian_random_number(float mu, float sigma)
{
	float temp;
	
	temp=mu+sigma*gsl_ran_ugaussian(rng);
	return temp;
}

int new_bchol(float **a,int n, float *det)
{ 
	int i,j,k;
	float d;

	if ((a[0][0]+1.0==1.0)||(a[0][0]<0.0))
	{ 
		  //printf("fail1\n");
		  //getchar();
		  return(-2);
	}
	a[0][0]=sqrt(a[0][0]);
	d=a[0][0];
	for(i=1; i<=n-1; i++)
		a[i][0]=a[i][0]/a[0][0];
	for(j=1; j<=n-1; j++)
	{
		for(k=0; k<=j-1; k++)
			a[j][j]=a[j][j]-a[j][k]*a[j][k];
		if((a[j][j]+1.0==1.0)||(a[j][j]<0.0))
		{ 
			//printf("fail2\n"); 
			return(-2);
		}
		a[j][j]=sqrt(a[j][j]);
		d=d*a[j][j];
		for (i=j+1; i<=n-1; i++)
		{ 
			for (k=0; k<=j-1; k++)
				a[i][j]=a[i][j]-a[i][k]*a[j][k];
			a[i][j]=a[i][j]/a[j][j];
		}
	}
	*det=d*d;
	for (i=0; i<=n-2; i++)
		for (j=i+1; j<=n-1; j++)
			a[i][j]=0.0;
	return(2);
}
				
int new_brinv(float **a, int n)
{ 
	int *is,*js,i,j,k;
	float d,p;
	
	is=(int *) malloc(n*sizeof(int));
	js=(int *) malloc(n*sizeof(int));

	for (k=0; k<=n-1; k++)
	{ 
		d=0.0;
	        for (i=k; i<=n-1; i++)
		        for (j=k; j<=n-1; j++)
		{ 
			if(a[i][j]<0)
				p=a[i][j]*(-1);
			else p=a[i][j];
			if (p>d) { d=p; is[k]=i; js[k]=j;}
	          }
        if (d+1.0==1.0)
	{ 
		free(is);
		free(js); //printf("err**not inv\n");
		return(0);
	}
        if (is[k]!=k)
		for (j=0; j<=n-1; j++)
		{ 
			p=a[k][j]; 
			a[k][j]=a[is[k]][j]; 
			a[is[k]][j]=p;
	         }
        if (js[k]!=k)
	for (i=0; i<=n-1; i++)
	{ 
		p=a[i][k]; 
		a[i][k]=a[i][js[k]]; 
		a[i][js[k]]=p;
	}
        
        a[k][k]=1.0/a[k][k];
        for (j=0; j<=n-1; j++)
	if (j!=k)
	{  
		a[k][j]=a[k][j]*a[k][k];}
	        for (i=0; i<=n-1; i++)
			if (i!=k)
				for (j=0; j<=n-1; j++)
			if (j!=k)
			{
				a[i][j]=a[i][j]-a[i][k]*a[k][j];
	                }
		for (i=0; i<=n-1; i++)
			if (i!=k)
			{
				a[i][k]=-a[i][k]*a[k][k];}
			}
		for (k=n-1; k>=0; k--)
		{ 
			if (js[k]!=k)				for (j=0; j<=n-1; j++)
			{ 
				p=a[k][j];
				a[k][j]=a[js[k]][j];
				a[js[k]][j]=p;
            		}
        	if (is[k]!=k)
			for (i=0; i<=n-1; i++)
		{
			p=a[i][k]; 
			a[i][k]=a[i][is[k]]; 
			a[i][is[k]]=p;
		}
	}
	free(is); 
	free(js);
	return(1);
}

//get the maximum in a floating number array 
float calculate_maximum(float *f, int num)
{
	float tempmax;
	int i;

	tempmax=f[0];
	for(i=0;i<num;i++)
		if(f[i]>tempmax)
			tempmax=f[i];
	return(tempmax);
}

//get the minimum in a floating number array
float calculate_minimum(float *f, int num)
{
	float tempmin;
	int i;

	tempmin=f[0];
	for(i=0;i<num;i++)
		if(f[i]<tempmin)
			tempmin=f[i];
	return(tempmin);
}

//get a number of random weights within the range of (0,1) and their sum equals to 1 
void generate_random_normalized_weights(int num, float *weights)
{
	int i,j;
	float subsum;
	float tempf;

	tempf=randf();
	weights[0]=1-(float)pow((float)tempf,(float)(1.0/((num-1)+0.0)));
	
	for(i=1;i<num-1;i++)
	{
		subsum=0;
		for(j=0;j<i;j++)
			subsum=subsum+weights[j];
		tempf=randf();
		weights[i]=(1-subsum)*(1-(float)pow((float)tempf,(float)(1.0/((num-1-i)+0.0))));
	};

	subsum=0;
	for(i=0;i<num-1;i++)
			subsum=subsum+weights[i];
	weights[num-1]=1-subsum;
}


float cal_fitness_function(int methodno, int pointnum, int varnum, float **target, float **fitted)
{
	float **abs_error;
	float *sum;
	float *totalsubsum, *random_weights;
	float totalsum;
	int rantimes;
	int i,j;

	totalsum=0;
	abs_error=(float **)malloc(varnum*sizeof(float *));
	for(i=0;i<varnum;i++)
		abs_error[i]=(float *)malloc(pointnum*sizeof(float));
	sum=(float *)malloc(varnum*sizeof(float));
	for(i=0;i<varnum;i++)
		for(j=0;j<pointnum;j++)
			abs_error[i][j]=fabs(fitted[i][j]-target[i][j]);
		
	if(methodno==1)
	{
		for(i=0;i<varnum;i++)
		{
			sum[i]=0;
			for(j=0;j<pointnum;j++)
				sum[i]=sum[i]+abs_error[i][j];
			
		}
		totalsum=0;
		for(i=0;i<varnum;i++)
			totalsum=totalsum+sum[i];
	};

	if(methodno==2)
	{
		rantimes=100;
		totalsubsum=(float *)malloc(rantimes*sizeof(float));
		for(i=0;i<varnum;i++)
		{
			sum[i]=0;
			for(j=0;j<pointnum;j++)
				sum[i]=sum[i]+abs_error[i][j];
			
		}
		random_weights=(float *)malloc(varnum*sizeof(float));
		for(j=0;j<rantimes;j++)
		{
			generate_random_normalized_weights(varnum,random_weights);
			totalsubsum[j]=0;
			for(i=0;i<varnum;i++)
				totalsubsum[j]=totalsubsum[j]+random_weights[i]*sum[i];
		}
		totalsum=0;
		for(i=0;i<rantimes;i++)
			totalsum=totalsum+totalsubsum[i];
		totalsum=totalsum/(rantimes+0.0);
		free(totalsubsum);
		free(random_weights);
	};
	
	for(i=0;i<varnum;i++)
		free(abs_error[i]);
	free(abs_error);
	free(sum);
	return(totalsum);
}

//get the number of significant digit by assigned precision
int precision_to_significant_digit(float precision)
{

	float temp;
	int i;
	
	temp=precision;
	i=0;
	if(precision<1.0)
	{
		while(temp<0.999999)
		{
			
			temp=10.0*temp;
			i++;
		}
		return(i);
	};
	if(precision==1.0)
		return(0);
	if(precision>1.0)
	{
		while(temp>0.999999)
		{
			temp=temp/10.0;
			i++;
		}
		return((-1)*i);
	};
	return(0);
}

//write xml file for the input parameters required by the simulator
void write_parameter_xmlfile_for_simulator(char *parafilename, char *inputxmlfilename, char*outputh5filename)
{
	FILE *fp;
	char st[1000];

	if((fp=fopen(parafilename,"wt"))==NULL)
	{
		printf("Cannot open output file !\n");
		exit(1);
	}
	strcpy(st,"<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n\n");
	fprintf(fp,"%s",st);
	strcpy(st,"<parameters xmlns=\"http://www.cpib.ac.uk/~jpt\"\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\txmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\txsi:schemaLocation=\"http://www.cpib.ac.uk/~jpt parameter.xsd\">\n\n");
	fprintf(fp,"%s",st);
	strcpy(st,"<!-- parameter definitions -->\n");
	fprintf(fp,"%s",st);
	strcpy(st,"<parameterSet name=\"SimulationParameters\">\n");
	fprintf(fp,"%s",st);
	sprintf(st,"\t<parameter name=\"model_file\" value=\"%s\"/>\n",inputxmlfilename);
	fprintf(fp,"%s",st);
	sprintf(st,"\t<parameter name=\"max_time\" value=\"%.0f\"/>\n",maxtime);
	fprintf(fp,"%s",st);
	strcpy(st,"\t<parameter name=\"simulation_algorithm\" value=\"dmq\"/>\n");
	fprintf(fp,"%s",st);
	sprintf(st,"\t<parameter name=\"data_file\" value=\"%s\"/>\n",outputh5filename);
	fprintf(fp,"%s",st);
	strcpy(st,"\t<parameter name=\"log_type\" value=\"levels\"/>\n");
	fprintf(fp,"%s",st);
	sprintf(st,"\t<parameter name=\"log_interval\" value=\"%.0f\"/>\n",interval);
	fprintf(fp,"%s",st);
	strcpy(st,"\t<parameter name=\"duplicate_initial_amounts\" value=\"0\"/>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t<parameter name=\"model_format\" value=\"xml\"/>\n");
	fprintf(fp,"%s",st);
	sprintf(st,"\t<parameter name=\"runs\" value=\"%d\"/>\n",runs);
	fprintf(fp,"%s",st);
	strcpy(st,"\t<parameter name=\"show_progress\" value=\"false\"/>\n");
	fprintf(fp,"%s",st);
	sprintf(st,"\t<parameter name=\"seed\" value=\"%d\"/>\n",randi(2147483647));
	fprintf(fp,"%s",st);
	strcpy(st,"</parameterSet>\n\n");
	fprintf(fp,"%s",st);
	strcpy(st,"</parameters>");
	fprintf(fp,"%s",st);
	fclose(fp);
}

//formalize the simulation number according to the maximum of the simulation number. For example, if the maximum is 1000, the simulation number will have the format as 0001~1000
void formalize_simulation_number(int simuno, int maxsimuno, char *str)
{
	float temp;
	char zerostr[10];
	int i,maxnolen,simunolen,zeronum;
	int tempint,digit;
	
	temp=(float)maxsimuno;
	if(temp<10)
		i=1;
	else
	{		
		i=0;		
		while(temp>0.999999)
		{
			temp=temp/10.0;
			i++;
		}
	}
	maxnolen=i;
	i=0;
	tempint=simuno;
	while(tempint>=1)
	{
		tempint=tempint/10;
		digit=tempint%10;
		i++;	
	}
	simunolen=i;
	zeronum=maxnolen-simunolen;
	strcpy(zerostr,"");
	for(i=0;i<zeronum;i++)
		strcat(zerostr,"0");
	sprintf(str,"%s%d",zerostr,simuno);
}


//write a number of xml files of protein and rna for the input parameters required by the model checker
/*void write_parameter_xmlfile_for_modelcheker(int pno,char *commonname)
{
	FILE *fp;
	char st[1000];
	char filename[100];
	int i,j;

	for(i=0;i<pno;i++)
	{
		
		sprintf(filename,"protein%d.xml",i+1);		
		if((fp=fopen(filename,"wt"))==NULL)
		{
			printf("Cannot open output file !\n");
			exit(1);
		}
		strcpy(st,"<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n\n");
		fprintf(fp,"%s",st);
		strcpy(st,"<parameters name=\"Model_checker_parameters\">\n\n");
		fprintf(fp,"%s",st);
		strcpy(st,"\t<!-- parameter definitions -->\n");
		fprintf(fp,"%s",st);
 		strcpy(st,"\t<parameterSet name=\"PRISMParameters\">\n");
		fprintf(fp,"%s",st);
    		sprintf(st,"\t\t<parameter name=\"P_system_specification\" value=\"%s.xml\"/>\n",commonname);
		fprintf(fp,"%s",st);
		sprintf(st,"\t\t<parameter name=\"PRISM_model\" value=\"%s.sm\"/>\n",commonname);
		fprintf(fp,"%s",st);
		strcpy(st,"\t\t<parameter name=\"temporal_formulas\" value=\"formulas.scl\"/>\n");
		fprintf(fp,"%s",st);
    		strcpy(st,"\t\t<parameter name=\"model_parameters\"\n\n");
		fprintf(fp,"%s",st);
	       	strcpy(st,"value=\"");
		fprintf(fp,"%s",st);
		for(j=0;j<objectsrealnum-1;j++)
		{
			sprintf(st,"upb_%s_bact=%d,lob_%s_bact=0,",allobjectsname[j],allobjectsupperb[j],allobjectsname[j]);
			fprintf(fp,"%s",st);
		}
		sprintf(st,"upb_%s_bact=%d,lob_%s_bact=0\"/>\n",allobjectsname[objectsrealnum-1],allobjectsupperb[j],allobjectsname[objectsrealnum-1]);
		fprintf(fp,"%s",st);
		sprintf(st,"\t\t<parameter name=\"formula_parameters\" value=\"T=0:%.0f:%.0f,molecule=%d,rule=0\"/>\n",interval,maxtime,get_real_object_alphabet_order(objectsrealnum,allobjectsname,namep[i]));
		fprintf(fp,"%s",st);
    		strcpy(st,"\t\t<parameter name=\"task\" value=\"Verify\"/>\n");
		fprintf(fp,"%s",st);
    		strcpy(st,"\t\t<parameter name=\"number_samples\" value=\"1000\" />\n");
		fprintf(fp,"%s",st);
    		strcpy(st,"\t\t<parameter name=\"confidence\" value=\"0.001\" />\n");
		fprintf(fp,"%s",st);
    		strcpy(st,"\t\t<parameter name=\"states_file\" value=\"file_states.psm\" />\n");
		fprintf(fp,"%s",st);
    		strcpy(st,"\t\t<parameter name=\"transitions_file\" value=\"file_transitions.psm\" />\n");
		fprintf(fp,"%s",st);
    		sprintf(st,"\t\t<parameter name=\"results_file\" value=\"%s_output.psm\" />\n",namep[i]);
		fprintf(fp,"%s",st);  		
		strcpy(st,"\t</parameterSet>\n\n");
		fprintf(fp,"%s",st);
		strcpy(st,"</parameters>\n");
		fprintf(fp,"%s",st);
		fclose(fp);
	}
	for(i=0;i<pno;i++)
	{
		
		sprintf(filename,"rna%d.xml",i+1);		
		if((fp=fopen(filename,"wt"))==NULL)
		{
			printf("Cannot open output file !\n");
			exit(1);
		}
		strcpy(st,"<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n\n");
		fprintf(fp,"%s",st);
		strcpy(st,"<parameters name=\"Model_checker_parameters\">\n\n");
		fprintf(fp,"%s",st);
		strcpy(st,"\t<!-- parameter definitions -->\n");
		fprintf(fp,"%s",st);
 		strcpy(st,"\t<parameterSet name=\"PRISMParameters\">\n");
		fprintf(fp,"%s",st);
    		sprintf(st,"\t\t<parameter name=\"P_system_specification\" value=\"%s.xml\"/>\n",commonname);
		fprintf(fp,"%s",st);
		sprintf(st,"\t\t<parameter name=\"PRISM_model\" value=\"%s.sm\"/>\n",commonname);
		fprintf(fp,"%s",st);
		strcpy(st,"\t\t<parameter name=\"temporal_formulas\" value=\"formulas.scl\"/>\n");
		fprintf(fp,"%s",st);
    		strcpy(st,"\t\t<parameter name=\"model_parameters\"\n\n");
		fprintf(fp,"%s",st);	       	
		strcpy(st,"value=\"");
		fprintf(fp,"%s",st);
		for(j=0;j<objectsrealnum-1;j++)
		{
			sprintf(st,"upb_%s_bact=%d,lob_%s_bact=0,",allobjectsname[j],allobjectsupperb[j],allobjectsname[j]);
			fprintf(fp,"%s",st);
		}
		sprintf(st,"upb_%s_bact=%d,lob_%s_bact=0\"/>\n",allobjectsname[objectsrealnum-1],allobjectsupperb[j],allobjectsname[objectsrealnum-1]);
		fprintf(fp,"%s",st);
    		sprintf(st,"\t\t<parameter name=\"formula_parameters\" value=\"T=0:%.0f:%.0f,molecule=%d,rule=0\"/>\n",interval,maxtime,get_real_object_alphabet_order(objectsrealnum,allobjectsname,namer[i]));
		fprintf(fp,"%s",st);
    		strcpy(st,"\t\t<parameter name=\"task\" value=\"Verify\"/>\n");
		fprintf(fp,"%s",st);
    		strcpy(st,"\t\t<parameter name=\"number_samples\" value=\"1000\" />\n");
		fprintf(fp,"%s",st);
    		strcpy(st,"\t\t<parameter name=\"confidence\" value=\"0.001\" />\n");
		fprintf(fp,"%s",st);
    		strcpy(st,"\t\t<parameter name=\"states_file\" value=\"file_states.psm\" />\n");
		fprintf(fp,"%s",st);
    		strcpy(st,"\t\t<parameter name=\"transitions_file\" value=\"file_transitions.psm\" />\n");
		fprintf(fp,"%s",st);
    		sprintf(st,"\t\t<parameter name=\"results_file\" value=\"%s_output.psm\" />\n",namer[i]);
		fprintf(fp,"%s",st);  	
  		strcpy(st,"\t</parameterSet>\n\n");
		fprintf(fp,"%s",st);
		strcpy(st,"</parameters>\n");
		fprintf(fp,"%s",st);
		fclose(fp);
	}
}
*/
//read the output of protein and rna from multiple psm files of model checker and combine them into one text file  
/*void write_file_combine_modelchecker_output(void)
{

	char str[100],title[600],fname[100];
	int i,j,k;
	FILE *fp;
	float time;
		
	
	for(i=0;i<totalp;i++)
	{
		
		sprintf(fname,"%s_output.psm",namep[i]);		
		if((fp=fopen(fname,"rt"))==NULL)
		{
			printf("Can not open input file !\n");
			exit(1);
		}
		k=fscanf(fp,"R=? [ I=T ]:\n%s",str);
		k=fscanf(fp,"%s",str);
		j=0;
		while(!feof(fp))
		{
			k=fscanf(fp,"%f",&time);
			k=fscanf(fp,"%f",&averageprotein[j][i]);
			j++;
		}
		fclose(fp);
	}
	for(i=0;i<totalp;i++)
	{
		
		sprintf(fname,"%s_output.psm",namer[i]);		
		if((fp=fopen(fname,"rt"))==NULL)
		{
			printf("Can not open input file !\n");
			exit(1);
		}
		k=fscanf(fp,"R=? [ I=T ]:\n%s",str);
		k=fscanf(fp,"%s",str);
		j=0;
		while(!feof(fp))
		{
			k=fscanf(fp,"%f",&time);
			k=fscanf(fp,"%f",&averagerna[j][i]);
			j++;
		}
		fclose(fp);
	}
	sprintf(fname,"bestsimulation_Run%d.txt",runno);
	if((fp=fopen(fname,"wt"))==NULL)
	{
		printf("Cannot open output file !\n");
		exit(1);
	}
	strcpy(title,"step  Time    ");
	for(i=0;i<totalp;i++)
	{
		sprintf(str,"model_protein%d",i+1);
		strcat(title,str); 
		strcat(title,"        ");
	}
	for(i=0;i<totalp;i++)
	{
		sprintf(str,"target_protein%d",i+1);
		strcat(title,str); 
		strcat(title,"        ");
	}
	for(i=0;i<totalp;i++)
	{
		sprintf(str,"model_rna%d",i+1);
		strcat(title,str); 
		strcat(title,"        ");
	}
	for(i=0;i<totalp;i++)
	{
		sprintf(str,"target_rna%d",i+1);
		strcat(title,str); 
		strcat(title,"        ");
	}
	fprintf(fp,"%s\n",title);
	for(i=0;i<timesteps;i++)
	{
		fprintf(fp,"%d   ",i);
		fprintf(fp,"%d    ",(int)normalizetime[i]);
		for(j=0;j<totalp;j++)
			fprintf(fp,"%f   ",averageprotein[i][j]);
		for(j=0;j<totalp;j++)
			fprintf(fp,"%f   ",targetprotein[i][j]);
		for(j=0;j<totalp;j++)
			fprintf(fp,"%f   ",averagerna[i][j]);
		for(j=0;j<totalp;j++)
			fprintf(fp,"%f   ",targetrna[i][j]);
		strcpy(str,"\n");
		fprintf(fp,"%s",str);
	}
	fclose(fp);
}
*/
//parse the string in the parameter file with seperator ch and get all possible value name for variables
void get_allobjects_valuename(char *wholestring, char ch)
{
	int i,j,k,pos;
	char str[500],tempstr[500], *pdest;

	strcpy(tempstr, wholestring);
	i=0;
	while(strcmp(tempstr,"")!=0)
	{
		pdest=strchr(tempstr,ch);
		if(pdest==NULL)
		{
			strcpy(varvaluename[i],tempstr);
			varvaluenum=i+1;
			return;
		}
		else
		{
			pos=pdest-tempstr;
			for(j=0;j<pos;j++)
				varvaluename[i][j]=tempstr[j];
			varvaluename[i][j]='\0';
			strcpy(str,tempstr);
			k=0;
			for(j=pos+1;j<strlen(str);j++)
			{
				tempstr[k]=str[j];
				k++;
			}
			tempstr[k]='\0';
			i++;
		}		
	}
}

/* int call_simulator(int maxtime, char parafile[100])
{
    // get child process using fork
    // fork() duplicates the running process and returns the parents process id,
    // pid, to the child process and the child's process id to the parent
    

	int pid;
    if((pid = fork()) < 0)
    {
        // pid < 0 means something went wrong
		printf("failed to fork");
        return(1);
    }
    if(pid == 0)
       execlp("mcss", "mcss", parafile, NULL);
    else
    { 
       int seconds = 0;
        int status = 0; // needed by waitpid, but not by us 
        while(true)
        {
            int rc = waitpid(pid, &status, WNOHANG);
            if(rc == -1) 
            {
                // mcss has finished normally
	       // printf("\nreasonable time! \n");
                return 0;
		//break;
            }
            if(rc != -1 && seconds > maxtime)
            {
                // mcss is taking too long, tell it to stop
                kill(pid, SIGINT);
                //printf("\ntime too long!\n");
		return 1;                
		//break;
            }
            sleep(1); // pause for 1 second
            seconds++;
        }
    }
    return 1;
} */

void selectsampleno(int candino, int size, int *selectno)
{
	int i,j,n; 
		
	for(i=0;i<size;i++)
		selectno[i]=i;
	for(i=0;i<6;i++)
	{
		if(i==0)
			j=candino;
		else
			j=randi(size-i)+i;
		n=selectno[i];
		selectno[i]=selectno[j];
		selectno[j]=n;
	}
//		printf("candino=%d  select[0]=%d  select[1]=%d select[2]=%d select[3]=%d select[4]=%d select[5]=%d \n",candino,selectno[0],selectno[1],selectno[2],selectno[3],selectno[4],selectno[5]);
}



int new_cjcbi(float **a,int n,float **v,float eps,int jt)
{ 
	int i,j,p,q,l;
    int fm,cn,sn,omega,x,y,d;
    
	l=1;
    for(i=0;i<=n-1;i++)
    { 
		v[i][i]=1.0;
        for(j=0;j<=n-1;j++)
          if(i!=j) 
			  v[i][j]=0.0;
    }
    while(1==1)
    { 
		fm=0.0;
        for(i=1;i<=n-1;i++)
	        for(j=0;j<=i-1;j++)
		    { 
				d=fabs(a[i][j]);
				if((i!=j)&&(d>fm))
				{ 
					fm=d;
					p=i; 
					q=j;
				}
			}
			if(fm<eps)
				return(1);
			if(l>jt)
				return(-1);
			l=l+1;
			x=(-1)*a[p][q]; 
			y=(a[q][q]-a[p][p])/2.0;
			omega=x/sqrt(x*x+y*y);
			if(y<0.0) 
				omega=-omega;
			sn=1.0+sqrt(1.0-omega*omega);
			sn=omega/sqrt(2.0*sn);
			cn=sqrt(1.0-sn*sn);
			fm=a[p][p];
			a[p][p]=fm*cn*cn+a[q][q]*sn*sn+a[p][q]*omega;
			a[q][q]=fm*sn*sn+a[q][q]*cn*cn-a[p][q]*omega;
			a[p][q]=0.0;
			a[q][p]=0.0;
			for(j=0;j<=n-1;j++)
				if((j!=p)&&(j!=q))
				{ 
					fm=a[p][j];
					a[p][j]=fm*cn+a[q][j]*sn;
					a[q][j]=(-1)*fm*sn+a[q][j]*cn;
				}
			for(i=0;i<=n-1;i++)
				if((i!=p)&&(i!=q))
				{ 
					fm=a[i][p];
					a[i][p]=fm*cn+a[i][q]*sn;
					a[i][q]=-fm*sn+a[i][q]*cn;
				}
			for(i=0;i<=n-1;i++)
			{ 
				fm=v[i][p];
				v[i][p]=fm*cn+v[i][q]*sn;
				v[i][q]=(-1)*fm*sn+v[i][q]*cn;
			}
		}
    return(1);
  }


// matrix multiply
void new_brmul(float **a,float **b,int m,int n,int k,float **c)
{ 
	int i,j,l;
    
	for(i=0;i<=m-1;i++)
		for(j=0;j<=k-1;j++)
		{ 
			c[i][j]=0.0;
			for(l=0;l<=n-1;l++)
				c[i][j]=c[i][j]+a[i][l]*b[l][j];
		}
	return;
 }


}

