// global variables and global functions Definitions
/* Date (yymmdd):               081215
 * Version:                     0.1
 * Written by:                  Hongqing Cao
 * Additional contributions by: 
				Dr. Francisco Jos Romero Campero
				Jonathan Blakes				
				Dr. Jamie Twycross
 *                              Dr. Natalio Krasnogor
 * Funded by:                   EPSRC grant EP/E017215/1
 * Website:                     http://www.infobiotic.org/
 * Contact:                     nxk@cs.nott.ac.uk
*/ 

#ifndef COMMON_H
#define COMMON_H

#include "string.h"
#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <math.h>
#include <stdlib.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

#include <mcss/SimulationDatasets.h>
#include "POptimizerModule.h"

namespace POptimizer{

/* global parameters*/
extern int paraoptimethodno,methodno;
extern int option_simu,option_mcheck;
extern char multiinputfile[10][100],multiinitfile[10][100];
extern char testfile2[100];
extern float maxtime,interval;

extern int maxmodulesno,maxbtypeno,maxmtypeno,maxrtypeno;
extern unsigned long maxsteps;
extern float scalevalue;
extern int runs,totalruns;
extern unsigned long seednum;
extern bool show_progress;
extern bool debug_mode;

extern int popsize,maxgeno,maxop;
extern int GApsize,GAmaxmu,GAmaxxo;
extern int EDApsize,EDAmaxgeno;
extern int DEpsize,DEmaxgeno;
extern int CMAESmaxgeno;

/* global variable*/
extern char membranetype[3][20];
extern int timesteps;
extern float *normalizetime;
extern int runno;
extern float besttotalerror;
extern float multiRMSE[10];
extern int totalevanum;
extern char allobjname[100][50];
extern float allobjinit[100];
extern int allobjnum;
extern int *allobjectsupperb;

extern int targetobjnum;
extern char **targetobjname;
extern float ***targetobjvalue, **modelobjvalue;

extern int varvaluenum;
extern char varvaluename[10][50];

extern int initsetnum;
extern int nonzeroobjnum[10];
extern char nonzeroobjname[10][50][50];
extern float nonzeroobjinit[10][50];

extern Module modulelib[10];	
extern Module fixmodulelib[10];
extern int fixmodulesno;

extern char library_file[100];
extern char fixlibrary_file[100];

extern int flagMDL;
extern gsl_rng *rng;
extern int gapeevalnum;
extern int flagfixlib;
extern int varindex[5];


extern float randfab(int a, int b);
extern float get_value(int i);
extern int randi(int i);
extern float randf(void);
extern int tournament_selection(int psize, int tsize, float *q);
extern void read_target_file(int targetfileno, char *fname);
extern float randfabwithprecision(float lowerb, float upperb, float precision);
extern float ftruncate(float f, int sigdigit);
extern float Gaussian_mutation(float para, float precision, int sigdigit, float fit, float lowerb, float upperb);
extern float calculate_maximum(float *f, int num);
extern int new_brinv(float **a, int n);
extern int EDA_Bayesian_distribution(int paranum,float *indi, float **covarmatrix, float *means, float *bvariance, float *bmeans);
extern int new_bchol(float **a,int n, float *det);
extern float Gaussian_random_number(float mu, float sigma);
extern void generate_random_normalized_weights(int num, float *weights);
extern float calculate_minimum(float *f, int num);
extern float cal_fitness_function(int methodno, int pointnum, int varnum, float **target, float **fitted);
extern int precision_to_significant_digit(float precision);
extern void write_parameter_xmlfile_for_simulator(char *parafilename, char *inputxmlfilename, char*outputh5filename);
extern void formalize_simulation_number(int simuno, int maxsimuno, char *str);
extern int get_real_object_alphabet_order(int objnum, char**allobjname, char *objname);
extern void write_parameter_xmlfile_for_modelcheker(int pno,char *commonname);
extern void write_file_combine_modelchecker_output(void);
extern void get_allobjects_valuename(char *wholestring, char ch);
extern void read_initialvalue_file(int initfileno, char *fname);
//extern int call_simulator(int maxtime, char parafile[100]);
extern void selectsampleno(int candino, int size, int *selectno);
extern int new_cjcbi(float **a,int n, float **v,float eps,int jt);
extern void new_brmul(float **a,float **b,int m,int n,int k,float **c);
extern float my_pow(float base, int times);
extern int my_int(float f);

}

#endif

