// GAoptimizer class implementations
/* Date (yymmdd):               081215
 * Version:                     0.1
 * Written by:                  Hongqing Cao
 * Additional contributions by: 
				Dr. Francisco Jos Romero Campero
				Jonathan Blakes				
				Dr. Jamie Twycross
 *                              Dr. Natalio Krasnogor
 * Funded by:                   EPSRC grant EP/E017215/1
 * Website:                     http://www.infobiotic.org/
 * Contact:                     nxk@cs.nott.ac.uk
*/ 

#if HAVE_CONFIG_H
        #include <config.h>
#endif
 
#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <math.h>
#include <stdlib.h> 
#include <string.h>

#include "POptimizerCommon.h"
#include "POptimizerRule.h"
#include "POptimizerModule.h"
#include "POptimizerMembrane.h"
#include "POptimizerGAoptimizer.h"

#include <fstream>
#include <iostream>
#include <string>
#include <sstream>
#include <ostream>
#include <map>

#include <ecsb/ecsb.h>

using namespace std;
using namespace xmlpp;

using namespace POptimizer;
using namespace ECSB;

//constructor
GAoptimizer::GAoptimizer()
{
	float temp;
	int i,j,k;
	int index;
	std::string sstr;
	std::vector<std::string> objstr,objstr1,objstr2;
	std::vector<ECSB::LPPRule> modulerules;
	float value1,value2;
	float tempf;
	char *pdest;
	map<std::string,ECSB::ModuleDefinition>::iterator it;
	
	// setup signal handlers
	// sigint handler (graceful exit)
	SignalHandler::set_handler(SIGINT, sigc::mem_fun(this, \
			&POptimizer::GAoptimizer::sigintHandler));
	// sigusr1 handler (optimizer information)
	SignalHandler::set_handler(SIGUSR1, sigc::mem_fun(this, \
			&POptimizer::GAoptimizer::sigusr1Handler));

	if(flagfixlib==1)
	{
		ECSB::ModuleLibrary fixlibrary(fixlibrary_file);
		std::map<std::string,ECSB::ModuleDefinition> fixlibraryModules = fixlibrary.getModules();
		
		index=0;
		maxmtypeno=1;
	 	// read modules from library and convert them into the data structure of POptimizer
		for(it = fixlibraryModules.begin(); it != fixlibraryModules.end(); ++it)	
		{
			sstr=(it->second).getName();
			strcpy(fixmodulelib[index].name,sstr.c_str());
			fixmodulelib[index].index=index;
			objstr=(it->second).getObjects();
			fixmodulelib[index].varnum=objstr.size();
			if(objstr.size()==0)
				fixmodulelib[index].fixflag=1;
			else
				fixmodulelib[index].fixflag=0;
			for(i=0;i<objstr.size();i++)
			{
				strcpy(fixmodulelib[index].varname[i],objstr[i].c_str());
				strcpy(fixmodulelib[index].varvaluestr[i],"");
				fixmodulelib[index].varvalueindex[i]=-1;
			}
			modulerules=(it->second).getRules();
			fixmodulelib[index].rulesno=modulerules.size();
			objstr=(it->second).getConstants();
			for(i=0;i<objstr.size();i++)
			{					
				strcpy(fixmodulelib[index].ruleset[i].name,modulerules[i].getName().c_str());
				value1=(float)((it->second).getLowerBounds())[objstr[i]];
				value2=(float)((it->second).getUpperBounds())[objstr[i]];
				if(value1==value2)
					fixmodulelib[index].ruleset[i].kcfixflag=1;
				else 
					fixmodulelib[index].ruleset[i].kcfixflag=0;
				fixmodulelib[index].ruleset[i].kclowerb=value1;
				fixmodulelib[index].ruleset[i].kcupperb=value2;
				tempf=(float)((it->second).getPrecisions())[objstr[i]];
				fixmodulelib[index].ruleset[i].kcprecision=tempf;
				fixmodulelib[index].ruleset[i].kcsigdigit=precision_to_significant_digit(tempf);
				if(fixmodulelib[index].ruleset[i].kcfixflag==1)
					fixmodulelib[index].ruleset[i].kcvalue=value1;
				sstr=((it->second).getScales())[objstr[i]];
				strcpy(fixmodulelib[index].ruleset[i].kcscale,sstr.c_str());
			}
			for(i=0;i<modulerules.size();i++)
			{
				objstr1=modulerules[i].getLhsInside();
				for(j=0;j<objstr1.size();j++)
				{
					strcpy(fixmodulelib[index].ruleset[i].lowithvar[j],"");
					strcpy(fixmodulelib[index].ruleset[i].lowithvalue[j],"");
				}
				objstr2=modulerules[i].getRhsInside();
				for(j=0;j<objstr2.size();j++)
				{
					strcpy(fixmodulelib[index].ruleset[i].rowithvar[j],"");
					strcpy(fixmodulelib[index].ruleset[i].rowithvalue[j],"");
				}
			}
			for(i=0;i<modulerules.size();i++)
			{	
				objstr1=modulerules[i].getLhsInside();
				fixmodulelib[index].ruleset[i].lonum=objstr1.size();
				for(j=0;j<objstr1.size();j++)
					strcpy(fixmodulelib[index].ruleset[i].lowithvar[j],objstr1[j].c_str());
				objstr2=modulerules[i].getRhsInside();
				fixmodulelib[index].ruleset[i].ronum=objstr2.size();
				for(j=0;j<objstr2.size();j++)
					strcpy(fixmodulelib[index].ruleset[i].rowithvar[j],objstr2[j].c_str());
			}
			if(modulelib[index].fixflag==1)
				fixmodulelib[index].ruleset[i].objfixflag=1;
			else
				for(i=0;i<fixmodulelib[index].rulesno;i++)
				{
					fixmodulelib[index].ruleset[i].objfixflag=1;
					for(j=0;j<fixmodulelib[index].ruleset[i].lonum;j++)
						for(k=0;k<fixmodulelib[index].varnum;k++)
						{
							pdest=strstr(fixmodulelib[index].ruleset[i].lowithvar[j],fixmodulelib[index].varname[k]);
							if(pdest!=NULL)
							{
								fixmodulelib[index].ruleset[i].objfixflag=0;
								break;
							}
						}
					for(j=0;j<fixmodulelib[index].ruleset[i].ronum;j++)
						for(k=0;k<fixmodulelib[index].varnum;k++)
						{
							pdest=strstr(fixmodulelib[index].ruleset[i].rowithvar[j],fixmodulelib[index].varname[k]);
							if(pdest!=NULL)
							{
								fixmodulelib[index].ruleset[i].objfixflag=0;
								break;
							}
						}
					if(fixmodulelib[index].ruleset[i].objfixflag==1)
					{
						for(j=0;j<fixmodulelib[index].ruleset[i].lonum;j++)
							strcpy(fixmodulelib[index].ruleset[i].lowithvalue[j],fixmodulelib[index].ruleset[i].lowithvar[j]);
						for(j=0;j<fixmodulelib[index].ruleset[i].ronum;j++)
							strcpy(fixmodulelib[index].ruleset[i].rowithvalue[j],fixmodulelib[index].ruleset[i].rowithvar[j]);
					}
				}	
			index++;
		}
		fixmodulesno=index;
	}
/*************************************************************************/
	ECSB::ModuleLibrary library(library_file);
	std::map<std::string,ECSB::ModuleDefinition> libraryModules = library.getModules();
	
	index=0;
	maxmtypeno=1;
	
	// read modules from library and convert them into the data structure of POptimizer
	for(it = libraryModules.begin(); it != libraryModules.end(); ++it)	
	{
		sstr=(it->second).getName();
		strcpy(modulelib[index].name,sstr.c_str());
		modulelib[index].index=index;
		objstr=(it->second).getObjects();
		modulelib[index].varnum=objstr.size();
		if(objstr.size()==0)
			modulelib[index].fixflag=1;
		else
			modulelib[index].fixflag=0;
		for(i=0;i<objstr.size();i++)
		{
			strcpy(modulelib[index].varname[i],objstr[i].c_str());
			strcpy(modulelib[index].varvaluestr[i],"");
			modulelib[index].varvalueindex[i]=-1;
		}
		modulerules=(it->second).getRules();
		modulelib[index].rulesno=modulerules.size();
		objstr=(it->second).getConstants();
		for(i=0;i<objstr.size();i++)
		{					
			strcpy(modulelib[index].ruleset[i].name,modulerules[i].getName().c_str());
			value1=(float)((it->second).getLowerBounds())[objstr[i]];
			value2=(float)((it->second).getUpperBounds())[objstr[i]];
			if(value1==value2)
				modulelib[index].ruleset[i].kcfixflag=1;
			else 
				modulelib[index].ruleset[i].kcfixflag=0;
			modulelib[index].ruleset[i].kclowerb=value1;
			modulelib[index].ruleset[i].kcupperb=value2;
			tempf=(float)((it->second).getPrecisions())[objstr[i]];
			modulelib[index].ruleset[i].kcprecision=tempf;
			modulelib[index].ruleset[i].kcsigdigit=precision_to_significant_digit(tempf);
			if(modulelib[index].ruleset[i].kcfixflag==1)
				modulelib[index].ruleset[i].kcvalue=value1;
			sstr=((it->second).getScales())[objstr[i]];
			strcpy(modulelib[index].ruleset[i].kcscale,sstr.c_str());
		}
		for(i=0;i<modulerules.size();i++)
		{
			objstr1=modulerules[i].getLhsInside();
			for(j=0;j<objstr1.size();j++)
			{
				strcpy(modulelib[index].ruleset[i].lowithvar[j],"");
				strcpy(modulelib[index].ruleset[i].lowithvalue[j],"");
			}
			objstr2=modulerules[i].getRhsInside();
			for(j=0;j<objstr2.size();j++)
			{
				strcpy(modulelib[index].ruleset[i].rowithvar[j],"");
				strcpy(modulelib[index].ruleset[i].rowithvalue[j],"");
			}
		}
		for(i=0;i<modulerules.size();i++)
		{	
			objstr1=modulerules[i].getLhsInside();
			modulelib[index].ruleset[i].lonum=objstr1.size();
			for(j=0;j<objstr1.size();j++)
				strcpy(modulelib[index].ruleset[i].lowithvar[j],objstr1[j].c_str());
			objstr2=modulerules[i].getRhsInside();
			modulelib[index].ruleset[i].ronum=objstr2.size();
			for(j=0;j<objstr2.size();j++)
				strcpy(modulelib[index].ruleset[i].rowithvar[j],objstr2[j].c_str());
		}
		if(modulelib[index].fixflag==1)
			modulelib[index].ruleset[i].objfixflag=1;
		else
			for(i=0;i<modulelib[index].rulesno;i++)
			{
				modulelib[index].ruleset[i].objfixflag=1;
				for(j=0;j<modulelib[index].ruleset[i].lonum;j++)
					for(k=0;k<modulelib[index].varnum;k++)
					{
						pdest=strstr(modulelib[index].ruleset[i].lowithvar[j],modulelib[index].varname[k]);
						if(pdest!=NULL)
						{
							modulelib[index].ruleset[i].objfixflag=0;
							break;
						}
					}
				for(j=0;j<modulelib[index].ruleset[i].ronum;j++)
					for(k=0;k<modulelib[index].varnum;k++)
					{
						pdest=strstr(modulelib[index].ruleset[i].rowithvar[j],modulelib[index].varname[k]);
						if(pdest!=NULL)
						{
							modulelib[index].ruleset[i].objfixflag=0;
							break;
						}
					}
				if(modulelib[index].ruleset[i].objfixflag==1)
				{
					for(j=0;j<modulelib[index].ruleset[i].lonum;j++)
						strcpy(modulelib[index].ruleset[i].lowithvalue[j],modulelib[index].ruleset[i].lowithvar[j]);
					for(j=0;j<modulelib[index].ruleset[i].ronum;j++)
						strcpy(modulelib[index].ruleset[i].rowithvalue[j],modulelib[index].ruleset[i].rowithvar[j]);
				}
			}
		index++;
	}
	maxbtypeno=index;
	temp=maxtime/interval+1;
	timesteps=(int)temp;
	normalizetime=(float*)malloc(timesteps*sizeof(float));
	popu=(Membrane*)malloc(popsize*sizeof(Membrane));
	diffpopu=(Membrane*)malloc(popsize*sizeof(Membrane));
	fitness=(float*)malloc(popsize*sizeof(float));
	twopopu=(Membrane*)malloc(2*popsize*sizeof(Membrane));
	if(twopopu==NULL)
	{
		printf("not enough space for twopopu!\n");
		getchar();
	}
	twofitness=(float*)malloc(2*popsize*sizeof(float));
	if(twofitness==NULL)
		printf("not enough space for twofitness!\n");

	newpopu=(Membrane*)malloc(popsize*sizeof(Membrane));
	newfitness=(float *)malloc(popsize*sizeof(float));
	crate=(float *)malloc(popsize*sizeof(float));
	mrate=(float *)malloc(popsize*sizeof(float));

	targetobjname=(char **)malloc(targetobjnum*sizeof(char*));
	for(i=0;i<targetobjnum;i++)
		targetobjname[i]=(char*)malloc(50*sizeof(char));
	
	targetobjvalue=(float ***)malloc(initsetnum*sizeof(float**));
	
	for(i=0;i<initsetnum;i++)
		targetobjvalue[i]=(float**)malloc(targetobjnum*sizeof(float*));

	for(i=0;i<initsetnum;i++)
		for(j=0;j<targetobjnum;j++)
			targetobjvalue[i][j]=(float*)malloc(timesteps*sizeof(float));

	modelobjvalue=(float **)malloc(targetobjnum*sizeof(float*));
	for(i=0;i<targetobjnum;i++)
		modelobjvalue[i]=(float*)malloc(timesteps*sizeof(float));
}

GAoptimizer::~GAoptimizer()
{
	int i,j;

	// release signal handlers
	SignalHandler::unset_handler(SIGUSR1);
	SignalHandler::unset_handler(SIGINT);

	free(normalizetime);
	for(i=0;i<targetobjnum;i++)
		free(targetobjname[i]);
	free(targetobjname);
	for(i=0;i<initsetnum;i++)
		for(j=0;j<targetobjnum;j++)
			free(targetobjvalue[i][j]);
	for(i=0;i<initsetnum;i++)
	   free(targetobjvalue[i]);
	free(targetobjvalue);
	for(i=0;i<targetobjnum;i++)
		free(modelobjvalue[i]);
	free(modelobjvalue);

	free(popu);
	free(diffpopu);
	free(fitness);
	free(newpopu);
	free(newfitness);
	free(crate);
	free(mrate);
	free(twopopu);
	free(twofitness);
	
}


void GAoptimizer::execute(void)
{
	char outputfile1[100],outputfile3[100],outputfile4[100],tempoutputfile[100],modulefilename[100];
	time_t t,start,end;
	int i,j,k,nn,count,diffnum;
	float pc,pm,ranf;
	int runtime;
	int same,nopti,sno,optimized[1000],gen,ran;
	FILE *fp;
	char str[600];
	Module tempmodule;
	
	if( debug_mode )
		printf("start Model Optimizer!\n");
	if( show_progress ) {
		printf("initialization\n");
		fflush(stdout);
	}

	for(i=0;i<initsetnum;i++)
		read_target_file(i,multiinputfile[i]);
	
	for(i=0;i<initsetnum;i++)
		read_initialvalue_file(i,multiinitfile[i]);
	
	strcpy(membranetype[0],"A");
	strcpy(membranetype[1],"B");
	strcpy(membranetype[2],"C");
  	 totalruns=1;
	for(runno=0;runno<totalruns;runno++)
	{
		
		//runs=2;
		t=time(NULL);
		time(&start);
		srand(t);
		if( debug_mode )
			printf("\nrun%d start\n",runno);		
		totalevanum=0;
		rng=gsl_rng_alloc(gsl_rng_mt19937);
		if( !seednum )
		 {
			if((fp = fopen("/dev/urandom", "r"))) 
			{
			// set seed from /dev/urandom
				fread(&seednum, sizeof(seednum), 1, fp);
				fclose(fp);
				//printf("use urandom in the system!\n");
			} else{

			// set seed from time
			seednum = (unsigned long)time(NULL);
			}
		}
		gsl_rng_set(rng,seednum);
		sprintf(outputfile3,"evolveprocess_Run%d.txt",runno);
		if((fp=fopen(outputfile3,"wt"))==NULL)
		{
			printf("Cannot open output file !\n");
			exit(1);
		}
		fclose(fp);
		sprintf(tempoutputfile,"evolnum_Run%d.txt",runno);
		if((fp=fopen(tempoutputfile,"wt"))==NULL)
		{
			printf("Cannot open output file !\n");
			exit(1);
		}
		fclose(fp);

		//intialize model population randomly according to the model library	
		for(i=0;i<popsize;i++)
		{
			do
			{
				popu[i]=tempmembrane.create_membrane_indi_from_library();
				//tempmembrane.display_membrane_indi(popu[i]);
				//getchar();
				popu[i]=tempmembrane.simplify_membrane_indi(popu[i]);
				popu[i]=tempmembrane.normalize_parameter__membrane_indi(popu[i]);
				if( debug_mode )				
					printf("i=%d\n",i);
				fitness[i]=tempmembrane.mulinit_cal_fitness_membrane_indi(popu[i],0);
            }while(fitness[i]==initsetnum*(1.0e+10));
		}
		if( debug_mode )
			printf("init end\n");
	
		tempmembrane.bubble_sort_membrane_pop(popsize,popu,fitness);

		gen=0;

		while(gen<maxgeno)
		{
			for(i=0;i<popsize;i++)
			{
				popu[i]=popu[i].simplify_membrane_indi(popu[i]);
				popu[i]=popu[i].normalize_parameter__membrane_indi(popu[i]);
			}
			//do parameter optimization for some top best individuals in the population 
			nopti=0;	
			for(i=popsize-maxop;i<popsize;i++)
			{
                                // print some progress
                                if( show_progress )
                                    printf("parameter optimization %d/%d\n",i-popsize+maxop+1,maxop);

				sno=-1;
				same=0;
				if(i==popsize-maxop)   
				{
					sno=popsize-maxop;
					optimized[0]=popsize-maxop;
					nopti=1;
				}
				else
				{
					for(j=0; j<nopti; j++)
					{
						if(tempmembrane.check_same_membrane(popu[i],popu[optimized[j]])==1) 
						{
							same=1;
							break;
						}
					}
					if(same==0) 
					{
						sno=i;
						optimized[nopti]=i;
						nopti++;
					}
				}
				if(sno!=-1)
				{
					if(popu[sno].check_possibility_parameter_optimization_membrane_indi(popu[sno])==1)
					{
						if( debug_mode )
							printf("Generation: %d   do para-optimization for Indivi No:%d  \n",gen,sno);
						if(paraoptimethodno==1)
						{
							if( debug_mode )
								printf("DE optimization!\n");
							popu[sno]=popu[sno].opposition_DE_parameter_optimization_membrane_indi(popu[sno],fitness,sno,DEpsize,DEmaxgeno);
							if((fp=fopen(tempoutputfile,"at"))==NULL)
							{
								printf("Cannot open output file !\n");
								exit(1);
							}
							fprintf(fp,"\n%d    %d    ",gen,gapeevalnum);
							fclose(fp);
						}

						if(paraoptimethodno==2)
						{
							if( debug_mode )
								printf("GA optimization!\n");
							popu[sno]=popu[sno].GA_parameter_optimization_membrane_indi(popu[sno],fitness,sno,GApsize,GAmaxxo);
							if((fp=fopen(tempoutputfile,"at"))==NULL)
							{
								printf("Cannot open output file !\n");
								exit(1);
							}
							fprintf(fp,"\n%d    %d    ",gen,gapeevalnum);
							fclose(fp);
						}
						if(paraoptimethodno==3)
						{
							if( debug_mode )
								printf("EDA optimization!\n");
							popu[sno]=popu[sno].EDA_parameter_optimization_membrane_indi(popu[sno],fitness,sno,EDApsize,EDAmaxgeno);
							if((fp=fopen(tempoutputfile,"at"))==NULL)
							{
								printf("Cannot open output file !\n");
								exit(1);
							}
							fprintf(fp,"\n%d    %d    ",gen,gapeevalnum);
							fclose(fp);
						}
						
						if(paraoptimethodno==4)
						{
							if( debug_mode )
								printf("CMAES optimization!\n");
							popu[sno]=popu[sno].CMAES_parameter_optimization_membrane_indi(popu[sno],fitness,sno,CMAESmaxgeno);
							if((fp=fopen(tempoutputfile,"at"))==NULL)
							{
								printf("Cannot open output file !\n");
								exit(1);
							}
								fprintf(fp,"\n%d    %d    ",gen,gapeevalnum);
								fclose(fp);
						}
					}
				}
			}
			tempmembrane.bubble_sort_membrane_pop(popsize,popu,fitness);
			
			//generate adaptive mutation and crossover rate for each individual 		
			for(i=0;i<popsize;i++)
			{
				mrate[i]=0.3*(1-fitness[popsize-1]/fitness[i]);
				crate[i]=0.9-mrate[i];
			}
			for(i=0;i<popsize;i++)
			{
				j=tournament_selection(popsize,4,fitness);
				k=tournament_selection(popsize,4,fitness);	
				while((j==k) || (popu[j].modulesno==1 && popu[k].modulesno==1 && tempmembrane.check_same_membrane(popu[j],popu[k])==1))
				{
					j=tournament_selection(popsize,4,fitness);
					k=tournament_selection(popsize,4,fitness);	
				}
				if(fitness[j]<=fitness[k])
				{
					pc=crate[j];
					pm=mrate[j];
				}
				else 
				{
					pc=crate[k];
					pm=mrate[k];
				}
				ranf=randf();
				if(ranf<pc)
				{
					//do crossover1: combine two parents into one offspring  		
					if((popu[j].modulesno==1)&& (popu[k].modulesno==1))
					{
						newpopu[i].type=popu[j].type;
						newpopu[i].modulesno=2;
						newpopu[i].moduleset[0]=popu[j].moduleset[0];
						newpopu[i].moduleset[1]=popu[k].moduleset[0];
						newpopu[i]=newpopu[i].simplify_membrane_indi(newpopu[i]);
					}
					else
					{
						ran=randi(2);
						//do crossover2: one-point crossover 
						if(ran==0)
							newpopu[i]=tempmembrane.one_point_crossover_membrane_indi(popu[j],popu[k],maxmodulesno);
						//do crossover3: module-exchange crossover 					
						else 
							newpopu[i]=tempmembrane.module_exchange_crossover_membrane_indi(popu[j],popu[k]);
					}				
				}
				else if(ranf<(pc+pm))
					{	
						//do mutaion including three operations 					
						ran=randi(2);
					nn=ran==0? j:k;
						newpopu[i]=popu[nn].mutation_membrane_indi(popu[nn]);
					}
					else
					{
						//do coloning  
						ran=randi(2);
					nn=ran==0? j:k;
					newpopu[i]=popu[nn];		
					}
			}
			for(i=0;i<popsize;i++)
				newfitness[i]=newpopu[i].mulinit_cal_fitness_membrane_indi(newpopu[i],0);

			for(i=0;i<popsize;i++)
			{
				twopopu[i]=popu[i];
				twofitness[i]=fitness[i];
				twopopu[i+popsize]=newpopu[i];
				twofitness[i+popsize]=newfitness[i];
			}
			tempmembrane.bubble_sort_membrane_pop(2*popsize,twopopu,twofitness);
			/*population replacement*/
			for(i=0;i<popsize;i++)
			{
				popu[popsize-1-i]=twopopu[2*popsize-1-i];
				fitness[popsize-1-i]=twofitness[2*popsize-1-i];
			}
			tempmembrane.bubble_sort_membrane_pop(popsize,popu,fitness);
			//printf("genetic over\n");
			i=0;
			//delete the redundant models in the population and replace them with random one 
			while(i<(popsize-1))
			{
				if(tempmembrane.check_same_membrane(popu[i],popu[i+1])==1)
				{
					j=1;
					while(tempmembrane.check_same_membrane(popu[i],popu[i+j])==1)
					{
						j++;
						if((i+j)>(popsize-1)) break;
					}
					j--;
					popu[i]=popu[i+j];
					fitness[i]=fitness[i+j];
					for(k=i+1;k<=i+j;k++)
					{
						ran=randi(2);
						if(ran==0)
							popu[k]=popu[k].mutation_membrane_indi(popu[k]);
						else 
						{
							popu[k]=newpopu[k].create_membrane_indi_from_library();
							popu[k]=newpopu[k].simplify_membrane_indi(popu[k]);
						}
						fitness[k]=newpopu[k].mulinit_cal_fitness_membrane_indi(popu[k],0);
					}                  
					i=i+j;
				}
				else i++;
			}
			//delete the invalid models in the population and replace them with random one
			for(i=0;i<popsize;i++)
				if(fitness[i]==initsetnum*(1.0e+10)) 
				{
					do
					{
						popu[i]=popu[i].create_membrane_indi_from_library();
						popu[i]=popu[i].simplify_membrane_indi(popu[i]);
						fitness[i]=popu[i].mulinit_cal_fitness_membrane_indi(popu[i],0);
					}while(fitness[i]==initsetnum*(1.0e+10));
				}
			tempmembrane.bubble_sort_membrane_pop(popsize,popu,fitness);
			for(i=0;i<popsize;i++)
				popu[i]=popu[i].normalize_parameter__membrane_indi(popu[i]);

			diffpopu[0]=popu[0];
			diffnum=1;			
			for(i=0;i<popsize;i++)
			{
				count=0;				
				for(j=0;j<diffnum;j++)
				{
					if(tempmembrane.check_same_membrane(popu[i],diffpopu[j])==1)
						break;
					else count++;
				}
				if(count==diffnum)
				{
					diffpopu[diffnum]=popu[i];
					diffnum++;
				}
			}
			if( debug_mode ) {
				printf("\n\nCurrent Gen:%d  different models:%d \n",gen,diffnum);
				printf("\nfitness=%f \n\n",fitness[popsize-1]);
			}
			if((fp=fopen(outputfile3,"at"))==NULL)
			{
				printf("Cannot open output file !\n");
				exit(1);
			}

			// print to stdout
			if( show_progress ) {		
				printf("%d %.2f ",gen,fitness[popsize-1]);

				for(i=0;i<popu[popsize-1].modulesno;i++)
				{
					tempmodule=popu[popsize-1].moduleset[i];
					if(tempmodule.fixflag==1)
						printf("%s ",tempmodule.name);
					else
					{
						printf("%s(",tempmodule.name);
						for(j=0;j<tempmodule.varnum-1;j++)
							printf("%s=%s,",tempmodule.varname[j],tempmodule.varvaluestr[j]);
						printf("%s=%s) ",tempmodule.varname[j],tempmodule.varvaluestr[j]);
					}
				}
				printf("\n");
				fflush(stdout);
			}	

			fprintf(fp,"\n%d    %f    %d    ",gen,fitness[popsize-1],diffnum);
			for(i=0;i<popu[popsize-1].modulesno;i++)
			{
				tempmodule=popu[popsize-1].moduleset[i];
				if(tempmodule.fixflag==1)
					fprintf(fp,"%s  ",tempmodule.name);
				else
				{
					fprintf(fp,"%s(",tempmodule.name);
					for(j=0;j<tempmodule.varnum-1;j++)
						fprintf(fp,"%s=%s,",tempmodule.varname[j],tempmodule.varvaluestr[j]);
					fprintf(fp,"%s=%s)",tempmodule.varname[j],tempmodule.varvaluestr[j]);
				}
			}
			fclose(fp);
			gen++;
	
		}	 
	
		//choose use model checker to get the simulation output of the final model 
		//if(option_mcheck==1)
		//choose use model checker to the final model to get the simulation output or not 	
			//fitness[popsize-1]=popu[popsize-1].get_modelchecker_output(popu[popsize-1]);
		//choose not to use model checker but use the average of 1000 runs to get the simulation
		// output of the final model

		if( show_progress ) {
			printf("simulate final model\n");
			fflush(stdout);
		}
   
		for(i=0;i<initsetnum;i++)
		{
			sprintf(str,"lastmodulefile%d.xml",i+1);
			strcpy(modulefilename,str);
			popu[popsize-1].write_model_xmlfile_modules_membrane_indi(modulefilename,popu[popsize-1],nonzeroobjnum[i],nonzeroobjname[i],nonzeroobjinit[i]);
		}
			runs=1000;
			fitness[popsize-1]=popu[popsize-1].mulinit_cal_fitness_membrane_indi(popu[popsize-1],1);
			
		sprintf(outputfile1,"bestPsystem_Run%d.txt",runno);
		if( debug_mode ) {		
			printf("write best p system begin\n");
			printf("fitness=%f\n",fitness[popsize-1]);
		}
		popu[popsize-1].write_file_best_P_system(outputfile1,popu[popsize-1],fitness[popsize-1]);
		if( debug_mode )		
			printf("write best p system end\n");
		
		for(i=0;i<initsetnum;i++)
		{
			sprintf(str,"lastmodulefile%d.txt",i+1);
			strcpy(modulefilename,str);
			popu[popsize-1].write_model_xmlfile_modules_membrane_indi(modulefilename,popu[popsize-1],nonzeroobjnum[i],nonzeroobjname[i],nonzeroobjinit[i]);
		}
		time(&end);
		runtime=difftime(end,start);
		//write the statistical results of multiple runs in the text file	
		sprintf(outputfile4,"statistics_1_%d.txt",totalruns);
		if((fp=fopen(outputfile4,"at"))==NULL)
		{
			printf("Cannot open output file !\n");
			exit(1);
		}
		if( debug_mode )
			printf("write statistics start\n");
		fprintf(fp,"Run%d:  %d   %f   %f   %d    %d   ",runno,totalevanum,fitness[popsize-1],besttotalerror,runtime,popu[popsize-1].modulesno);
		for(i=0;i<popu[popsize-1].modulesno;i++)
		{
			tempmodule=popu[popsize-1].moduleset[i];
			if(tempmodule.fixflag==1)
				fprintf(fp,"%s  ",tempmodule.name);
			else
			{
				fprintf(fp,"%s(",tempmodule.name);
				for(j=0;j<tempmodule.varnum-1;j++)
					fprintf(fp,"%s=%s,",tempmodule.varname[j],tempmodule.varvaluestr[j]);
				fprintf(fp,"%s=%s)  ",tempmodule.varname[j],tempmodule.varvaluestr[j]);
			}
		}
		if( debug_mode )
			printf("write statistics end\n");
		strcpy(str,"\n");
		fprintf(fp,"%s",str);
		fclose(fp);
		if( debug_mode )
			printf("\nrun%d end\n",runno);
	}
}

// output information about state of simulation
void GAoptimizer::sigusr1Handler(void)
{
	// output number of function evaluations
	std::cout << totalevanum << "\n";
	fflush(stdout);
}

// gracefully exit upon sigint
void GAoptimizer::sigintHandler(void)
{
	// ignore further INT signals while doing this	
	SignalHandler::ignore_handler(SIGINT);

	//popu[popsize-1].write_file_best_P_system("bestPsystem_Run0.txt",popu[popsize-1],fitness[popsize-1]);

	exit(EXIT_SUCCESS);
}
