// Membrane class implementations
// copyright 2008 Hongqing Cao, hxc AT cs.nott.ac.uk

#if HAVE_CONFIG_H
        #include <config.h>
#endif

#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <math.h>
#include <stdlib.h> 
#include <string.h>

#include "POptimizerCommon.h"
#include "POptimizerRule.h"
#include "POptimizerModule.h"
#include "POptimizerMembrane.h"

using namespace POptimizer;

Membrane::Membrane()
{
}

Membrane::~Membrane()
{
}

//randomly create a valid model individual according to the modules defined in the library  
Membrane Membrane::create_membrane_indi_from_library ()
{
	int i,libmtype;
    	Membrane memb;
	
	memb.type=randi(maxmtypeno);
	memb.modulesno=randi(maxmodulesno)+1;
	for(i=0;i<memb.modulesno;i++)
	{
		libmtype=randi(maxbtypeno);
		memb.moduleset[i]=memb.moduleset[i].instantiate_module_in_library(libmtype,1);
	}
	return(memb);
}

// simplify a model individual by deleting the redundant modules within it
Membrane Membrane::simplify_membrane_indi(Membrane memb)
{
	int i,j,same,remained[2000],nremain;
	int comparei;

	
	nremain=1;	
	for(i=0;i<memb.modulesno;i++)
	{
		same=0;
		if(i==0)   
		{
			remained[0]=0;
			nremain=1;
		}
		else
		{
			for(j=0;j<nremain;j++)
			{
				comparei=tempmodule.check_same_module(memb.moduleset[i],memb.moduleset[remained[j]]);
				if(comparei==1) 
				{
					same=1;
					break;
				}
			}
			if(same==0) 
			{
				remained[nremain]=i;
				nremain++;
			}
		}
	}
	memb.modulesno=nremain;
	for(i=0;i<memb.modulesno;i++)
		memb.moduleset[i]=memb.moduleset[remained[i]];
	return(memb);
}

//check if two model individuals have the same structure and return 1 (same) or 0 (different)
int Membrane::check_same_membrane(Membrane memb1,Membrane memb2)
{
	int i,j;
	int *includeflag1,*includeflag2;

	// if two models have different number of modules, they are different.	
	if(memb1.modulesno!=memb2.modulesno)
		return(0);	
	includeflag1=(int *)malloc(memb1.modulesno*sizeof(int));
	includeflag2=(int *)malloc(memb2.modulesno*sizeof(int));
	for(i=0;i<memb1.modulesno;i++)
	{
		includeflag1[i]=0;
		includeflag2[i]=0;
	}
	//check if any module in model1 is contained in model2
	for(i=0;i<memb1.modulesno;i++)
	{
		for(j=0;j<memb2.modulesno;j++)
			if(tempmodule.check_same_module(memb1.moduleset[i],memb2.moduleset[j])==1)
			{
				includeflag1[i]=1;
				break;
			}
	}
	
	//If there is one module in model1 that is not contained in model2, then model1 and model2 are different. 
	for(i=0;i<memb1.modulesno;i++)
	{	
		if(includeflag1[i]==0)
		{
			free(includeflag1);
			free(includeflag2);
			return(0);
		}
	}

	//check if any module in model2 is contained in model1
	for(i=0;i<memb2.modulesno;i++)
	{
		for(j=0;j<memb1.modulesno;j++)
			if(tempmodule.check_same_module(memb2.moduleset[i],memb1.moduleset[j])==1)
			{
				includeflag2[i]=1;
				break;
			}
	}

	//If there is one module in model2 that is not contained in model1, then model1 and model2 are different. 
	for(i=0;i<memb2.modulesno;i++)
	{	
		if(includeflag2[i]==0)
		{
			free(includeflag1);
			free(includeflag2);
			return(0);
		}
	}
/*If each module in model1 is contained in model2 and each module in model2 is contained in model1, then model1 and model2 are the same
 */
	free(includeflag1);
	free(includeflag2);
	return(1);
}

void Membrane::display_membrane_indi(Membrane memb)
{
	display_nonfixed_membrane_indi(memb);
	if(flagfixlib==1)
		display_fix_library_modules();
}

				
//display the details of a model individual including the module set and rule set 
void Membrane::display_nonfixed_membrane_indi(Membrane memb)
{
	int i,j;

	printf("\n*******non-fixed module set in the model**************\n");
	printf("\n\n\nmembrane_type:%s\n",membranetype[memb.type]);
	printf("number of modules: %d\n\n", memb.modulesno);
	for(i=0;i<memb.modulesno;i++)
	{
		printf("\n********module[%d]:\n",i);
		printf("Module Name in Library: %s\n",memb.moduleset[i].name);
		printf("Index in Library: %d\n",memb.moduleset[i].index);
		printf("Number of rules: %d\n",memb.moduleset[i].rulesno);
		for(j=0;j<memb.moduleset[i].rulesno;j++)
			temprule.display_singlerule(memb.moduleset[i].ruleset[j]);
	}
}

void Membrane::display_fix_library_modules(void)
{
	int i,j;
	
	printf("\n*******fixed module set in the model**************\n");
	printf("number of modules: %d\n\n", fixmodulesno);
	for(i=0;i<fixmodulesno;i++)
	{
		printf("\n********module[%d]:\n",i);
		printf("Module Name in Library: %s\n",fixmodulelib[i].name);
		printf("Index in Library: %d\n",fixmodulelib[i].index);
		printf("Number of rules: %d\n",fixmodulelib[i].rulesno);
		for(j=0;j<fixmodulelib[i].rulesno;j++)
			temprule.display_singlerule(fixmodulelib[i].ruleset[j]);
	}
}



//bubble sort a model population according to the fitness
void Membrane::bubble_sort_membrane_pop(int size, Membrane *pop, float *q)
{
	int i,j;	float tempq;
	Membrane tempm; 

	for(i=0;i<size;i++)
		for(j=size-1;j>i;j--)
			if(q[j-1]<q[j])
			{
				tempq=q[j];
				q[j]=q[j-1];
				q[j-1]=tempq;
				tempm=pop[j];
				pop[j]=pop[j-1];
				pop[j-1]=tempm;
			}
}

//keep the best model in current population to the new population according to the fitness 
void Membrane::elitism_membrane_newpop(int psize,Membrane *newmemb, Membrane *memb,float *newq, float *q)
{
	int i;

	i=randi(psize);
	if(newq[i]>q[psize-1])
 	{
		newmemb[i]=memb[psize-1];
		newq[i]=q[psize-1];
	}
}

//generate a valid crossover offspring by using one-point crossover which swaps a set of modules before and after the crossover point
Membrane Membrane::one_point_crossover_membrane_indi(Membrane membp1, Membrane membp2, int maxb)
{
	int i,i1,i2,d12,d21;
	Membrane membch1,membch2,membch;
	float fit1,fit2;
	Membrane tempch;
	int count;
	int tryflag;
	int validflag,validflag1,validflag2;
	
	count=0;
	tryflag=1;
	//count is the number of trials to get a valid crossover offspring
	while(tryflag==1 && count<30)
	{
		i1=randi(membp1.modulesno)+1;
		i2=randi(membp2.modulesno)+1;
		d12=d21=0;
		if(membp1.modulesno-i1+i2<=maxb) d12=1;
		if(membp2.modulesno-i2+i1<=maxb) d21=1; 
		//if it is possible to get two offspring whose number of modules do not exceed the predefined maximum  		
		if((d12==1)&&(d21==1))
		{
			for(i=0;i<i2;i++)
				membch1.moduleset[i]=membp2.moduleset[i];
			for(i=i2;i<membp1.modulesno-i1+i2;i++)
				membch1.moduleset[i]=membp1.moduleset[i+i1-i2];
			membch1.modulesno=membp1.modulesno+i2-i1;
			membch1.type=membp1.type;
			membch1=simplify_membrane_indi(membch1);
			validflag1=check_validity_membrane_indi(membch1);
			for(i=0;i<i1;i++)
				membch2.moduleset[i]=membp1.moduleset[i];
			for(i=i1;i<membp2.modulesno-i2+i1;i++)
				membch2.moduleset[i]=membp2.moduleset[i+i2-i1];
			membch2.modulesno=membp2.modulesno+i1-i2;
			membch2.type=membp2.type;
			membch2=simplify_membrane_indi(membch2);
			validflag2=check_validity_membrane_indi(membch2);
			if(validflag1==1 && validflag2==0)
			{
				tempch=membch1;
				tryflag=0;
			}
			if(validflag2==1 && validflag1==0)
			{
				tempch=membch2;
				tryflag=0;
			}
			if(validflag1==1 && validflag2==1)
			{
				fit1=mulinit_cal_fitness_membrane_indi(membch1,0);
				fit2=mulinit_cal_fitness_membrane_indi(membch2,0);
				if(fit1<=fit2)
					tempch=membch1;
				else 
					tempch=membch2;
				tryflag=0;
			}
		}
		//if it is possible to get one offspring whose number of modules does not exceed the predefined maximum  		
		else
		{
			if(d12==1)
			{
				for(i=0;i<i2;i++)
					membch.moduleset[i]=membp2.moduleset[i];
				for(i=i2;i<membp1.modulesno-i1+i2;i++)
					membch.moduleset[i]=membp1.moduleset[i+i1-i2];
				membch.modulesno=membp1.modulesno+i2-i1;
			}
			if(d21==1)
			{
				for(i=0;i<i1;i++)
					membch.moduleset[i]=membp1.moduleset[i];
				for(i=i1;i<membp2.modulesno-i2+i1;i++)
					membch.moduleset[i]=membp2.moduleset[i+i2-i1];
				membch.modulesno=membp2.modulesno+i1-i2;
			}
			membch.type=membp1.type;
			membch=simplify_membrane_indi(membch);
			validflag=check_validity_membrane_indi(membch);
			if(validflag==1)
			{
				tempch=membch;
				tryflag=0;
			}
		}
		count++;
	}
	//if the total number of trials reaches 30, return either of the parents as the return model 
	if(count>=30) 
	{
		i=randi(2);
		if(i==0)
			return(membp1);
		if(i==1)
			return(membp2);
	}
	else
		return(tempch);
}

//generate a valid crossover offspring by using  module-exchage crossover which randomly swaps two modules between two parents
Membrane Membrane::module_exchange_crossover_membrane_indi(Membrane membp1, Membrane membp2)
{
	int i1,i2,i,j,commonflag;
	Membrane membch1,membch2;
	Module tempmodule,tempmodule1,tempmodule2;
	float fit1,fit2,tempkc;
	Membrane tempch;
	int count;
	int tryflag;
	int validflag1,validflag2;
	
	count=0;
	tryflag=1;
	while(tryflag==1 && count<30)
	{	
		//choose two modules randomly from two parents
		i1=randi(membp1.modulesno);
		i2=randi(membp2.modulesno);
		membch1=membp1;
		membch2=membp2;
		tempmodule1=membp1.moduleset[i1];
		tempmodule2=membp2.moduleset[i2];
		commonflag=0;
		//If those two modules have some common rules, then swap their kc values.  
		for(i=0;i<tempmodule1.rulesno;i++)
		{
			for(j=0;j<tempmodule2.rulesno;j++)
				if(temprule.check_same_singlerule(tempmodule1.ruleset[i],tempmodule2.ruleset[j])==1)
				{
					commonflag=1;
					tempkc=tempmodule1.ruleset[i].kcvalue;
					tempmodule1.ruleset[i].kcvalue=tempmodule2.ruleset[j].kcvalue;
					tempmodule2.ruleset[j].kcvalue=tempkc;
				}
		}
		//If two modules do not have common rules, then swap the modules and get two offspring.
		if(commonflag==0)
		{
			tempmodule=membch1.moduleset[i1];
			membch1.moduleset[i1]=membch2.moduleset[i2];
			membch2.moduleset[i2]=tempmodule;
			membch1=simplify_membrane_indi(membch1);
			membch2=simplify_membrane_indi(membch2);
		}
		else
		{	membch1.moduleset[i1]=tempmodule1;
			membch2.moduleset[i2]=tempmodule2;
			membch1=simplify_membrane_indi(membch1);
			membch2=simplify_membrane_indi(membch2);
			
		}
		//check the validity of two offspring and choose the valid one as the crossover offspring
		validflag1=check_validity_membrane_indi(membch1);
		validflag2=check_validity_membrane_indi(membch2);
		if(validflag1==1 && validflag2==0)
		{
			tempch=membch1;
			tryflag=0;
		}
		if(validflag2==1 && validflag1==0)
		{
			tempch=membch2;
			tryflag=0;
		}
		//If both offspring are valid, choose the one with better fitness as the crossover offspring
		if(validflag1==1 && validflag2==1)
		{
			fit1=mulinit_cal_fitness_membrane_indi(membch1,0);
			fit2=mulinit_cal_fitness_membrane_indi(membch2,0);
			if(fit1<=fit2)
				tempch=membch1;
			else 
				tempch=membch2;
			tryflag=0;
		}
		count++;
	}
	if(count>=30) 
	{
		i=randi(2);
		if(i==0)
			return(membp1);
		if(i==1)
			return(membp2);
	}
	else
		return(tempch);
}

//generate a mutation offspring by using three mutation operations
Membrane Membrane::mutation_membrane_indi(Membrane parent)
{
	int i,ii,j,jj,kk,sameflag;
	Module tempmodule;
	float tempc,fitness;
	Membrane membp;
	int varkcnum,count;
    	int tryflag;
	int validflag;
	int *varindex,*mvarindex;
	
	count=0;
	tryflag=1;
	while(tryflag==1 && count<30)
	{	
		membp=parent;
		//randomly choose a module from the parent 
		ii=randi(membp.modulesno);
		tempmodule=membp.moduleset[ii];
		varindex=(int *)malloc(tempmodule.varnum*sizeof(int));
		mvarindex=(int *)malloc(tempmodule.varnum*sizeof(int));

		varkcnum=0;	
		for(i=0;i<tempmodule.rulesno;i++)
			if(tempmodule.ruleset[i].kcfixflag==0)
				varkcnum++;
		fitness=mulinit_cal_fitness_membrane_indi(membp,0);
		switch(randi(3))
		{
				//mutation 1: randomly choose a rule from the module and change the kc value by using Gaussian mutation
				case 0:
				if(varkcnum>=1)
				{			
				//randomly choose a rule from the module whose kc value is not fixed					
					do{
						j=randi(tempmodule.rulesno);
					}while(tempmodule.ruleset[j].kcfixflag==1);
					//change the kc value by using Gaussian mutation
					tempc=Gaussian_mutation(tempmodule.ruleset[j].kcvalue,tempmodule.ruleset[j].kcprecision,tempmodule.ruleset[j].kcsigdigit,fitness,tempmodule.ruleset[j].kclowerb,tempmodule.ruleset[j].kcupperb);
					tempmodule.ruleset[j].kcvalue=tempc;

//change the kc values of all other rules within the model which have the same structure as the chosen rule into the mutated value 
					for(jj=0;jj<membp.modulesno;jj++)
						for(kk=0;kk<membp.moduleset[jj].rulesno;kk++)
						{
							if(temprule.check_same_singlerule(membp.moduleset[jj].ruleset[kk],tempmodule.ruleset[j])==1)
								membp.moduleset[jj].ruleset[kk].kcvalue=tempc;
						}
					tryflag=0;
				}
				break;

			//mutation 2: randomly choose a different module type and instantiate it.
			case 1:
				if(maxbtypeno>1)
				{			
					j=tempmodule.index;
					do{
						jj=randi(maxbtypeno);
					}while(jj==j);
					tempmodule=tempmodule.instantiate_module_in_library(jj,1);
					membp.moduleset[ii]=tempmodule;
					membp=simplify_membrane_indi(membp);
					validflag=check_validity_membrane_indi(membp);
					if(validflag==1)
						tryflag=0;
				}
				break;

			//mutation 3: keep the module type unchanged, but change the objects in the module
			case 2:
				if(tempmodule.fixflag==0 && varvaluenum>1)
				{
					for(i=0;i<tempmodule.varnum;i++)
						mvarindex[i]=tempmodule.varvalueindex[i];
					sameflag=1;
					while(sameflag==1)
					{
						for(i=0;i<tempmodule.varnum;i++)
							varindex[i]=randi(varvaluenum);
						for(i=0;i<tempmodule.varnum;i++)
							if(varindex[i]!=mvarindex[i])
								sameflag=0;
					}
					tempmodule=tempmodule.instantiate_module_in_library(tempmodule.index,0);
					membp.moduleset[ii]=tempmodule;
					membp=simplify_membrane_indi(membp);
					validflag=check_validity_membrane_indi(membp);
					if(validflag==1)
						tryflag=0;
				}
				break;
		};
		free(varindex);
		free(mvarindex);
		count++;
	}
	if(count>=30) 
		return(parent);
	else
		return(membp);
}

//do the parameter optimization to the kinetic constants contained in the model by using genetic algorithms (GA)
Membrane Membrane::GA_parameter_optimization_membrane_indi(Membrane memb,float *q,int mnum,int oppopsize,int maxxo)
{

	int xnum,i,ii,j,k,n,M,selectno[1000];
	float *tempp,tempq;
	float **oldparapop,*oldparafit;
	float max1,min1,a,b,max2,min2,weight[1000];
	Membrane tempmemb;
	Rule *totalrules;
	Rule *totaloprules;
	int totalrulesnum;
	int totaloprulesnum;
	int starte,ende;
	
	starte=totalevanum;
	//get the total rule set of the model by combining all the modules		
		totalrulesnum=0;
		for(i=0;i<memb.modulesno;i++)
			totalrulesnum=totalrulesnum+memb.moduleset[i].rulesno;
		totalrules=(Rule *)malloc(totalrulesnum*sizeof(Rule));
		totalrulesnum=combine_modules_into_one_ruleset(memb,totalrulesnum,totalrules);

		//get the total rule set of the model whose kc value is to be optimized
		totaloprulesnum=0;
		for(i=0;i<totalrulesnum;i++)
		{
			if(totalrules[i].kcfixflag==0)
				totaloprulesnum++;
		}
		totaloprules=(Rule *)malloc(totaloprulesnum*sizeof(Rule));

		j=0;		
		for(i=0;i<totalrulesnum;i++)
		{
			if(totalrules[i].kcfixflag==0)
			{
				totaloprules[j]=totalrules[i];
				j++;
			}
		}
		//allocate space for the parameter population and its fitness
		oldparapop=(float **)malloc(oppopsize*sizeof(float *));
		for(i=0;i<oppopsize;i++)
			oldparapop[i]=(float *)malloc(totaloprulesnum*sizeof(float));
		oldparafit=(float *)malloc(oppopsize*sizeof(float));
    		tempp=(float *)malloc(totaloprulesnum*sizeof(float));
		
		//initialize the parameter population randomly
		if( debug_mode ) 
			printf("init parameter popu start!\n");
		init_membrane_indi_para_pop(memb,oppopsize,totaloprules,totaloprulesnum,oldparapop,oldparafit);
		if( debug_mode )
			printf("init parameter popu end!\n");
		
		// bubble sort the parameter according to the fitness		
		bubble_sort_membrane_indi_parapop(oppopsize,totaloprulesnum,oldparapop,oldparafit);
		xnum=0;
		while(xnum<maxxo)
		{
			if( debug_mode )
				printf("xnum=%d\n",xnum);			
			xnum++;
			b=1.5;
			a=-0.5;
			M=4;
			//randomly select M parents from the population and produce M normalized weights randomly			
			for(i=0;i<oppopsize;i++)
				selectno[i]=i;
			max1=1-a*M;
			min1=1-b*M;
			for(i=0;i<M;i++)
			{
				j=randi(oppopsize-i)+i;
				n=selectno[i];
				selectno[i]=selectno[j];
				selectno[j]=n;
				max1+=a;
				min1+=b;
				min2=a<min1?min1:a;
				max2=b>max1?max1:b;
				weight[i]=min2+(max2-min2)*randi(16384)/16383.0;
				max1-=weight[i];
				min1-=weight[i];
			}
			
			for(j=0;j<totaloprulesnum;j++)
			{
				//generate a new parameter value by the nonconvex combination of M parents 
				tempp[j]=0;
				for(i=0;i<M;i++)
					tempp[j]+=weight[i]*oldparapop[selectno[i]][j];
				tempp[j]=ftruncate(tempp[j],totaloprules[j].kcsigdigit);
			//If the new parameter is out of the range, then randomly generate a valid non-zero value to replace it.   
				if(tempp[j]<=totaloprules[j].kclowerb || tempp[j]>totaloprules[j].kcupperb)
				{
					if(strcmp(totaloprules[j].kcscale,"linear")==0)
					{
						do{
							tempp[j]=randfabwithprecision(totaloprules[j].kclowerb,totaloprules[j].kcupperb,totaloprules[j].kcprecision);
						}while(tempp[j]==0);
					}
					if(strcmp(totaloprules[j].kcscale,"logarithmic")==0)
						tempp[j]=randfabwithprecision(totaloprules[j].kclowerb,totaloprules[j].kcupperb,totaloprules[j].kcprecision);
				}
			}
	// keep the kc values of all rules in the model which have the same structures updated with the newly parameter value   			
			tempmemb=memb;
			for(j=0;j<totaloprulesnum;j++)
			{
				for(i=0;i<tempmemb.modulesno;i++)
					for(ii=0;ii<tempmemb.moduleset[i].rulesno;ii++)
					{
						if(temprule.check_same_singlerule(tempmemb.moduleset[i].ruleset[ii],totaloprules[j])==1)
							tempmemb.moduleset[i].ruleset[ii].kcvalue=tempp[j];
					}
			}
			tempq=mulinit_cal_fitness_membrane_indi(tempmemb,0);
			//If the new model is better than the worst one in the population, then replace it with  the new model. 
//			printf("tempq=%f\n",tempq);			
			if(tempq<oldparafit[0])
			{
//				printf("better parameter than the worst one!\n");				
				for(i=0;i<totaloprulesnum;i++)
					oldparapop[0][i]=tempp[i];
				oldparafit[0]=tempq;
				bubble_sort_membrane_indi_parapop(oppopsize,totaloprulesnum,oldparapop,oldparafit);
			}
		} 
		/* When the evolution is over, if the best individual in the population can get a  better fitness than the origin model, then replace all the kc values in the model with the new parameter set. */  
		if(oldparafit[oppopsize-1]<q[mnum])
		{                     
		//printf("\nparameter optimization improved the fitness!\n"); 			
			for(i=0;i<totaloprulesnum;i++)
			{
				for(j=0;j<memb.modulesno;j++)
					for(k=0;k<memb.moduleset[j].rulesno;k++)
					{
						if(temprule.check_same_singlerule(memb.moduleset[j].ruleset[k],totaloprules[i])==1)
							memb.moduleset[j].ruleset[k].kcvalue=oldparapop[oppopsize-1][i];
					}
			}
			q[mnum]=oldparafit[oppopsize-1];
		}
		ende=totalevanum;
		gapeevalnum=ende-starte;
		for(i=0;i<oppopsize;i++)
			free(oldparapop[i]);
	    free(oldparapop);        		
		free(tempp);
		free(oldparafit);
		free(totalrules);
		free(totaloprules);
		return(memb);
	
}

//Initialize the parameter population randomly 
void Membrane::init_membrane_indi_para_pop(Membrane memb, int size, Rule *totalr, int rnum, float **parapop, float *q)
{
	int i,j,ii,jj;
	Membrane tempmemb;

	tempmemb=memb;
	
	// keep the original parameter set as one of the individuals in the population
	for(i=0;i<rnum;i++)
		parapop[0][i]=totalr[i].kcvalue;
	q[0]=mulinit_cal_fitness_membrane_indi(memb,0);
	//randomly generate N-1 parameter set as the individuals within the range and precision
	for(i=1;i<size;i++)
	{
		//printf("\ninit para i=%d \n",i);		
		for(j=0;j<rnum;j++)
		{
	
			if(strcmp(totalr[j].kcscale,"linear")==0)
			{
				do{
	 			parapop[i][j]=randfabwithprecision(totalr[j].kclowerb,totalr[j].kcupperb,totalr[j].kcprecision);
				}while(parapop[i][j]==0);
			}
			if(strcmp(totalr[j].kcscale,"logarithmic")==0)
				parapop[i][j]=randfabwithprecision(totalr[j].kclowerb,totalr[j].kcupperb,totalr[j].kcprecision);

			for(ii=0;ii<memb.modulesno;ii++)
				for(jj=0;jj<memb.moduleset[ii].rulesno;jj++)
				{
					if(temprule.check_same_singlerule(tempmemb.moduleset[ii].ruleset[jj],totalr[j])==1)
						tempmemb.moduleset[ii].ruleset[jj].kcvalue=parapop[i][j];
				}
		}
		q[i]=mulinit_cal_fitness_membrane_indi(tempmemb,0);
	}
}

//bubble sort the parameter population based on the fitness
void Membrane::bubble_sort_membrane_indi_parapop(int size, int rsize, float **parapop, float *q)
{
	int i,j,k;
	float tempq,temppara;
  
	for(i=0;i<size;i++)
		for(j=size-1;j>i;j--)
			if(q[j-1]<q[j])
			{
				tempq=q[j];
				q[j]=q[j-1];
				q[j-1]=tempq;
				for(k=0;k<rsize;k++)
				{
					temppara=parapop[j][k];
					parapop[j][k]=parapop[j-1][k];
					parapop[j-1][k]=temppara;
				}
			}
}

/* write the output text file of the best-evolved model including the methods used for parameter optimization and fitness function and the details of the model*/
 
void Membrane::write_file_best_P_system(char fname[50], Membrane memb, float fit)
{
	int i,j,k;
	FILE *fp;
	char str[600];
	Rule tempr;

	if((fp=fopen(fname,"wt"))==NULL)
	{
		printf("Cannot open output file !\n");
		exit(1);
	}
	for(i=0;i<initsetnum;i++)
		fprintf(fp,"Input target file %d : %s\n\n", i, multiinputfile[i]);
	for(i=0;i<initsetnum;i++)
		fprintf(fp,"Input initial value file %d : %s\n\n", i, multiinitfile[i]);
	
	fprintf(fp,"Input module library file: %s\n\n", library_file);
	if(paraoptimethodno==1)
		strcpy(str,"Parameter optimization method: DE\n");
	if(paraoptimethodno==2)
		strcpy(str,"Parameter optimization method: GA\n");
	if(paraoptimethodno==3)
		strcpy(str,"Parameter optimization method: EDA\n");
	if(paraoptimethodno==4)
		strcpy(str,"Parameter optimization method: CMA-ES\n");
	fprintf(fp,"%s",str);	 	
	if(methodno==1)
		strcpy(str,"fitness calculation method: Equal weighted sum\n");
	if(methodno==2)
		strcpy(str,"fitness calculation method: Random weighted sum\n");
	fprintf(fp,"%s",str);	


	strcpy(str,"\n*******non-fixed module set in the model**************\n");
	fprintf(fp,"%s",str);
	fprintf(fp,"Number of modules: %d\n\n", memb.modulesno);
	for(i=0;i<memb.modulesno;i++)
	{
		fprintf(fp,"\nModule[%d]:\n",i);
		fprintf(fp,"Module Name in Library:  %s\n",memb.moduleset[i].name);
		fprintf(fp,"Index in Library:%d\n",memb.moduleset[i].index);
	        fprintf(fp,"Number of rules: %d\n",memb.moduleset[i].rulesno);
		for(j=0;j<memb.moduleset[i].rulesno;j++)
		{
			tempr=memb.moduleset[i].ruleset[j];
			sprintf(str,"%s:  ",tempr.name);
			for(k=0;k<tempr.lonum-1;k++)
			{
				strcat(str,tempr.lowithvalue[k]);
				strcat(str,"+");
			}			
			strcat(str,tempr.lowithvalue[k]);
			strcat(str,"-->");
			if(tempr.ronum==0)
				strcat(str,"Q");
			else
			{
				for(k=0;k<tempr.ronum-1;k++)
				{
					strcat(str,tempr.rowithvalue[k]);
					strcat(str,"+");
				}			
				strcat(str,tempr.rowithvalue[k]);
			}			
			fprintf(fp,"%s",str);
			fprintf(fp,"\nkc value=%f  scale=%s\n\n",memb.moduleset[i].ruleset[j].kcvalue,memb.moduleset[i].ruleset[j].kcscale);
		}
	}
	if(flagfixlib==1)
	{
		strcpy(str,"\n*******fixed module set in the model**************\n");
		fprintf(fp,"%s",str);
		fprintf(fp,"number of modules: %d\n\n", fixmodulesno);
		for(i=0;i<fixmodulesno;i++)
		{
			fprintf(fp,"\n********module[%d]:\n",i);
			fprintf(fp,"Module Name in Library: %s\n",fixmodulelib[i].name);
			fprintf(fp,"Index in Library: %d\n",fixmodulelib[i].index);
			fprintf(fp,"Number of rules: %d\n",fixmodulelib[i].rulesno);
			for(j=0;j<fixmodulelib[i].rulesno;j++)
			{
				tempr=fixmodulelib[i].ruleset[j];
				sprintf(str,"%s:  ",tempr.name);
				for(k=0;k<tempr.lonum-1;k++)
				{
					strcat(str,tempr.lowithvalue[k]);
					strcat(str,"+");
				}			
				strcat(str,tempr.lowithvalue[k]);
				strcat(str,"-->");
				if(tempr.ronum==0)
					strcat(str,"Q");
				else
				{
					for(k=0;k<tempr.ronum-1;k++)
					{
						strcat(str,tempr.rowithvalue[k]);
						strcat(str,"+");
					}			
					strcat(str,tempr.rowithvalue[k]);
				}			
				fprintf(fp,"%s",str);
				fprintf(fp,"\nkc value=%f  scale=%s\n\n",fixmodulelib[i].ruleset[j].kcvalue,fixmodulelib[i].ruleset[j].kcscale);
			}
		}
	}
	fprintf(fp,"\ntotal fitness= %f  total RMSE=%f  \n\n",fit,besttotalerror);
	fclose(fp);
}

// check the validity of a model, return 1 (valid) or 0 (invalid)
int Membrane::check_validity_membrane_indi(Membrane memb)
{
	int i;



	//check if all the target objects are included in the model. if not, return 0 else return 1    	
	i=check_all_target_objects_membrane_indi(memb);
	
	return(i);
}


//make the same rules in the model has the same kc values 
Membrane Membrane::normalize_parameter__membrane_indi(Membrane memb)
{
	int i,j,k;
	Rule *totalrules;
	int totalrulesnum;

	totalrulesnum=0;
	for(i=0;i<memb.modulesno;i++)
		totalrulesnum=totalrulesnum+memb.moduleset[i].rulesno;

	totalrules=(Rule *)malloc(totalrulesnum*sizeof(Rule));
	totalrulesnum=combine_modules_into_one_ruleset(memb,totalrulesnum,totalrules);	
	for(i=0;i<totalrulesnum;i++)
	{
		for(j=0;j<memb.modulesno;j++)
			for(k=0;k<memb.moduleset[j].rulesno;k++)
			{
				if(temprule.check_same_singlerule(memb.moduleset[j].ruleset[k],totalrules[i])==1)
					memb.moduleset[j].ruleset[k].kcvalue=totalrules[i].kcvalue;
			}
	}
	return(memb);
}

//do the local search for a parameter randomly chosen from the model by using hill-climbing technique
Membrane Membrane::parameter_local_search_membrane_indi(Membrane memb, float *q, int pnum, int maxmunum)
{
	int i,j,jj,ii,kk;
	Membrane tempmemb,bestmemb; 
	float tempc,bestfit,tempfit;

	tempmemb=memb;
	bestmemb=memb;
	bestfit=q[pnum];
	tempfit=q[pnum];
	for(i=0;i<maxmunum;i++)
	{
		//randomly choose a module and a rule in the module with non-fixed value 		
		do{
			ii=randi(tempmemb.modulesno);
			j=randi(tempmemb.moduleset[ii].rulesno);
		}while(tempmemb.moduleset[ii].ruleset[j].kcfixflag==1);
		//generate a new kc value by using Gaussian mutation
		tempc=Gaussian_mutation(tempmemb.moduleset[ii].ruleset[j].kcvalue,tempmemb.moduleset[ii].ruleset[j].kcprecision,tempmemb.moduleset[ii].ruleset[j].kcsigdigit,tempfit,tempmemb.moduleset[ii].ruleset[j].kclowerb,tempmemb.moduleset[ii].ruleset[j].kcupperb);
		tempmemb.moduleset[ii].ruleset[j].kcvalue=tempc;
		for(jj=0;jj<tempmemb.modulesno;jj++)
			for(kk=0;kk<tempmemb.moduleset[jj].rulesno;kk++)
				{
					if(temprule.check_same_singlerule(tempmemb.moduleset[jj].ruleset[kk],tempmemb.moduleset[ii].ruleset[j])==1)
						tempmemb.moduleset[jj].ruleset[kk].kcvalue=tempmemb.moduleset[ii].ruleset[j].kcvalue;
				}
		tempfit=mulinit_cal_fitness_membrane_indi(tempmemb,0);
 /* If the model with new parameter set can get a better fitness than the current best one, then update the best one, otherwise abandon it.*/   
		if(tempfit<bestfit)
		{
			bestfit=tempfit;
			bestmemb=tempmemb;
		}
		else 
		{
			tempmemb=bestmemb;
			tempfit=bestfit;
		}	
	}
	q[pnum]=bestfit;
	return(bestmemb);
}

//randomly sample a parameter set which satisfies normal distribution with assigned mean, variance and covariance matrix 
void Membrane::EDA_ndf_indi_sampling(int paranum,float **covarmatrix,float *means, float *variance, float *sample, Rule *totalr)
{
	int i,j,count;
	float sum;
	float *newmatrix,*tempx;
	float det;
	float **tempmatrix;
	float **triamatrix;

	tempx=(float *)malloc(paranum*sizeof(float));
	newmatrix=(float *)malloc(paranum*sizeof(float));

	tempmatrix=(float **)malloc(paranum*sizeof(float*));
	for(i=0;i<paranum;i++)
		tempmatrix[i]=(float *)malloc(paranum*sizeof(float));

	triamatrix=(float **)malloc(paranum*sizeof(float*));
	for(i=0;i<paranum;i++)
		triamatrix[i]=(float *)malloc(paranum*sizeof(float));

	for(i=0;i<paranum;i++)
		for(j=0;j<paranum;j++)
			tempmatrix[i][j]=covarmatrix[i][j];
	i=new_bchol(covarmatrix,paranum,&det);
	if(i==-2)
	{

		free(tempx);
		free(newmatrix);
		for(j=0;j<paranum;j++)
			free(tempmatrix[j]);
		free(tempmatrix);
		for(j=0;j<paranum;j++)
			free(triamatrix[j]);
		free(triamatrix);
		for(i=0;i<paranum;i++)
		{
			count=0;
			if(strcmp(totalr[i].kcscale,"linear")==0)
			{
				do{
					sample[i]=Gaussian_random_number(means[i],variance[i]);
					sample[i]=ftruncate(sample[i],totalr[i].kcsigdigit);
					count++;
				}while((sample[i]==0 || sample[i]<=totalr[i].kclowerb||sample[i]>totalr[i].kcupperb)&&(count<200));
			}
			if(strcmp(totalr[i].kcscale,"logarithmic")==0)
			{
				do{
					sample[i]=Gaussian_random_number(means[i],variance[i]);
					sample[i]=ftruncate(sample[i],totalr[i].kcsigdigit);
					count++;
				}while((sample[i]<=totalr[i].kclowerb||sample[i]>totalr[i].kcupperb)&&(count<200));
			}
			if(count==200)
			{
				if(sample[i]<=totalr[i].kclowerb || sample[i]>totalr[i].kcupperb)
				{
					if(strcmp(totalr[i].kcscale,"linear")==0)
					{				
						do{
							sample[i]=randfabwithprecision(totalr[i].kclowerb,totalr[i].kcupperb,totalr[i].kcprecision);
						}while(sample[i]==0);
					}
					if(strcmp(totalr[i].kcscale,"logarithmic")==0)
						sample[i]=randfabwithprecision(totalr[i].kclowerb,totalr[i].kcupperb,totalr[i].kcprecision);
				}
			}
		}
		return;
	}
	for(i=0;i<paranum;i++)
		for(j=0;j<paranum;j++)
			triamatrix[i][j]=covarmatrix[i][j];
	for(i=0;i<paranum;i++)
		for(j=0;j<paranum;j++)
			covarmatrix[i][j]=tempmatrix[i][j];
	for(i=0;i<paranum;i++)
	{
		count=0;
		if(strcmp(totalr[i].kcscale,"linear")==0)
		{
			do{
				tempx[i]=Gaussian_random_number(0,1);
				sum=0;
				for(j=0;j<paranum;j++)
					sum=sum+triamatrix[i][j]*tempx[j];
				newmatrix[i]=sum;
				newmatrix[i]=means[i]+newmatrix[i];
				sample[i]=newmatrix[i];
				sample[i]=ftruncate(sample[i],totalr[i].kcsigdigit);
				count++;
			}while((sample[i]==0 || sample[i]<=totalr[i].kclowerb || sample[i]>totalr[i].kcupperb)&&(count<200));
		}
		if(strcmp(totalr[i].kcscale,"logarithmic")==0)
		{
			do{
				tempx[i]=Gaussian_random_number(0,1);
				sum=0;
				for(j=0;j<paranum;j++)
					sum=sum+triamatrix[i][j]*tempx[j];
				newmatrix[i]=sum;
				newmatrix[i]=means[i]+newmatrix[i];
				sample[i]=newmatrix[i];
				sample[i]=ftruncate(sample[i],totalr[i].kcsigdigit);
				count++;
			}while((sample[i]<=totalr[i].kclowerb || sample[i]>totalr[i].kcupperb)&&(count<200));
		}
		if(count==200)
		{
			if(sample[i]<=totalr[i].kclowerb || sample[i]>totalr[i].kcupperb)
			{
				if(strcmp(totalr[i].kcscale,"linear")==0)
				{
					do{
						sample[i]=randfabwithprecision(totalr[i].kclowerb,totalr[i].kcupperb,totalr[i].kcprecision);
					}while(sample[i]==0);
				}
				if(strcmp(totalr[i].kcscale,"logarithmic")==0)
					sample[i]=randfabwithprecision(totalr[i].kclowerb,totalr[i].kcupperb,totalr[i].kcprecision);
			}
		}
	}
	free(tempx);
	free(newmatrix);
	for(j=0;j<paranum;j++)
		free(tempmatrix[j]);
	free(tempmatrix);
	for(j=0;j<paranum;j++)
		free(triamatrix[j]);
	free(triamatrix);
}

/*calculate the mean and variance of the Bayesian distribution for each parameeter in the individual and return 1 (successful) or 0 (fail)*/
int Membrane::EDA_Bayesian_distribution(int paranum,float *indi, float **covarmatrix, float *means, float *bvariance, float *bmeans)
{
	int i,j,k;
	float sum;
	float **dimen_covarmatrix;
	int dimenno;

	for(dimenno=0;dimenno<paranum;dimenno++)
	{		
		dimen_covarmatrix=(float **)malloc((paranum-dimenno)*sizeof(float*));
		for(i=0;i<paranum-dimenno;i++)
			dimen_covarmatrix[i]=(float *)malloc((paranum-dimenno)*sizeof(float));
		for(i=0;i<paranum-dimenno;i++)
			for(j=0;j<paranum-dimenno;j++)
				dimen_covarmatrix[i][j]=covarmatrix[i+dimenno][j+dimenno];
		k=new_brinv(dimen_covarmatrix,paranum-dimenno);
		if(k!=0)
		{ 
			bvariance[dimenno]=1.0/sqrt(dimen_covarmatrix[0][0]);
			sum=0;
			for(j=0;j<paranum-dimenno-1;j++)
				sum=sum+(indi[dimenno+j+1]-means[dimenno+j+1])*dimen_covarmatrix[j+1][0];
			bmeans[dimenno]=(means[dimenno]*dimen_covarmatrix[0][0]-sum)/dimen_covarmatrix[0][0];
			for(i=0;i<paranum-dimenno;i++)
				free(dimen_covarmatrix[i]);
			free(dimen_covarmatrix);
		}
		else
		{
			for(i=0;i<paranum-dimenno;i++)
				free(dimen_covarmatrix[i]);
			free(dimen_covarmatrix);
			return(0);
		}
		
	}
	return(1);
}

//do the parameter optimization to the kinetic constants contained in the model by using estimation of distribution algorithms (EDA)
Membrane Membrane::EDA_parameter_optimization_membrane_indi(Membrane memb,float *q,int mnum, int oppopsize, int maxgeno)
{
	float per_trun,con_trigger_SDR,con_decrease_AVS,con_increase_AVS,per_select_AMS,len_adapt_AMS,con_multiplier_AVS;
	int selepsize,samplepsize,AMSoppopsize;
	Membrane tempmemb;
	Rule *totalrules;
	int totalrulesnum;
	Rule *totaloprules;
	int totaloprulesnum;
	int i,j,k,ii,jj,count,ran;
	float **parapop,**seleparapop,**sampleparapop;
	float *parafit,*seleparafit,*sampleparafit;
	float *ndf_means,*ndf_variance,*bayesian_means,*bayesian_variance,**ndf_covarmatrix;
	float *ndf_means_shift,*previous_ndf_means, *x_improve;
	int gen,totalevalno,n_improve,errorcode;
	float sum;
	float *sampleindi,*improve_SDR;
	float samplefit,bestfitness,tempvalue,tempfit,stdevratio_improve_SDR;
	int starte,ende;

	starte=totalevanum;

	// parameter settings for some control parameters of EDA	
	per_trun=0.3;
	con_trigger_SDR=1.0;
	con_decrease_AVS=0.9;
	con_increase_AVS=1.0/con_decrease_AVS;
	per_select_AMS=per_trun/(2-2*per_trun);
	len_adapt_AMS=2;
	con_multiplier_AVS=1;
	selepsize=(int)(per_trun*oppopsize);
	samplepsize=oppopsize-selepsize;
	totalrulesnum=0;
	for(i=0;i<memb.modulesno;i++)
		totalrulesnum=totalrulesnum+memb.moduleset[i].rulesno;
	totalrules=(Rule *)malloc(totalrulesnum*sizeof(Rule));
	totalrulesnum=combine_modules_into_one_ruleset(memb,totalrulesnum,totalrules);	
	totaloprulesnum=0;
	for(i=0;i<totalrulesnum;i++)
	{
		if(totalrules[i].kcfixflag==0)
			totaloprulesnum++;
	}
	totaloprules=(Rule *)malloc(totaloprulesnum*sizeof(Rule));
	j=0;		
	for(i=0;i<totalrulesnum;i++)
	{
		if(totalrules[i].kcfixflag==0)
		{
  			totaloprules[j]=totalrules[i];
			j++;
		}
	}
	parapop=(float **)malloc(oppopsize*sizeof(float *));
	seleparapop=(float **)malloc(selepsize*sizeof(float *));
	sampleparapop=(float **)malloc(samplepsize*sizeof(float *));
	
	for(i=0;i<oppopsize;i++)
    		parapop[i]=(float *)malloc(totaloprulesnum*sizeof(float));
	
	for(i=0;i<selepsize;i++)
    		seleparapop[i]=(float *)malloc(totaloprulesnum*sizeof(float));
	
	for(i=0;i<samplepsize;i++)
    		sampleparapop[i]=(float *)malloc(totaloprulesnum*sizeof(float));
	
	parafit=(float *)malloc(oppopsize*sizeof(float));
    	seleparafit=(float *)malloc(selepsize*sizeof(float));
	sampleparafit=(float *)malloc(samplepsize*sizeof(float));

	ndf_means=(float *)malloc(totaloprulesnum*sizeof(float));
	ndf_variance=(float *)malloc(totaloprulesnum*sizeof(float));

	bayesian_means=(float *)malloc(totaloprulesnum*sizeof(float));
	bayesian_variance=(float *)malloc(totaloprulesnum*sizeof(float));

	ndf_covarmatrix=(float **)malloc(totaloprulesnum*sizeof(float*));
	for(i=0;i<totaloprulesnum;i++)
		ndf_covarmatrix[i]=(float *)malloc(totaloprulesnum*sizeof(float));
	
	ndf_means_shift=(float *)malloc(totaloprulesnum*sizeof(float));
	previous_ndf_means=(float *)malloc(totaloprulesnum*sizeof(float));
	x_improve=(float *)malloc(totaloprulesnum*sizeof(float));

	sampleindi=(float *)malloc(totaloprulesnum*sizeof(float));
	totalevalno=0;
	gen=0;
	init_membrane_indi_para_pop(memb,oppopsize,totaloprules,totaloprulesnum,parapop,parafit);
	bubble_sort_membrane_indi_parapop(oppopsize,totaloprulesnum,parapop,parafit);
	while(gen<maxgeno)
	{
		//use truncation selection to get the population to do the estimation of the distribution
		for(i=0;i<selepsize;i++)
		{
			for(j=0;j<totaloprulesnum;j++)
				seleparapop[i][j]=parapop[oppopsize-selepsize+i][j];
			seleparafit[i]=parafit[oppopsize-selepsize+i];
		}
		bubble_sort_membrane_indi_parapop(selepsize,totaloprulesnum,seleparapop,seleparafit);
               //estimate the probability distribution of selected individuals based on maximum-likelihood Gaussian models
		for(i=0;i<totaloprulesnum;i++)
		{
			sum=0;
			for(j=0;j<selepsize;j++)
				sum=sum+seleparapop[j][i];
			ndf_means[i]=(sum+0.0)/selepsize;
		}
		for(i=0;i<totaloprulesnum;i++)
			for(j=0;j<totaloprulesnum;j++)
			{
				sum=0;
				for(k=0;k<selepsize;k++)
					sum=sum+(seleparapop[k][i]-ndf_means[i])*(seleparapop[k][j]-ndf_means[j]);
				ndf_covarmatrix[i][j]=sum/(selepsize+0.0);
			}
		for(i=0;i<totaloprulesnum;i++)
			for(j=0;j<totaloprulesnum;j++)
				ndf_covarmatrix[i][j]=con_multiplier_AVS*ndf_covarmatrix[i][j];
		for(i=0;i<totaloprulesnum;i++)
			ndf_variance[i]=sqrt(ndf_covarmatrix[i][i]);

		//Sample the individuals followed by the above distribution by using reject sampling
		for(i=0;i<samplepsize;i++)
		{
			count=0;
			do{
				EDA_ndf_indi_sampling(totaloprulesnum,ndf_covarmatrix,ndf_means,ndf_variance,sampleindi,totaloprules);
				tempmemb=memb;
				for(j=0;j<totaloprulesnum;j++)
				{
					for(ii=0;ii<memb.modulesno;ii++)
						for(jj=0;jj<memb.moduleset[ii].rulesno;jj++)
						{
							if(temprule.check_same_singlerule(tempmemb.moduleset[ii].ruleset[jj],totaloprules[j])==1)
								tempmemb.moduleset[ii].ruleset[jj].kcvalue=sampleindi[j];
						}
				}
				samplefit=mulinit_cal_fitness_membrane_indi(tempmemb,0);
				count++;
			}while(samplefit>parafit[0]&&count<500);
			sampleparafit[i]=samplefit;
			for(j=0;j<totaloprulesnum;j++)
				sampleparapop[i][j]=sampleindi[j];
		}
		bubble_sort_membrane_indi_parapop(samplepsize,totaloprulesnum,sampleparapop,sampleparafit);
 		//change some individuals in the sampling population by Anticipated Mean Shift technique
		if(gen>0)
		{
			for(i=0;i<totaloprulesnum;i++)
				ndf_means_shift[i]=previous_ndf_means[i]-ndf_means[i];
			AMSoppopsize=(int)(per_select_AMS*samplepsize);
			for(i=0;i<AMSoppopsize;i++)
			{
				ran=randi(selepsize);
				for(j=0;j<totaloprulesnum;j++)
				{
					tempvalue=sampleparapop[ran][j]+con_multiplier_AVS*len_adapt_AMS*ndf_means_shift[j];
					tempvalue=ftruncate(tempvalue,totaloprules[j].kcsigdigit);
					if(tempvalue>totaloprules[j].kclowerb && tempvalue<=totaloprules[j].kcupperb)
						sampleparapop[ran][j]=tempvalue;
				}
				tempmemb=memb;
				for(j=0;j<totaloprulesnum;j++)
				{
					for(ii=0;ii<memb.modulesno;ii++)
						for(jj=0;jj<memb.moduleset[ii].rulesno;jj++)
						{
							if(temprule.check_same_singlerule(tempmemb.moduleset[ii].ruleset[jj],totaloprules[j])==1)
								tempmemb.moduleset[ii].ruleset[jj].kcvalue=sampleparapop[ran][j];
						}
				}
				sampleparafit[ran]=mulinit_cal_fitness_membrane_indi(tempmemb,0);
				totalevalno++;
			}
		}
		bubble_sort_membrane_indi_parapop(samplepsize,totaloprulesnum,sampleparapop,sampleparafit);
		bestfitness=seleparafit[selepsize-1];
//Adaptively change the variance according to the improved individuals in the sampling popuation  
		n_improve=0;
		for(i=0;i<totaloprulesnum;i++)
			x_improve[i]=0.0;
		for(i=0;i<samplepsize;i++)
		{
			tempfit=sampleparafit[i];
			if(tempfit<bestfitness)
			{
				n_improve++;
				for(j=0;j<totaloprulesnum;j++)
					x_improve[j]=x_improve[j]+sampleparapop[i][j];
			}
		}
		if(n_improve>0)
		{
			improve_SDR=(float *)malloc(totaloprulesnum*sizeof(float));
			errorcode=EDA_Bayesian_distribution(totaloprulesnum,x_improve,ndf_covarmatrix,ndf_means,bayesian_variance,bayesian_means);
			if(errorcode!=0)
			{
				for(i=0;i<totaloprulesnum;i++)
					improve_SDR[i]=fabs(x_improve[i]-bayesian_means[i])/bayesian_variance[i];
				stdevratio_improve_SDR=calculate_maximum(improve_SDR,totaloprulesnum);
				if(stdevratio_improve_SDR>con_trigger_SDR)
					con_multiplier_AVS=con_multiplier_AVS*con_increase_AVS;
				free(improve_SDR);
			}
		}
		else
			con_multiplier_AVS=con_multiplier_AVS*con_decrease_AVS;
		if(con_multiplier_AVS<1)
			con_multiplier_AVS=1;
		//generate the new ppulation by using the elitist replacement 
		for(i=0;i<samplepsize;i++)
		{
			for(j=0;j<totaloprulesnum;j++)
				parapop[i][j]=sampleparapop[i][j];
			parafit[i]=sampleparafit[i];
		}
		for(i=0;i<selepsize;i++)
		{
			for(j=0;j<totaloprulesnum;j++)
				parapop[samplepsize+i][j]=seleparapop[i][j];
			parafit[samplepsize+i]=seleparafit[i];
		}
		bubble_sort_membrane_indi_parapop(oppopsize,totaloprulesnum,parapop,parafit);
		for(i=0;i<totaloprulesnum;i++)
			previous_ndf_means[i]=ndf_means[i];
		if( debug_mode )		
			printf("pegen=%d  bestfitness:%f  \n",gen,parafit[oppopsize-1]);
		gen++;
		
	}
	/* If finally the best individual in the population can get a  better fitness than the origin model, then replace all the kc values in the model with the new parameter set. */
	if(parafit[oppopsize-1]<q[mnum])
	{                     
		for(i=0;i<totaloprulesnum;i++)
		{
			for(j=0;j<memb.modulesno;j++)
				for(k=0;k<memb.moduleset[j].rulesno;k++)
				{
					if(temprule.check_same_singlerule(memb.moduleset[j].ruleset[k],totaloprules[i])==1)
						memb.moduleset[j].ruleset[k].kcvalue=parapop[oppopsize-1][i];
				}
		}
		q[mnum]=parafit[oppopsize-1];
	}
	ende=totalevanum;
	gapeevalnum=ende-starte;
	for(i=0;i<oppopsize;i++)
		free(parapop[i]);
	free(parapop);
	for(i=0;i<selepsize;i++)
		free(seleparapop[i]);
	free(seleparapop);
	for(i=0;i<samplepsize;i++)
		free(sampleparapop[i]);
	free(sampleparapop);
	free(parafit);
	free(seleparafit);
	free(sampleparafit);
	free(ndf_means);
	free(ndf_variance);
	free(bayesian_means);
	free(bayesian_variance);
	for(i=0;i<totaloprulesnum;i++)
		free(ndf_covarmatrix[i]);
	free(ndf_covarmatrix);
	free(ndf_means_shift);
	free(previous_ndf_means);
	free(x_improve);
	free(sampleindi);
	free(totalrules);
	free(totaloprules);
	return(memb);
}

//get the total rule set of a model by combining all the modules and deleting the redundant rules in the model 
int Membrane::combine_modules_into_one_ruleset(Membrane memb, int totalrulesnum, Rule *totalrules)
{
	int i,j,k;
	int combinerulenum;
		
	i=0;
	while(i<totalrulesnum)
	{
		for(j=0;j<memb.modulesno;j++)
			for(k=0;k<memb.moduleset[j].rulesno;k++)
			{
				totalrules[i]=memb.moduleset[j].ruleset[k];
				i++;
			}
	}
	combinerulenum=temprule.simplify_ruleset(totalrules,totalrulesnum);
	return(combinerulenum);
}

int Membrane::fixlib_combine_modules_into_one_ruleset(Membrane memb, int totalrulesnum, Rule *totalrules)
{
	int combinerulenum;
		
	combinerulenum=temprule.simplify_ruleset(totalrules,totalrulesnum);
	return(combinerulenum);
}


//write xml file for a P system model and return the numer of objects contained in the model
int Membrane::write_model_xmlfile_membrane_indi(char filenames[3][100],Membrane memb,int nzeroobjnum, char nzeroobjname[50][50], float nzeroobjinit[50])
{
	int i,j,k,totalrulesnum,combinerulesnum;
	Rule *totalrules;
	FILE *fp;
	char st[1000];
	float tempkc;
	
	totalrulesnum=0;

	if(flagfixlib==1)
	{
		for(i=0;i<fixmodulesno;i++)
			totalrulesnum=totalrulesnum+fixmodulelib[i].rulesno;
	}
	for(i=0;i<memb.modulesno;i++)
		totalrulesnum=totalrulesnum+memb.moduleset[i].rulesno;
	totalrules=(Rule *)malloc(totalrulesnum*sizeof(Rule));
	
	k=0;
	if(flagfixlib==1)
	{
		for(i=0;i<fixmodulesno;i++)
			for(j=0;j<fixmodulelib[i].rulesno;j++)
			{
				totalrules[k]=fixmodulelib[i].ruleset[j];
				k++;
			}
	}
	for(i=0;i<memb.modulesno;i++)
		for(j=0;j<memb.moduleset[i].rulesno;j++)
		{
			totalrules[k]=memb.moduleset[i].ruleset[j];
			k++;
		}
	combinerulesnum=memb.fixlib_combine_modules_into_one_ruleset(memb,totalrulesnum,totalrules);
	allobjnum=totalrules[0].get_objects_in_ruleset(totalrules,combinerulesnum,nzeroobjnum,nzeroobjname,nzeroobjinit);

        // filenames contain the name of the 3 files for input model in mcss
        // filenames[0] specifies the main structure
        // filenames[1] specifies the lattice structure
        // filenames[2] specifies the list of p systems

        // write to main model file
	if((fp=fopen(filenames[0],"wt"))==NULL)
	{
		printf("Cannot open file %s!\n",filenames[0]);
		exit(1);
	}
	strcpy(st,"<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n");
	fprintf(fp,"%s",st);
	sprintf(st,"<LPPsystem name=\"%s\">\n","ModelToEvaluate");
	fprintf(fp,"%s",st);
	strcpy(st,"\t<listOfPSystems>\n");
	fprintf(fp,"%s",st);
	sprintf(st,"\t\t<PSystem name=\"%s\" file=\"%s\"/>\n","psystem1",filenames[2]);
	fprintf(fp,"%s",st);
        strcpy(st, "\t</listOfPSystems>\n\n");
	fprintf(fp,"%s",st);
        sprintf(st,"\t<lattice name=\"%s\" file=\"%s\"/>\n\n","onePoint",filenames[1]);
        fprintf(fp,"%s",st);
        strcpy(st, "\t<spatialDistribution name=\"distribution\" distribution=\"fixed\">\n");
	fprintf(fp,"%s",st);
	sprintf(st, "\t\t<position pSystem=\"%s\">\n","psystem1");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t\t<Xcoordinate value=\"0\"/>\n");
	fprintf(fp,"%s",st);
	strcpy(st, "\t\t\t<Ycoordinate value=\"0\"/>\n");
	fprintf(fp,"%s",st);
	strcpy(st, "\t\t</position>\n");
	fprintf(fp,"%s",st);
	strcpy(st, "\t</spatialDistribution>\n\n");
	fprintf(fp,"%s",st);
	strcpy(st, "</LPPsystem>");
	fprintf(fp,"%s",st);
	fclose(fp);

        // write to lattice specification file
	if((fp=fopen(filenames[1],"wt"))==NULL)
	{
		printf("Cannot open file %s!\n",filenames[1]);
		exit(1);
	}
        strcpy(st,"<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n");
	fprintf(fp,"%s",st);
        sprintf(st,"<lattice name=\"%s\" dimension=\"2\" xmin=\"0\" xmax=\"0\" ymin=\"0\" ymax=\"0\">\n","onePoint");
        fprintf(fp,"%s",st);
        strcpy(st, "\t<basis>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t\t<vector x=\"a\" y=\"0\"/>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t\t<vector x=\"0\" y=\"a\"/>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t</basis>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t<vertices>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t\t<vector x=\"a/2\" y=\"a/2\"/>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t\t<vector x=\"-a/2\" y=\"a/2\"/>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t\t<vector x=\"-a/2\" y=\"-a/2\"/>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t\t<vector x=\"-a/2\" y=\"-a/2\"/>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t</vertices>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t<neighbours>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t\t<vector x=\"1\" y=\"0\"/>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t\t<vector x=\"-1\" y=\"0\"/>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t\t<vector x=\"0\" y=\"1\"/>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t\t<vector x=\"0\" y=\"-1\"/>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "\t</neighbours>\n");
	fprintf(fp,"%s",st);
        strcpy(st, "</lattice>\n");
	fprintf(fp,"%s",st);
        fclose(fp);

        // write p sytem specification
        if((fp=fopen(filenames[2],"wt"))==NULL)
	{
		printf("Cannot open file %s!\n",filenames[2]);
		exit(1);
	}
        sprintf(st,"\t\t<PSystem name=\"%s\">\n","psystem1");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t\t<alphabetOfObjects>\n");
	fprintf(fp,"%s",st);
	for(i=0;i<allobjnum;i++)
	{
		sprintf(st,"\t\t\t\t<object name=\"%s\"/>\n",allobjname[i]);
		fprintf(fp,"%s",st);
	}
	strcpy(st,"\t\t\t</alphabetOfObjects>\n\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t\t<compartmentLabels>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t\t\t<compartmentLabel label=\"label\"/>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t\t</compartmentLabels>\n\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t\t<membraneStructure>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t\t\t<compartment name=\"label\" label=\"label\"/>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t\t</membraneStructure>\n\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t\t<initialMultisets>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t\t\t<initialMultiset label=\"label\">\n");
	fprintf(fp,"%s",st);
	for(i=0;i<allobjnum;i++)
	{ 
		sprintf(st,"\t\t\t\t\t<object name=\"%s\" multiplicity=\"%1.0f\"/>\n",allobjname[i],allobjinit[i]);
		fprintf(fp,"%s",st);
	}
	strcpy(st,"\t\t\t\t</initialMultiset>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t\t</initialMultisets>\n\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t\t<ruleSets>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t\t\t<ruleSet label=\"label\">\n\n");
	fprintf(fp,"%s",st);
	for(i=0;i<combinerulesnum;i++)
	{
		tempkc=totalrules[i].kcvalue;
		if(strcmp(totalrules[i].kcscale,"linear")==0)
			tempkc=totalrules[i].kcvalue;
		if(strcmp(totalrules[i].kcscale,"logarithmic")==0)
			tempkc=pow((float)10.0,totalrules[i].kcvalue);
		
		sprintf(st,"\t\t\t\t\t<rule name=\"r%d\" constant=\"c%d\" value=\"%.10f\">\n",i+1,i+1,tempkc);
		fprintf(fp,"%s",st);
	    strcpy(st,"\t\t\t\t\t\t<lhs>\n");
		fprintf(fp,"%s",st);
		strcpy(st,"\t\t\t\t\t\t\t<listOfInsideObjects label=\"label\">\n");
		fprintf(fp,"%s",st);
		for(j=0;j<totalrules[i].lonum;j++)
		{
			sprintf(st,"\t\t\t\t\t\t\t\t<object name=\"%s\"/>\n",totalrules[i].lowithvalue[j]);
			fprintf(fp,"%s",st);
		}
		strcpy(st, "\t\t\t\t\t\t\t</listOfInsideObjects>\n");
		fprintf(fp,"%s",st);
		strcpy(st, "\t\t\t\t\t\t</lhs>\n");
		fprintf(fp,"%s",st);
		strcpy(st, "\t\t\t\t\t\t<rhs>\n");
		fprintf(fp,"%s",st);
		strcpy(st,"\t\t\t\t\t\t\t<listOfInsideObjects label=\"label\">\n");
		fprintf(fp,"%s",st);
		for(j=0;j<totalrules[i].ronum;j++)
		{
			sprintf(st,"\t\t\t\t\t\t\t\t<object name=\"%s\"/>\n",totalrules[i].rowithvalue[j]);
			fprintf(fp,"%s",st);
		}
		strcpy(st, "\t\t\t\t\t\t\t</listOfInsideObjects>\n");
		fprintf(fp,"%s",st);
            	strcpy(st, "\t\t\t\t\t\t</rhs>\n");
		fprintf(fp,"%s",st);
            	strcpy(st, "\t\t\t</rule>\n\n");
		fprintf(fp,"%s",st);
	}
	strcpy(st, "\t\t\t\t</ruleSet>\n\n");
	fprintf(fp,"%s",st);
	strcpy(st, "\t\t\t</ruleSets>\n\n");
	fprintf(fp,"%s",st);
	strcpy(st, "\t\t</PSystem>\n\n");
	fprintf(fp,"%s",st);
        fclose(fp);
	
	free(totalrules);
	return(allobjnum);
}

//write xml file for a P system model and return the numer of objects contained in the model
void Membrane::write_model_xmlfile_modules_membrane_indi(char* filename,Membrane memb,int nzeroobjnum, char nzeroobjname[50][50], float nzeroobjinit[50])
{
	int i,j,k,totalrulesnum,combinerulesnum;
	Rule *totalrules;
	FILE *fp;
	char st[1000];
	float tempkc;
	
	totalrulesnum=0;

	if(flagfixlib==1)
	{
		for(i=0;i<fixmodulesno;i++)
			totalrulesnum=totalrulesnum+fixmodulelib[i].rulesno;
	}
	for(i=0;i<memb.modulesno;i++)
		totalrulesnum=totalrulesnum+memb.moduleset[i].rulesno;
	totalrules=(Rule *)malloc(totalrulesnum*sizeof(Rule));
	
	k=0;
	if(flagfixlib==1)
	{
		for(i=0;i<fixmodulesno;i++)
			for(j=0;j<fixmodulelib[i].rulesno;j++)
			{
				totalrules[k]=fixmodulelib[i].ruleset[j];
				k++;
			}
	}
	for(i=0;i<memb.modulesno;i++)
		for(j=0;j<memb.moduleset[i].rulesno;j++)
		{
			totalrules[k]=memb.moduleset[i].ruleset[j];
			k++;
		}
	combinerulesnum=memb.fixlib_combine_modules_into_one_ruleset(memb,totalrulesnum,totalrules);
	allobjnum=totalrules[0].get_objects_in_ruleset(totalrules,combinerulesnum,nzeroobjnum,nzeroobjname,nzeroobjinit);
	

	if((fp=fopen(filename,"wt"))==NULL)
	{
		printf("Cannot open output file !\n");
		exit(1);
	}
	sprintf(st,"<PSystem name=\"%s\">\n\n",filename);
	fprintf(fp,"%s",st);
	strcpy(st,"\t<alphabetOfObjects>\n");
	fprintf(fp,"%s",st);
	for(i=0;i<allobjnum;i++)
	{
		sprintf(st,"\t\t<object name=\"%s\"/>\n",allobjname[i]);
		fprintf(fp,"%s",st);
	}
	strcpy(st,"\t</alphabetOfObjects>\n\n");
	fprintf(fp,"%s",st);

	strcpy(st,"\t<compartmentLabels>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t<compartmentLabel label=\"bact\"/>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t</compartmentLabels>\n\n");
	fprintf(fp,"%s",st);
	
	strcpy(st,"\t<membraneStructure>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t<compartment name=\"bact\" label=\"bact\"/>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t</membraneStructure>\n\n");
	fprintf(fp,"%s",st);
	
	strcpy(st,"\t<initialMultisets>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t<initialMultiset label=\"bact\">\n");
	fprintf(fp,"%s",st);
	for(i=0;i<allobjnum;i++)
	{ 
		if(allobjinit[i]!=0)
		{
			sprintf(st,"\t\t\t<object name=\"%s\" multiplicity=\"%1.0f\"/>\n",allobjname[i],allobjinit[i]);
			fprintf(fp,"%s",st);
		}
	}
	strcpy(st,"\t\t</initialMultiset>\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t</initialMultisets>\n\n");
	fprintf(fp,"%s",st);

	strcpy(st,"\t<ruleSets>\n\n");
	fprintf(fp,"%s",st);
	strcpy(st,"\t\t<ruleSet label=\"bact\">\n\n");
	fprintf(fp,"%s",st);

	for(i=0;i<memb.modulesno;i++)
	{
		sprintf(st,"\t\t\t<module name=\"%s\" \n",memb.moduleset[i].name);
		fprintf(fp,"%s",st);
		strcpy(st,"\t\t\t\t\t objects=\"");
		fprintf(fp,"%s",st);
		for(j=0;j<memb.moduleset[i].varnum-1;j++)
			fprintf(fp,"%s,",memb.moduleset[i].varvaluestr[j]);
		fprintf(fp,"%s\"\n",memb.moduleset[i].varvaluestr[j]);
	
		strcpy(st,"\t\t\t\t\t constants=\"");
		fprintf(fp,"%s",st);
		for(j=0;j<memb.moduleset[i].rulesno-1;j++)
		{
			if(strcmp(memb.moduleset[i].ruleset[j].kcscale,"linear")==0)
				tempkc=memb.moduleset[i].ruleset[j].kcvalue;
			if(strcmp(memb.moduleset[i].ruleset[j].kcscale,"logarithmic")==0)
				tempkc=pow((float)10.0,memb.moduleset[i].ruleset[j].kcvalue);
			fprintf(fp,"%.10f,",tempkc);
		}
		if(strcmp(memb.moduleset[i].ruleset[j].kcscale,"linear")==0)
				tempkc=memb.moduleset[i].ruleset[j].kcvalue;
		if(strcmp(memb.moduleset[i].ruleset[j].kcscale,"logarithmic")==0)
			tempkc=pow((float)10.0,memb.moduleset[i].ruleset[j].kcvalue);
		fprintf(fp,"%.10f\"\n",tempkc);
		strcpy(st,"\t\t\t\t\t labels=\"bact\"\n");
		fprintf(fp,"%s",st);

		sprintf(st,"\t\t\t\t\tfile=\"%s\" />\n",library_file);
		fprintf(fp,"%s",st);
	}
	if(flagfixlib==1)
	{
		for(i=0;i<fixmodulesno;i++)
		{
			sprintf(st,"\t\t\t<module name=\"%s\" \n",fixmodulelib[i].name);
			fprintf(fp,"%s",st);
			strcpy(st,"\t\t\t\t\t objects=\"\"\n");
			fprintf(fp,"%s",st);
			strcpy(st,"\t\t\t\t\t constants=\"");
			fprintf(fp,"%s",st);
			for(j=0;j<fixmodulelib[i].rulesno-1;j++)
			{
				if(strcmp(fixmodulelib[i].ruleset[j].kcscale,"linear")==0)
					tempkc=fixmodulelib[i].ruleset[j].kcvalue;
				if(strcmp(fixmodulelib[i].ruleset[j].kcscale,"logarithmic")==0)
					tempkc=pow((float)10.0,fixmodulelib[i].ruleset[j].kcvalue);
				fprintf(fp,"%.10f,",tempkc);
			}
			if(strcmp(fixmodulelib[i].ruleset[j].kcscale,"linear")==0)
				tempkc=fixmodulelib[i].ruleset[j].kcvalue;
			if(strcmp(fixmodulelib[i].ruleset[j].kcscale,"logarithmic")==0)
				tempkc=pow((float)10.0,fixmodulelib[i].ruleset[j].kcvalue);
			fprintf(fp,"%.10f\"\n",tempkc);
			strcpy(st,"\t\t\t\t\t labels=\"bact\"\n");
			fprintf(fp,"%s",st);
			sprintf(st,"\t\t\t\t\tfile=\"%s\" />\n",fixlibrary_file);
			fprintf(fp,"%s",st);
		}
	}
	strcpy(st, "\t\t</ruleSet>\n\n");
	fprintf(fp,"%s",st);
	strcpy(st, "\t</ruleSets>\n\n");
	fprintf(fp,"%s",st);
	strcpy(st, "</PSystem>\n\n");
	fprintf(fp,"%s",st);
	fclose(fp);
	free(totalrules);
}

//get the simulation results by using the model checker and return the fitness value
/*float Membrane::get_modelchecker_output(Membrane memb)
{
	char commonname[100],xmlfn[100];
	int i,j;
	char command[100];
	float sum1,sum2,sum,fitnessvalue;	

	strcpy(xmlfn,"bestmodel.xml");	
	objectsrealnum=write_model_xmlfile_membrane_indi(xmlfn,memb);
	strcpy(commonname,"bestmodel");
	write_parameter_xmlfile_for_modelcheker(totalp,commonname);
	printf("write parameter file for model checker finished\n");
	//call model checker for each xml parameter files of protein and rna	
	for(i=0;i<totalp;i++)
	{
		sprintf(command,"./PModelChecker protein%d.xml",i+1);
		system(command);
	}
	for(i=0;i<totalp;i++)
	{
		sprintf(command,"./PModelChecker rna%d.xml",i+1);
		system(command);
	}
	//write the combined simulation output of all protein and rna from multiple output files of model checker in one text file 
	write_file_combine_modelchecker_output();
	//calculate the fitness value and RMSE value of the model by using model checker	
	sum1=0;
	for(i=0;i<timesteps;i++)
		for(j=0;j<totalp;j++)
			sum1=sum1+(averageprotein[i][j]-targetprotein[i][j])*(averageprotein[i][j]-targetprotein[i][j]);
	sum2=0;
	for(i=0;i<timesteps;i++)
		for(j=0;j<totalp;j++)
			sum2=sum2+(averagerna[i][j]-targetrna[i][j])*(averagerna[i][j]-targetrna[i][j]);
	sum=sum1+sum2;
	sum=sqrt(sum/timesteps);
	besttotalerror=sum;
	fitnessvalue=cal_fitness_function(methodno,timesteps,totalp,targetprotein,targetrna,averageprotein,averagerna);
	return(fitnessvalue);
}
*/

float Membrane::mulinit_cal_fitness_membrane_indi(Membrane memb,int outputflag)
{
	int i;
	float finalfitness;

	besttotalerror=0;
	finalfitness=0;
	totalevanum++;
	for(i=0;i<initsetnum;i++)
		finalfitness=finalfitness+cal_fitness_membrane_indi(memb,outputflag,i,nonzeroobjnum[i],nonzeroobjname[i],nonzeroobjinit[i],targetobjvalue[i]);
	if(flagMDL==1)
		finalfitness=finalfitness*(memb.modulesno/(maxmodulesno+0.0));
	if(outputflag==1)
	{
		for(i=0;i<initsetnum;i++)
			besttotalerror=besttotalerror+multiRMSE[i];
	}
	return(finalfitness);
}

//calculate the fitness of a model as the returned value of the function. If outputflag=1, write the simulation results in a text file. 
float Membrane::cal_fitness_membrane_indi(Membrane memb,int outputflag, int targetfileno, int nzeroobjnum, char nzeroobjname[50][50], float nzeroobjinit[50], float **targetvalue)
{

	FILE *fp;
	char str[200],title[600];
    int i,j;
	float fitnessvalue;
	float sum;
	char outputfile2[100];
	char parafn[100],h5fn[100];
	char command[100],st[1000];
        char xmlfiles[3][100];
	int k;
	float time1,tempf;
	char outputdatafile[50];
	int errorcode;
	time_t start,end;
	int runtime;
	
	time(&start);	

	// If the model is invalid, return 1.0e+10 as the fitness
	if(check_validity_membrane_indi(memb)==0)
	{
		//printf("invalid objects!\n");
		return(1.0e+10);
	}
	
	time1=0;
	i=0;
	while(time1<maxtime)
	{
		normalizetime[i]=time1;
		time1=time1+interval;
		i++;
	}
	normalizetime[i]=maxtime;
	// If choose to use the simulator

		//write the xml files for the model
		strcpy(xmlfiles[0],"XML_main.xml");
                strcpy(xmlfiles[1],"XML_lattice.xml");
                strcpy(xmlfiles[2],"XML_psystem1.xml");
		i=memb.write_model_xmlfile_membrane_indi(xmlfiles,memb,nzeroobjnum,nzeroobjname,nzeroobjinit);
		strcpy(parafn,"simulator.params");
		strcpy(h5fn,"simulation.h5");
		//write xml file of the paramters required by the simulator			
		write_parameter_xmlfile_for_simulator(parafn,xmlfiles[0],h5fn);
		//printf("write simulator para file end!\n");
		//call simulator by system command to get the simualtion result of multiple run
		//printf("mcss start! \n");		
		sprintf(command,"mcss %s",parafn);
		errorcode=system(command);
		
//		errorcode=call_simulator(20,parafn);

		if(errorcode!=0)
			{
			//printf("Error with mcss!\n");
			exit(1);
			//return(1.0e+10);
			}

		// get thename list of the target output objects		
		strcpy(st,"");
		for(i=0;i<targetobjnum-1;i++)
		{
			strcat(st,targetobjname[i]);
			strcat(st,",");
		}
		strcat(st,targetobjname[i]);
		// write the output data file from the postprocess of the simulator 		
		strcpy(outputdatafile, "outputdata.txt");
		sprintf(command,"mcss-postprocess -l -s %s -o %s  %s",st,outputdatafile,h5fn);
		errorcode=system(command);
		
		if(errorcode!=0)
			return(1.0e+10);
		if((fp=fopen(outputdatafile,"rt"))==NULL)
		{
			printf("Cannot open output data file !\n");
			exit(1);
		}
		k=fscanf(fp,"%s",st);
		for(i=0;i<targetobjnum;i++)
			{
				k=fscanf(fp,"%s",st);
				k=fscanf(fp,"%s",st);
				k=fscanf(fp,"%s",st);
			}
		for(j=0;j<timesteps;j++)
			{
				k=fscanf(fp,"%f",&tempf);
				for(i=0;i<targetobjnum;i++)
				{
					k=fscanf(fp,"%f",&modelobjvalue[i][j]);
					k=fscanf(fp,"%f",&tempf);
					k=fscanf(fp,"%f",&tempf);
				}
			};
	fclose(fp);	
	//printf("end of read data from mcss!\n");
	// If output flag=1, write the simulation output and the target output of protein and rna in a text file
	if(outputflag==1)
	{
		if( debug_mode )
			printf("get results!\n");
		sprintf(outputfile2,"bestsimulation_Run%d_initfile%d.txt",runno,targetfileno);
		if((fp=fopen(outputfile2,"wt"))==NULL)
		{
			printf("Cannot open output file !\n");
			exit(1);
		}
		strcpy(title,"step  Time    ");
		for(i=0;i<targetobjnum;i++)
		{
			sprintf(str,"model_%s",targetobjname[i]);
			strcat(title,str); 
			strcat(title,"        ");
		}
		for(i=0;i<targetobjnum;i++)
		{
			sprintf(str,"target_%s",targetobjname[i]);
			strcat(title,str); 
			strcat(title,"        ");
		}
		fprintf(fp,"%s\n",title);
		for(i=0;i<timesteps;i++)
		{
			fprintf(fp,"%d   ",i);
			fprintf(fp,"%d    ",(int)normalizetime[i]);
			for(j=0;j<targetobjnum;j++)
				fprintf(fp,"%f   ",modelobjvalue[j][i]);
			for(j=0;j<targetobjnum;j++)
				fprintf(fp,"%f   ",targetvalue[j][i]);
			strcpy(str,"\n");
			fprintf(fp,"%s",str);
		}
		fclose(fp);
	}
	//calculate the total RMSE error of the model
	if(outputflag==1)
	{
		sum=0;
		for(i=0;i<targetobjnum;i++)
			for(j=0;j<timesteps;j++)
				sum=sum+(modelobjvalue[i][j]-targetvalue[i][j])*(modelobjvalue[i][j]-targetvalue[i][j]);
		sum=sqrt(sum/(targetobjnum*timesteps));
		multiRMSE[targetfileno]=sum;
	}
	//printf("cal value begin\n");
	fitnessvalue=cal_fitness_function(methodno,timesteps,targetobjnum,targetvalue,modelobjvalue);
	//printf("cal value end! fitness=%f\n",fitnessvalue);
	time(&end);		
	runtime=difftime(end,start);
//if((fp2=fopen(testfile2,"at"))==NULL)
//	{
//		printf("Cannot open output file !\n");
//		exit(1);
	//}
//printf("time: %d  fitness: %f  \n ",runtime,fitnessvalue);	
//fprintf(fp2,"\ntime: %d  fitness: %f  \n\n ",runtime,fitnessvalue);
//fclose(fp2);

return(fitnessvalue);
}

//get the simulation results bu=y using the model checker and return the fitness value
/*float Membrane::get_modelchecker_output(Membrane memb)
{
	char commonname[100],xmlfn[100];
	int i,j;
	char command[100];
	float sum1,sum2,sum,fitnessvalue;	

	strcpy(xmlfn,"bestmodel.xml");	
	objectsrealnum=write_model_xmlfile_membrane_indi(xmlfn,memb);
	strcpy(commonname,"bestmodel");
	write_parameter_xmlfile_for_modelcheker(totalp,commonname);
	printf("write parameter file for model checker finished\n");
	//call model checker for each xml parameter files of protein and rna	
	for(i=0;i<totalp;i++)
	{
		sprintf(command,"./PModelChecker protein%d.xml",i+1);
		system(command);
	}
	for(i=0;i<totalp;i++)
	{
		sprintf(command,"./PModelChecker rna%d.xml",i+1);
		system(command);
	}
	//write the combined simulation output of all protein and rna from multiple output files of model checker in one text file 
	write_file_combine_modelchecker_output();
	//calculate the fitness value and RMSE value of the model by using model checker	
	sum1=0;
	for(i=0;i<timesteps;i++)
		for(j=0;j<totalp;j++)
			sum1=sum1+(averageprotein[i][j]-targetprotein[i][j])*(averageprotein[i][j]-targetprotein[i][j]);
	sum2=0;
	for(i=0;i<timesteps;i++)
		for(j=0;j<totalp;j++)
			sum2=sum2+(averagerna[i][j]-targetrna[i][j])*(averagerna[i][j]-targetrna[i][j]);
	sum=sum1+sum2;
	sum=sqrt(sum/timesteps);
	besttotalerror=sum;
	fitnessvalue=cal_fitness_function(methodno,timesteps,totalp,targetprotein,targetrna,averageprotein,averagerna);
	return(fitnessvalue);
}
*/

//calculate the number of proteins contained in the model
int Membrane::check_all_target_objects_membrane_indi(Membrane memb)
{
	int i,j,k;
	Rule *totalrules;
	int totalrulesnum;
	int *targetobjflag;

	targetobjflag=(int *)malloc(targetobjnum*sizeof(int));
	for(i=0;i<targetobjnum;i++)
		targetobjflag[i]=0;

	totalrulesnum=0;
 
	if(flagfixlib==1)
	{
		for(i=0;i<fixmodulesno;i++)
			totalrulesnum=totalrulesnum+fixmodulelib[i].rulesno;
	}
	for(i=0;i<memb.modulesno;i++)
		totalrulesnum=totalrulesnum+memb.moduleset[i].rulesno;

	totalrules=(Rule *)malloc(totalrulesnum*sizeof(Rule));
	
	k=0;
	if(flagfixlib==1)
	{
		for(i=0;i<fixmodulesno;i++)
			for(j=0;j<fixmodulelib[i].rulesno;j++)
			{
				totalrules[k]=fixmodulelib[i].ruleset[j];
				k++;
			}
	}
	for(i=0;i<memb.modulesno;i++)
		for(j=0;j<memb.moduleset[i].rulesno;j++)
		{
			totalrules[k]=memb.moduleset[i].ruleset[j];
			k++;
		}
	totalrulesnum=fixlib_combine_modules_into_one_ruleset(memb,totalrulesnum,totalrules);	
	for(i=0;i<totalrulesnum;i++)
	{
		for(j=0;j<totalrules[i].lonum;j++)
			for(k=0;k<targetobjnum;k++)
				if(strcmp(totalrules[i].lowithvalue[j],targetobjname[k])==0)
					targetobjflag[k]=1;
		for(j=0;j<totalrules[i].ronum;j++)
			for(k=0;k<targetobjnum;k++)
				if(strcmp(totalrules[i].rowithvalue[j],targetobjname[k])==0)
					targetobjflag[k]=1;
	}
	for(i=0;i<targetobjnum;i++)
		if(targetobjflag[i]==0)
		{
			free(totalrules);
			free(targetobjflag);
			return(0);
		}
	free(totalrules);
	free(targetobjflag);
	return(1);
}

int Membrane::check_possibility_parameter_optimization_membrane_indi(Membrane memb)
{
	int i,j,count;

	count=0;
	for(i=0;i<memb.modulesno;i++)
		for(j=0;j<memb.moduleset[i].rulesno;j++)
			if(memb.moduleset[i].ruleset[j].kcfixflag==0) 
				count++;
	if(count==0) return(0);
	else return(1);
} 

//do the parameter optimization to the kinetic constants contained in the model by using Covariance Matrix Adaptation-Evolutionary Stratege (CMA-ES)
Membrane Membrane::CMAES_parameter_optimization_membrane_indi(Membrane memb,float *q,int mnum, int maxgeno)
{
	
	Membrane tempmemb;
	Rule *totalrules;
	int totalrulesnum,N,lambda,mu,jt;
	Rule *totaloprules;
	int totaloprulesnum;
	int i,j,k,ii,jj;
	float *fitness,*muweights,*psigma,*pc;
	float tempf,mup,sum,mueff,csigma,temp1,temp2,temp,dsigma,cc,mucov,ccov,chiN,normpsigma,hsigma,delta;
	int gen;
	float **C, **tempC, **tempC1,**tempC2,**tempC3,**tempC4;
	float **B, **D, eps;
	float *xmean,*zmean,*sigma,*tempvector;
	float **xk,**zk,**yk;
	int starte,ende,mide;
	char evolvefile[100];
	FILE *fp;
	float *bestp;
	float bestfitness;
	int count; 

	starte=totalevanum;
	sprintf(evolvefile,"evolveprocess_Run%d.txt",runno);
	if((fp=fopen(evolvefile,"wt"))==NULL)
	{
		printf("Cannot open output file !\n");
		exit(1);
	}
	fclose(fp);

	
	totalrulesnum=0;
	for(i=0;i<memb.modulesno;i++)
		totalrulesnum=totalrulesnum+memb.moduleset[i].rulesno;
	totalrules=(Rule *)malloc(totalrulesnum*sizeof(Rule));
	totalrulesnum=combine_modules_into_one_ruleset(memb,totalrulesnum,totalrules);	
	totaloprulesnum=0;
	for(i=0;i<totalrulesnum;i++)
	{
		if(totalrules[i].kcfixflag==0)
			totaloprulesnum++;
	}
	totaloprules=(Rule *)malloc(totaloprulesnum*sizeof(Rule));
	j=0;		
	for(i=0;i<totalrulesnum;i++)
	{
		if(totalrules[i].kcfixflag==0)
		{
  			totaloprules[j]=totalrules[i];
			j++;
		}
	}
	N=totaloprulesnum;
	// parameter settings for some control parameters of CMA-ES	
	lambda=4+floor(3*log(N));
	fitness=(float *)malloc(lambda*sizeof(float)); 

	mup=(lambda-1)/2.0;
	mu=ceil(mup);
	
	muweights=(float *)malloc(mu*sizeof(float));
	
	for(i=0;i<mu;i++)
		muweights[i]=log(mup+1)-log(i+1);
		
	sum=0;
	for(i=0;i<mu;i++)
		sum=sum+muweights[i];
	for(i=0;i<mu;i++)
		muweights[i]=muweights[i]/sum;
	sum=0;
	for(i=0;i<mu;i++)
		sum=sum+muweights[i]*muweights[i];
	mueff=1.0/sum;
	csigma=(mueff+2)/(N+mueff+3);
	temp1=0;
	temp2=sqrt((mueff-1)/(N+1))-1;
	if(temp1>temp2)
		temp=temp1;
	else temp=temp2;
	dsigma=1+2*temp+csigma;
	cc=4.0/(N+4);
	mucov=mueff;
	temp1=1;
	temp2=(2*mucov-1)/((N+2)*(N+2)+mucov);
	if(temp1<temp2)
		temp=temp1;
	else temp=temp2;
	ccov=(1/mucov)*(2/((N+1.414)*(N+1.414)))+(1-1/mucov)*temp;
	// Initialization
	psigma=(float *)malloc(N*sizeof(float));
	bestp=(float *)malloc(N*sizeof(float));
	pc=(float *)malloc(N*sizeof(float));
	for(i=0;i<N;i++)
	{
		psigma[i]=0;
		pc[i]=0;
	}
	C=(float**)malloc(N*sizeof(float *));
	for(i=0;i<N;i++)
		C[i]=(float *)malloc(N*sizeof(float));
	
	tempC=(float**)malloc(N*sizeof(float *));
	for(i=0;i<N;i++)
		tempC[i]=(float *)malloc(N*sizeof(float));
	
	tempC1=(float**)malloc(N*sizeof(float *));
	for(i=0;i<N;i++)
		tempC1[i]=(float *)malloc(N*sizeof(float));
	
	tempC2=(float**)malloc(N*sizeof(float *));
	for(i=0;i<N;i++)
		tempC2[i]=(float *)malloc(N*sizeof(float));
	
	tempC3=(float**)malloc(N*sizeof(float *));
	for(i=0;i<N;i++)
		tempC3[i]=(float *)malloc(N*sizeof(float));
	
	tempC4=(float**)malloc(N*sizeof(float *));
	for(i=0;i<N;i++)
		tempC4[i]=(float *)malloc(N*sizeof(float));
		
	B=(float**)malloc(N*sizeof(float *));
	for(i=0;i<N;i++)
		B[i]=(float *)malloc(N*sizeof(float));
	
	D=(float**)malloc(N*sizeof(float *));
	for(i=0;i<N;i++)
		D[i]=(float *)malloc(N*sizeof(float));

	for(i=0;i<N;i++)
		for(j=0;j<N;j++)
		{
			if(i==j)
				C[i][j]=1;
			else C[i][j]=0;
		}

	gen=0;
	xmean=(float *)malloc(N*sizeof(float));
	zmean=(float *)malloc(N*sizeof(float));
	sigma=(float *)malloc(N*sizeof(float));
	tempvector=(float *)malloc(N*sizeof(float));
	zk=(float **)malloc(lambda*sizeof(float*));
	for(i=0;i<lambda;i++)
		zk[i]=(float*)malloc(N*sizeof(float));
	yk=(float **)malloc(lambda*sizeof(float*));
	for(i=0;i<lambda;i++)
		yk[i]=(float*)malloc(N*sizeof(float));
	xk=(float **)malloc(lambda*sizeof(float*));
	for(i=0;i<lambda;i++)
		xk[i]=(float*)malloc(N*sizeof(float));


	for(i=0;i<N;i++)
	{
		xmean[i]=totaloprules[i].kcvalue;
		sigma[i]=0.3*(totaloprules[i].kcupperb-totaloprules[i].kclowerb);
	}
	chiN=sqrt(N)*(1-1.0/(4*N)+1.0/(21*N*N));
	eps=0.000001;
	jt=100;
	for(i=0;i<N;i++)
		bestp[i]=totaloprules[i].kcvalue;
	bestfitness=q[mnum];
	mide=starte;
	while(gen<maxgeno)
	{
		for(i=0;i<N;i++)
			for(j=0;j<N;j++)
				tempC[i][j]=C[i][j];
		i=new_cjcbi(C,N,B,eps,jt);
		for(i=0;i<N;i++)
			for(j=0;j<N;j++)
				D[i][j]=0;
		for(i=0;i<N;i++)
			for(j=0;j<N;j++)
				if(i==j) 
					D[i][j]=sqrt(C[i][j]);
		for(i=0;i<N;i++)
			for(j=0;j<N;j++)
				C[i][j]=tempC[i][j];
		new_brmul(B,D,N,N,N,tempC);
		for(k=0;k<lambda;k++)
		{
			for(jj=0;jj<N;jj++)
			{
				count=0;
				if(strcmp(totaloprules[jj].kcscale,"linear")==0)
				{
					do{
						zk[k][jj]=Gaussian_random_number(0,1);
						yk[k][jj]=0;
						for(j=0;j<N;j++)
							yk[k][jj]=yk[k][jj]+tempC[jj][j]*zk[k][j];
						xk[k][jj]=xmean[i]+sigma[jj]*yk[k][jj];
						xk[k][jj]=ftruncate(xk[k][jj],totaloprules[jj].kcsigdigit);
						count++;
					}while((xk[k][jj]==0 || xk[k][jj]<=totaloprules[jj].kclowerb||xk[k][jj]>totaloprules[jj].kcupperb)&&(count<20));
				}
				if(strcmp(totaloprules[jj].kcscale,"logarithmic")==0)
				{
					do{
						zk[k][jj]=Gaussian_random_number(0,1);
						yk[k][jj]=0;
						for(j=0;j<N;j++)
							yk[k][jj]=yk[k][jj]+tempC[jj][j]*zk[k][j];
						xk[k][jj]=xmean[i]+sigma[jj]*yk[k][jj];
						xk[k][jj]=ftruncate(xk[k][jj],totaloprules[jj].kcsigdigit);
						count++;
					}while((xk[k][jj]<=totaloprules[jj].kclowerb||xk[k][jj]>totaloprules[jj].kcupperb)&&(count<20));
				}		
				if(count==20)
				{
					if(xk[k][jj]<=totaloprules[jj].kclowerb || xk[k][jj]>totaloprules[jj].kcupperb)
					{
						if(strcmp(totaloprules[jj].kcscale,"linear")==0)
						{				
							do{
								xk[k][jj]=randfabwithprecision(totaloprules[jj].kclowerb,totaloprules[jj].kcupperb,totaloprules[jj].kcprecision);
							}while(xk[k][jj]==0);
						}
						if(strcmp(totaloprules[jj].kcscale,"logarithmic")==0)
							xk[k][jj]=randfabwithprecision(totaloprules[jj].kclowerb,totaloprules[jj].kcupperb,totaloprules[jj].kcprecision);
					}
				}
			}

			tempmemb=memb;
			for(j=0;j<N;j++)
			{
				for(ii=0;ii<memb.modulesno;ii++)
					for(jj=0;jj<memb.moduleset[ii].rulesno;jj++)
					{
						if(temprule.check_same_singlerule(tempmemb.moduleset[ii].ruleset[jj],totaloprules[j])==1)
							tempmemb.moduleset[ii].ruleset[jj].kcvalue=xk[k][j];
					}
			}
			fitness[k]=mulinit_cal_fitness_membrane_indi(tempmemb,0);
		}
		bubble_sort_membrane_indi_parapop(lambda,N,xk,fitness);
		bubble_sort_membrane_indi_parapop(lambda,N,yk,fitness);
		bubble_sort_membrane_indi_parapop(lambda,N,zk,fitness);

		for(i=0;i<N;i++)
		{
			sum=0;
			for(j=0;j<mu;j++)
				sum=sum+muweights[j]*xk[lambda-1-j][i];
			xmean[i]=sum;
		}
		for(i=0;i<N;i++)
		{
			sum=0;
			for(j=0;j<mu;j++)
				sum=sum+muweights[j]*zk[lambda-1-j][i];
			zmean[i]=sum;
		}

		for(i=0;i<N;i++)
		{
			sum=0;
			for(j=0;j<N;j++)
				sum=sum+B[i][j]*zmean[j];
			tempvector[i]=sum;
			psigma[i]=(1-cc)*psigma[i]+sqrt(csigma*(2-csigma)*mueff)*tempvector[i];
		}
		sum=0;
		for(i=0;i<N;i++)
			sum=sum+psigma[i]*psigma[i];
		normpsigma=sqrt(sum);
		temp1=normpsigma/sqrt(1-pow((1-csigma),2*(gen+1)));
		temp2=(1.4+2.0/(N+1))*chiN;
		if(temp1<temp2)
			hsigma=1;
		else hsigma=0;
		for(i=0;i<N;i++)
		{
			sum=0;
			for(j=0;j<mu;j++)
				sum=sum+muweights[j]*yk[lambda-1-j][i];
			tempvector[i]=sum;
			pc[i]=(1-cc)*pc[i]+hsigma*sqrt(cc*(2-cc)*mueff)*tempvector[i];
			
		}
		delta=(1-hsigma)*cc*(2-cc);
		
		for(i=0;i<N;i++)
			for(j=0;j<N;j++)
				tempC1[i][j]=pc[i]*pc[j];
		
		for(i=0;i<N;i++)
			for(j=0;j<N;j++)
				tempC2[i][j]=delta*C[i][j];

		for(i=0;i<N;i++)
			for(j=0;j<N;j++)
				tempC3[i][j]=(ccov/mucov)*(tempC1[i][j]+tempC2[i][j]);
		
		for(i=0;i<N;i++)
			for(j=0;j<N;j++)
				tempC1[i][j]=(1-ccov)*C[i][j]+tempC3[i][j];

		for(j=0;j<N;j++)
			for(k=0;k<N;k++)
				tempC4[j][k]=0;

		for(i=0;i<mu;i++)
		{
			for(j=0;j<N;j++)
				for(k=0;k<N;k++)
				{
					tempC2[j][k]=yk[lambda-1-i][j]*yk[lambda-1-i][k];
					tempC3[j][k]=muweights[i]*tempC2[j][k];
					tempC4[j][k]=tempC4[j][k]+tempC3[j][k];
				}
		}
		for(i=0;i<N;i++)
			for(j=0;j<N;j++)
				tempC4[i][j]=ccov*(1-1.0/mucov)*tempC4[i][j];

		for(i=0;i<N;i++)
			for(j=0;j<N;j++)
				C[i][j]=tempC1[i][j]+tempC4[i][j];

		tempf=exp((csigma/dsigma)*(normpsigma/chiN-1));
		for(i=0;i<N;i++)
			sigma[i]=sigma[i]*tempf;
		
		bubble_sort_membrane_indi_parapop(lambda,N,xk,fitness);
		if((fp=fopen(evolvefile,"at"))==NULL)
		{
			printf("Cannot open output file !\n");
			exit(1);
		}
		fprintf(fp, "%d:  %f \n",gen,fitness[lambda-1]);		
		fclose(fp);
		if( debug_mode )
	    		printf("gen:%d  best fitness: %f \n",gen,fitness[lambda-1]);		
		if(fitness[lambda-1]<bestfitness)
		{
			for(i=0;i<N;i++)
				bestp[i]=xk[lambda-1][i];
			bestfitness=fitness[lambda-1];
		}
		gen++;
		mide=totalevanum;
	}
	/* If finally the best individual in the population can get a  better fitness than the origin model, then replace all the kc values in the model with the new parameter set. */
	if(bestfitness<q[mnum])
	{                     
		for(i=0;i<N;i++)
		{
			for(j=0;j<memb.modulesno;j++)
				for(k=0;k<memb.moduleset[j].rulesno;k++)
				{
					if(temprule.check_same_singlerule(memb.moduleset[j].ruleset[k],totaloprules[i])==1)
						memb.moduleset[j].ruleset[k].kcvalue=bestp[i];
				}
		}
		q[mnum]=bestfitness;
	}
	ende=totalevanum;
	gapeevalnum=ende-starte;

	free(fitness);
	free(muweights);
	for(i=0;i<N;i++)
	{
		free(C[i]);
		free(B[i]);
		free(D[i]);
		free(tempC[i]);
        free(tempC1[i]);
		free(tempC2[i]);
	    free(tempC3[i]);
	    free(tempC4[i]);
	}

	free(C);
	free(B);
	free(D);
	free(tempC);
    	free(tempC1);
	free(tempC2);
	free(tempC3);
	free(tempC4);
	free(xmean);
	free(zmean);
	free(sigma);
	free(tempvector);
	for(i=0;i<lambda;i++)
	{
		free(zk[i]);
		free(yk[i]);
		free(xk[i]);
	}
	free(xk);
	free(yk);
	free(zk);
    free(totalrules);
	free(totaloprules);
	return(memb);
}

Membrane Membrane::opposition_DE_parameter_optimization_membrane_indi(Membrane memb,float *q,int mnum, int oppopsize, int opmaxgeno)
{
	int i,j,k,ii,jj,L;
	float *tempp,tempq,*bestp,bestq;
	float **oldparapop,*oldparafit;
	float **oldopposiparapop,*oldopposiparafit;
	float **newparapop,*newparafit;
	float **doubleparapop,*doubleparafit;

	Membrane tempmemb;
	Rule *totalrules;
	Rule *totaloprules;
	int totalrulesnum;
	int totaloprulesnum;
	float ranf,temp;
	int gen,startn,nn;
	int *selectr;
	int starte,ende,mide;
	float crate,F;
	char evolvefile[100];
	FILE *fp;
	float *temparray,*maxpara,*minpara;

	crate=0.9;
	//F=0.5;
	sprintf(evolvefile,"evolveprocess_Run%d.txt",runno);

	if((fp=fopen(evolvefile,"wt"))==NULL)
	{
		printf("Cannot open output file !\n");
		exit(1);
	}
	fclose(fp);

	starte=totalevanum;
	mide=starte;
	totalrulesnum=0;
	for(i=0;i<memb.modulesno;i++)
		totalrulesnum=totalrulesnum+memb.moduleset[i].rulesno;
	totalrules=(Rule *)malloc(totalrulesnum*sizeof(Rule));
	totalrulesnum=combine_modules_into_one_ruleset(memb,totalrulesnum,totalrules);
	//get the total rule set of the model whose kc value is to be optimized
	totaloprulesnum=0;
	for(i=0;i<totalrulesnum;i++)
	{
		if(totalrules[i].kcfixflag==0)
			totaloprulesnum++;
	}
	totaloprules=(Rule *)malloc(totaloprulesnum*sizeof(Rule));
	j=0;		
	for(i=0;i<totalrulesnum;i++)
	{
		if(totalrules[i].kcfixflag==0)
		{
			totaloprules[j]=totalrules[i];
			j++;
		}
	}
	//allocate space for the parameter population and its fitness
	selectr=(int *)malloc(oppopsize*sizeof(int));		
	oldparapop=(float **)malloc(oppopsize*sizeof(float *));
	for(i=0;i<oppopsize;i++)
		oldparapop[i]=(float *)malloc(totaloprulesnum*sizeof(float));
	oldparafit=(float *)malloc(oppopsize*sizeof(float));

	oldopposiparapop=(float **)malloc(oppopsize*sizeof(float *));
	for(i=0;i<oppopsize;i++)
		oldopposiparapop[i]=(float *)malloc(totaloprulesnum*sizeof(float));
	oldopposiparafit=(float *)malloc(oppopsize*sizeof(float));

	newparapop=(float **)malloc(oppopsize*sizeof(float *));
	for(i=0;i<oppopsize;i++)
		newparapop[i]=(float *)malloc(totaloprulesnum*sizeof(float));
	newparafit=(float *)malloc(oppopsize*sizeof(float));

	doubleparapop=(float **)malloc(2*oppopsize*sizeof(float *));
	for(i=0;i<2*oppopsize;i++)
		doubleparapop[i]=(float *)malloc(totaloprulesnum*sizeof(float));
	doubleparafit=(float *)malloc(2*oppopsize*sizeof(float));

	tempp=(float *)malloc(totaloprulesnum*sizeof(float));
	bestp=(float *)malloc(totaloprulesnum*sizeof(float));

	temparray=(float *)malloc(oppopsize*sizeof(float));
	minpara=(float *)malloc(totaloprulesnum*sizeof(float));
	maxpara=(float *)malloc(totaloprulesnum*sizeof(float));

	//initialize the parameter population randomly 
	init_membrane_indi_para_pop(memb,oppopsize,totaloprules,totaloprulesnum,oldparapop,oldparafit);
	for(ii=0;ii<oppopsize;ii++)
	{
		for(j=0;j<totaloprulesnum;j++)
		{
			temp=totaloprules[j].kclowerb+totaloprules[j].kcupperb-oldparapop[ii][j];
			temp=ftruncate(temp,totaloprules[j].kcsigdigit);	
			if(temp<=totaloprules[j].kclowerb || temp>totaloprules[j].kcupperb)
			{
				if(strcmp(totaloprules[j].kcscale,"linear")==0)
				{
					do{
	 					temp=randfabwithprecision(totaloprules[j].kclowerb,totaloprules[j].kcupperb,totaloprules[j].kcprecision);
					}while(temp==0);
				}
				if(strcmp(totaloprules[j].kcscale,"logarithmic")==0)
					temp=randfabwithprecision(totaloprules[j].kclowerb,totaloprules[j].kcupperb,totaloprules[j].kcprecision);
			}
			oldopposiparapop[ii][j]=temp;
		}	

		tempmemb=memb;
		for(j=0;j<totaloprulesnum;j++)
		{
			for(i=0;i<tempmemb.modulesno;i++)
				for(jj=0;jj<tempmemb.moduleset[i].rulesno;jj++)
				{
					if(temprule.check_same_singlerule(tempmemb.moduleset[i].ruleset[jj],totaloprules[j])==1)
						tempmemb.moduleset[i].ruleset[jj].kcvalue=oldopposiparapop[ii][j];
				}
		}
		oldopposiparafit[ii]=mulinit_cal_fitness_membrane_indi(tempmemb,0);
	}
	for(i=0;i<oppopsize;i++)
	{
		for(j=0;j<totaloprulesnum;j++)
			doubleparapop[i][j]=oldparapop[i][j];
		doubleparafit[i]=oldparafit[i];
	}
	for(i=0;i<oppopsize;i++)
	{
		for(j=0;j<totaloprulesnum;j++)
			doubleparapop[oppopsize+i][j]=oldopposiparapop[i][j];
		doubleparafit[oppopsize+i]=oldopposiparafit[i];
	}
	bubble_sort_membrane_indi_parapop(2*oppopsize,totaloprulesnum,doubleparapop,doubleparafit);
	for(i=0;i<oppopsize;i++)
	{
		for(j=0;j<totaloprulesnum;j++)
			oldparapop[i][j]=doubleparapop[2*oppopsize-1-i][j];
		oldparafit[i]=doubleparafit[2*oppopsize-1-i];
	}
	
	bubble_sort_membrane_indi_parapop(oppopsize,totaloprulesnum,oldparapop,oldparafit);

	for(i=0;i<totaloprulesnum;i++)
		bestp[i]=oldparapop[oppopsize-1][i];
	bestq=oldparafit[oppopsize-1];
	gen=0;

	while(gen<opmaxgeno)
	{
		for(ii=0;ii<oppopsize;ii++)
		{
			selectsampleno(ii,oppopsize,selectr);
			startn=randi(totaloprulesnum);
			for(i=0;i<totaloprulesnum;i++)
				tempp[i]=oldparapop[ii][i];
			L=0;
			do{
				L=L+1;
				ranf=randf();
			} while ((ranf<crate) && (L<totaloprulesnum));
			
			for(i=startn;i<startn+L;i++)
			{
				nn=i%totaloprulesnum;
				F=randf();
				switch(randi(5))
				{
					case 0: /* DE-rand1 scheme */
						tempp[nn]=oldparapop[selectr[1]][nn]+F*(oldparapop[selectr[2]][nn]-oldparapop[selectr[3]][nn]);
						break;
					case 1: /* DE-rand2 scheme */
						tempp[nn]=oldparapop[selectr[1]][nn]+F*(oldparapop[selectr[2]][nn]+oldparapop[selectr[3]][nn]-oldparapop[selectr[4]][nn]-oldparapop[selectr[5]][nn]);
						break;
					case 2:   /* DE-best1 scheme */
						tempp[nn]=bestp[nn]+F*(oldparapop[selectr[1]][nn]-oldparapop[selectr[2]][nn]);
						break;	
					case 3:   /* DE-best2 scheme */
						tempp[nn]=bestp[nn]+F*(oldparapop[selectr[2]][nn]+oldparapop[selectr[3]][nn]-oldparapop[selectr[4]][nn]-oldparapop[selectr[5]][nn]);
						break;	
					case 4:   /* randomtobest1 scheme */
						tempp[nn]=tempp[nn]+F*(bestp[nn]-tempp[nn])+F*(oldparapop[selectr[1]][nn]-oldparapop[selectr[2]][nn]);
						break;
				}
				tempp[nn]=ftruncate(tempp[nn],totaloprules[nn].kcsigdigit);
				if(tempp[nn]<=totaloprules[nn].kclowerb || tempp[nn]>totaloprules[nn].kcupperb)
				{
					if(strcmp(totaloprules[nn].kcscale,"linear")==0)
					{
						do{
							tempp[nn]=randfabwithprecision(totaloprules[nn].kclowerb,totaloprules[nn].kcupperb,totaloprules[nn].kcprecision);
						}while(tempp[nn]==0);
					}
					if(strcmp(totaloprules[nn].kcscale,"logarithmic")==0)
						tempp[nn]=randfabwithprecision(totaloprules[nn].kclowerb,totaloprules[nn].kcupperb,totaloprules[nn].kcprecision);
				}
			}
			// keep the kc values of all rules in the model which have the same structures updated with the newly parameter value   			
			tempmemb=memb;
			for(j=0;j<totaloprulesnum;j++)
			{
				for(i=0;i<tempmemb.modulesno;i++)
					for(jj=0;jj<tempmemb.moduleset[i].rulesno;jj++)
					{
						if(temprule.check_same_singlerule(tempmemb.moduleset[i].ruleset[jj],totaloprules[j])==1)
							tempmemb.moduleset[i].ruleset[jj].kcvalue=tempp[j];
					}
			}
			tempq=tempmemb.mulinit_cal_fitness_membrane_indi(tempmemb,0);
			//If the new parameters are better than the old ones, then replace them with  the new ones. 
			if(tempq<oldparafit[ii])
			{
				for(i=0;i<totaloprulesnum;i++)
					newparapop[ii][i]=tempp[i];
				newparafit[ii]=tempq;
			}
			else
			{
				for(i=0;i<totaloprulesnum;i++)
					newparapop[ii][i]=oldparapop[ii][i];
				newparafit[ii]=oldparafit[ii];
			}
		} /* end of for */
		for(ii=0;ii<oppopsize;ii++)
		{
			for(i=0;i<totaloprulesnum;i++)
				oldparapop[ii][i]=newparapop[ii][i];
			oldparafit[ii]=newparafit[ii];
		}
		for(i=0;i<totaloprulesnum;i++)
		{
			for(j=0;j<oppopsize;j++)
				temparray[j]=oldparapop[j][i];
			maxpara[i]=calculate_maximum(temparray,oppopsize);
			minpara[i]=calculate_minimum(temparray,oppopsize);
		}
		ranf=randf();
		if(ranf<0.3)
		{
			for(ii=0;ii<oppopsize;ii++)
			{
				for(j=0;j<totaloprulesnum;j++)
				{
					temp=minpara[j]+maxpara[j]-oldparapop[ii][j];
					temp=ftruncate(temp,totaloprules[j].kcsigdigit);
					if(temp<=totaloprules[j].kclowerb || temp>totaloprules[j].kcupperb)
					{
						if(strcmp(totaloprules[j].kcscale,"linear")==0)
						{
							do{
	 							temp=randfabwithprecision(minpara[j],maxpara[j],totaloprules[j].kcprecision);
							}while(temp==0);
						}
						if(strcmp(totaloprules[j].kcscale,"logarithmic")==0)
							temp=randfabwithprecision(minpara[j],maxpara[j],totaloprules[j].kcprecision);
					}
					oldopposiparapop[ii][j]=temp;
				}
				tempmemb=memb;
				for(j=0;j<totaloprulesnum;j++)
				{
					for(i=0;i<tempmemb.modulesno;i++)
						for(jj=0;jj<tempmemb.moduleset[i].rulesno;jj++)
						{
							if(temprule.check_same_singlerule(tempmemb.moduleset[i].ruleset[jj],totaloprules[j])==1)
								tempmemb.moduleset[i].ruleset[jj].kcvalue=oldopposiparapop[ii][j];
						}
				}
				oldopposiparafit[ii]=mulinit_cal_fitness_membrane_indi(tempmemb,0);
			}
			for(i=0;i<oppopsize;i++)
			{
				for(j=0;j<totaloprulesnum;j++)
					doubleparapop[i][j]=oldparapop[i][j];
				doubleparafit[i]=oldparafit[i];
			}
			for(i=0;i<oppopsize;i++)
			{
				for(j=0;j<totaloprulesnum;j++)
					doubleparapop[oppopsize+i][j]=oldopposiparapop[i][j];
				doubleparafit[oppopsize+i]=oldopposiparafit[i];
			}
			bubble_sort_membrane_indi_parapop(2*oppopsize,totaloprulesnum,doubleparapop,doubleparafit);
			for(i=0;i<oppopsize;i++)
			{
				for(j=0;j<totaloprulesnum;j++)
					oldparapop[i][j]=doubleparapop[2*oppopsize-1-i][j];
				oldparafit[i]=doubleparafit[2*oppopsize-1-i];
			}
		}
		bubble_sort_membrane_indi_parapop(oppopsize,totaloprulesnum,oldparapop,oldparafit);
		for(i=0;i<totaloprulesnum;i++)
			bestp[i]=oldparapop[oppopsize-1][i];
		bestq=oldparafit[oppopsize-1];
		if((fp=fopen(evolvefile,"at"))==NULL)
		{
			printf("Cannot open output file !\n");
			exit(1);
		}
		fprintf(fp, "%d:  %f \n",gen,oldparafit[oppopsize-1]);		
		fclose(fp);
		if( debug_mode )
	    		printf("para gen:%d  best fitness: %f \n",gen,oldparafit[oppopsize-1]);		
		gen++;
		mide=totalevanum;
	}
	/* When the evolution is over, if the best individual in the population can get a  better fitness than the origin model, then replace all the kc values in the model with the new parameter set. */  
	if(bestq<q[mnum])
	{                     
		//printf("\nparameter optimization improved the fitness!\n"); 			
		for(i=0;i<totaloprulesnum;i++)
		{
			for(j=0;j<memb.modulesno;j++)
				for(k=0;k<memb.moduleset[j].rulesno;k++)
				{
					if(temprule.check_same_singlerule(memb.moduleset[j].ruleset[k],totaloprules[i])==1)
						memb.moduleset[j].ruleset[k].kcvalue=bestp[i];
				}
		}
		q[mnum]=bestq;
	}
	
	ende=totalevanum;
	gapeevalnum=ende-starte;
	free(selectr);
	
	for(i=0;i<oppopsize;i++)
		free(oldparapop[i]);
	free(oldparapop);       
	for(i=0;i<oppopsize;i++)
		free(newparapop[i]);
	free(newparapop);   
	for(i=0;i<oppopsize;i++)
		free(oldopposiparapop[i]);
	free(oldopposiparapop);  
	for(i=0;i<2*oppopsize;i++)
		free(doubleparapop[i]);
	free(doubleparapop);    
	free(tempp);
	free(temparray);
	free(maxpara);
	free(minpara);
	free(bestp);
	free(oldparafit);
	free(oldopposiparafit);
	free(newparafit);
	free(doubleparafit);
	free(totalrules);
	free(totaloprules);
	return(memb);
}
