// Membrane class definitions
/* Date (yymmdd):               081215
 * Version:                     0.1
 * Written by:                  Hongqing Cao
 * Additional contributions by: 
				Dr. Francisco Jos Romero Campero
				Jonathan Blakes				
				Dr. Jamie Twycross
 *                              Dr. Natalio Krasnogor
 * Funded by:                   EPSRC grant EP/E017215/1
 * Website:                     http://www.infobiotic.org/
 * Contact:                     nxk@cs.nott.ac.uk
*/ 

#ifndef MEMBRANE_H
#define MEMBRANE_H

#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <math.h>
#include <stdlib.h> 
#include <string.h>

#include "POptimizerCommon.h"
#include "POptimizerRule.h"
#include "POptimizerModule.h"

namespace POptimizer{
class Membrane
{
public:
	int type;
	int modulesno;
	Module moduleset[8];
	Rule temprule;
	Module tempmodule;
	
	Membrane();
	virtual ~Membrane();

	Membrane simplify_membrane_indi(Membrane memb);
	int check_same_membrane(Membrane memb1,Membrane memb2);
	void display_membrane_indi(Membrane memb);
	void bubble_sort_membrane_pop(int size, Membrane *pop, float *q);
	void elitism_membrane_newpop(int psize,Membrane *newmemb, Membrane *memb,float *newq, float *q);
	Membrane one_point_crossover_membrane_indi(Membrane membp1, Membrane membp2, int maxb);
	void bubble_sort_membrane_indi_parapop(int size, int rsize, float **parapop, float *q);
	void write_file_best_P_system(char fname[50], Membrane memb, float fit);
	int check_validity_membrane_indi(Membrane memb);
	void init_membrane_indi_para_pop(Membrane memb, int size, Rule *totalr, int rnum, float **parapop, float *q);
	Membrane module_exchange_crossover_membrane_indi(Membrane membp1, Membrane membp2);
	Membrane normalize_parameter__membrane_indi(Membrane memb);
	Membrane parameter_local_search_membrane_indi(Membrane memb, float *q, int pnum, int maxmunum);
	Membrane GA_parameter_optimization_membrane_indi(Membrane memb,float *q,int mnum,int oppopsize,int maxxo);
	Membrane EDA_parameter_optimization_membrane_indi(Membrane memb,float *q,int mnum, int popsize, int maxgeno);
	int EDA_Bayesian_distribution(int paranum,float *indi, float **covarmatrix, float *means, float *bvariance, float *bmeans);
	void EDA_ndf_indi_sampling(int paranum,float **covarmatrix,float *means, float *variance, float *sample, Rule *totalr);
	int combine_modules_into_one_ruleset(Membrane memb, int totalrulesnum, Rule *totalrules);
	float get_modelchecker_output(Membrane memb);
	Membrane create_membrane_indi_from_library();
	Membrane mutation_membrane_indi(Membrane parent);
	int check_possibility_parameter_optimization_membrane_indi(Membrane memb);
	float cal_fitness_membrane_indi(Membrane memb,int outputflag, int targetfileno, int nzeroobjnum, char nzeroobjname[50][50], float nzeroobjinit[50], float **targetvalue);
	int check_all_target_objects_membrane_indi(Membrane memb);
	int fixlib_combine_modules_into_one_ruleset(Membrane memb, int totalrulesnum, Rule *totalrules);
	int write_model_xmlfile_membrane_indi(char filenames[3][100],Membrane memb,int nzeroobjnum, char nzeroobjname[50][50], float nzeroobjinit[50]);
	float mulinit_cal_fitness_membrane_indi(Membrane memb,int outputflag);
	void display_fix_library_modules(void);
	void display_nonfixed_membrane_indi(Membrane memb);
	void write_model_xmlfile_modules_membrane_indi(char* filename,Membrane memb,int nzeroobjnum, char nzeroobjname[50][50], float nzeroobjinit[50]);
    Membrane opposition_DE_parameter_optimization_membrane_indi(Membrane memb,float *q,int mnum, int oppopsize, int opmaxgeno);
	Membrane CMAES_parameter_optimization_membrane_indi(Membrane memb,float *q,int mnum, int maxgeno);



};
}
#endif






				
