// Module class implementations
/* Date (yymmdd):               081215
 * Version:                     0.1
 * Written by:                  Hongqing Cao
 * Additional contributions by: 
				Dr. Francisco Jos Romero Campero
				Jonathan Blakes				
				Dr. Jamie Twycross
 *                              Dr. Natalio Krasnogor
 * Funded by:                   EPSRC grant EP/E017215/1
 * Website:                     http://www.infobiotic.org/
 * Contact:                     nxk@cs.nott.ac.uk
*/ 

#if HAVE_CONFIG_H
        #include <config.h>
#endif

#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <math.h>
#include <stdlib.h> 
#include <string.h>

#include "POptimizerCommon.h"
#include "POptimizerRule.h"
#include "POptimizerModule.h"

using namespace POptimizer;

Module::Module()
{
}

Module::~Module()
{
}

/*
instantiate a module in the library. If the random flag is 1, the module is instantiated randomly; otherwise the module is instantiated by using the input variable values
*/ 
Module Module::instantiate_module_in_library(int libindex, int randomflag)
{
	Module module;
	int i,j,k,ii,jj,pos;
	char *pdest,headstr[100],tailstr[100],tempstr[600];

	//ge the module in the library with specified index
	module=modulelib[libindex];
//	printf("module name:%s varnum=%d\n",module.name,module.varnum);
	if(module.fixflag==1)
		return(module);
	for(i=0;i<module.varnum;i++)
	{
		if(randomflag==1)
		{
//			printf("varvaluenum=%d\n",varvaluenum);			
			k=randi(varvaluenum);
//			printf("k=%d\n",k);
			module.varvalueindex[i]=k;
			
		}
		else
			module.varvalueindex[i]=varindex[i];
		strcpy(module.varvaluestr[i],varvaluename[module.varvalueindex[i]]);
	}
//	printf("111\n");
	for(i=0;i<module.rulesno;i++)
	{
		if(module.ruleset[i].objfixflag==1)
		{
			for(j=0;j<module.ruleset[i].lonum;j++)
				strcpy(module.ruleset[i].lowithvalue[j],module.ruleset[i].lowithvar[j]);
			for(j=0;j<module.ruleset[i].ronum;j++)
				strcpy(module.ruleset[i].rowithvalue[j],module.ruleset[i].rowithvar[j]);
		}
		else
		{
			for(j=0;j<module.ruleset[i].lonum;j++)
			{
				strcpy(module.ruleset[i].lowithvalue[j],module.ruleset[i].lowithvar[j]);				
				for(k=0;k<module.varnum;k++)
				{
					strcpy(tempstr,module.ruleset[i].lowithvalue[j]);
					pdest=strstr(tempstr,module.varname[k]);
					if(pdest!=NULL)				
					{
						pos=pdest-tempstr;
						for(ii=0;ii<pos;ii++)
							headstr[ii]=tempstr[ii];
						headstr[ii]='\0';
						jj=0;
						for(ii=pos+strlen(module.varname[k]);ii<strlen(tempstr);ii++)
							{
							tailstr[jj]=tempstr[ii];
							jj++;
						}
						tailstr[jj]='\0';
						strcpy(tempstr,headstr);
						strcat(tempstr,module.varvaluestr[k]);
						strcat(tempstr,tailstr);
					}
					strcpy(module.ruleset[i].lowithvalue[j],tempstr);
				}
			}
			for(j=0;j<module.ruleset[i].ronum;j++)
			{
				strcpy(module.ruleset[i].rowithvalue[j],module.ruleset[i].rowithvar[j]);				
				for(k=0;k<module.varnum;k++)
				{
					strcpy(tempstr,module.ruleset[i].rowithvalue[j]);
					pdest=strstr(tempstr,module.varname[k]);
					if(pdest!=NULL)				
					{
						pos=pdest-tempstr;
						for(ii=0;ii<pos;ii++)
							headstr[ii]=tempstr[ii];
						headstr[ii]='\0';
						jj=0;
						for(ii=pos+strlen(module.varname[k]);ii<strlen(tempstr);ii++)
							{
							tailstr[jj]=tempstr[ii];
							jj++;
						}
						tailstr[jj]='\0';
						strcpy(tempstr,headstr);
						strcat(tempstr,module.varvaluestr[k]);
						strcat(tempstr,tailstr);
					}
					strcpy(module.ruleset[i].rowithvalue[j],tempstr);
				}
			}
		}
//		printf("222\n");
		// If the kc is fixed, assign the value as the fixed value
		if(module.ruleset[i].kcfixflag==1)
			;
		//module.ruleset[i].kcvalue=module.ruleset[i].kcvalue;
		// If the kc is not fixed, randomly generate a floating number within the predefined range and precision 
		if(module.ruleset[i].kcfixflag==0)
		{
			if(strcmp(module.ruleset[i].kcscale,"linear")==0)
			{
				do
				{
					module.ruleset[i].kcvalue=randfabwithprecision(module.ruleset[i].kclowerb,module.ruleset[i].kcupperb,module.ruleset[i].kcprecision);
				}while(module.ruleset[i].kcvalue==0);
			};
			if(strcmp(module.ruleset[i].kcscale,"logarithmic")==0)
				module.ruleset[i].kcvalue=randfabwithprecision(module.ruleset[i].kclowerb,module.ruleset[i].kcupperb,module.ruleset[i].kcprecision);
		}
	}
	return(module);
}

//check if two modules are the same and return 1 (same) or 0 (different)
int Module::check_same_module(Module b1,Module b2)
{
	int i;
			
	// If two modules have different index or have different number of variables, they are different.	
	if(b1.index!=b2.index || b1.varnum!=b2.varnum) 
		return(0);
	else
	{
		for(i=0;i<b1.varnum;i++)
			if(b1.varvalueindex[i]!=b2.varvalueindex[i])
				return(0);
		return(1);
	}
}

