
// Rule class implementations
/* Date (yymmdd):               081215
 * Version:                     0.1
 * Written by:                  Hongqing Cao
 * Additional contributions by: 
				Dr. Francisco Jos Romero Campero
				Jonathan Blakes				
				Dr. Jamie Twycross
 *                              Dr. Natalio Krasnogor
 * Funded by:                   EPSRC grant EP/E017215/1
 * Website:                     http://www.infobiotic.org/
 * Contact:                     nxk@cs.nott.ac.uk
*/ 

#if HAVE_CONFIG_H
        #include <config.h>
#endif

#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <math.h>
#include <stdlib.h> 
#include <string.h>

#include "POptimizerCommon.h"
#include "POptimizerRule.h"

using namespace POptimizer;

POptimizer::Rule::Rule()
{
}

POptimizer::Rule::~Rule()
{
	
}

//check if two rules are the same and return 1 (same) or 0 (different)
int POptimizer::Rule::check_same_singlerule(Rule r1,Rule r2)
{
	int i;
	// If two rules have different number of objects on either left side or right side, they are different.	
	if(r1.lonum!=r2.lonum || r1.ronum!=r2.ronum)
		return(0);
	// If two rules are the same type	
	else
	{
		for(i=0;i<r1.lonum;i++)
			if(strcmp(r1.lowithvalue[i],r2.lowithvalue[i])!=0)
				return(0);
		for(i=0;i<r1.ronum;i++)
			if(strcmp(r1.rowithvalue[i],r2.rowithvalue[i])!=0)
				return(0);
		return(1);
	}
}

//display the rule structure and kc value of a rule on the screen.
void POptimizer::Rule::display_singlerule(Rule rule)
{
	char str[2000];
	int i;
	sprintf(str,"%s:  ",rule.name);
	for(i=0;i<rule.lonum-1;i++)
	{
		strcat(str,rule.lowithvalue[i]);
		strcat(str,"+");
	}
	strcat(str,rule.lowithvalue[i]);
	strcat(str,"-->");
	if(rule.ronum==0) 
		strcat(str,"Q");
	else
	{
		for(i=0;i<rule.ronum-1;i++)
		{
			strcat(str,rule.rowithvalue[i]);
			strcat(str,"+");
		}
		strcat(str,rule.rowithvalue[i]);
	}
	printf("%s",str);
	printf("\nkc value=%f scale=%s \n\n",rule.kcvalue,rule.kcscale);
}

//Simplify a rule set by deleting the redundant rules within it and return the final total number of rules
int POptimizer::Rule::simplify_ruleset(Rule *totalr, int rnum)
{
	int i,j,sno,same,remained[200],nremain;

	nremain=1;	
	for(i=0; i<rnum; i++)
	{
		sno=-1;
		same=0;
		if(i==0)   
		{
			sno=0;
			remained[0]=0;
			nremain=1;
		}
		else
		{
			for(j=0;j<nremain;j++)
			{
				if(check_same_singlerule(totalr[i],totalr[remained[j]])==1) 
				{
					same=1;
					break;
				}
			}
			if(same==0) 
			{
				sno=i;
				remained[nremain]=i;
				nremain++;
			}
		}
	}
	rnum=nremain;
	for(i=0;i<rnum;i++)
		totalr[i]=totalr[remained[i]];
	return(rnum);
}

//get the names of all the objects turning up in a rule set and return the total number of objects 
int POptimizer::Rule::get_objects_in_ruleset(Rule *rules, int rnum, int nonzobjnum, char nonzobjname[50][50], float nonzobjinit[50])
{
	int i,j,k,kk,sameflag;

	kk=0;
	for(i=0;i<rnum;i++)
	{
		for(j=0;j<rules[i].lonum;j++)
		{
			sameflag=0;
			for(k=0;k<kk;k++)
				if(strcmp(rules[i].lowithvalue[j],allobjname[k])==0)
					sameflag=1;
			if(sameflag==0)
			{
				strcpy(allobjname[kk],rules[i].lowithvalue[j]);
				kk++;
			}
		}
		for(j=0;j<rules[i].ronum;j++)
		{
			sameflag=0;
			for(k=0;k<kk;k++)
				if(strcmp(rules[i].rowithvalue[j],allobjname[k])==0)
					sameflag=1;
			if(sameflag==0)
			{
				strcpy(allobjname[kk],rules[i].rowithvalue[j]);
				kk++;
			}
		}
	}

	for(i=0;i<kk;i++)
	{
		allobjinit[i]=0;
		for(j=0;j<nonzobjnum;j++)
			if(strcmp(allobjname[i],nonzobjname[j])==0)
			{
				allobjinit[i]=nonzobjinit[j];
				break;
			}
	}
	return(kk);
}

/*get the simulation output for all the objects contained in the ruleset by using Gillespie algorithm and return the number of the time steps as the function value */
int POptimizer::Rule::Gillespie_algorithm(Rule *totalrules, int totalrulesnum,float **outputobj)
{
	int i,j,k,kk,ruleno,step;
	float *propensity,totalprop,sum,temp;
	float ran1,ran2;
	float deltat;
	float tempkc;
	int numchange,sameflag,ruleobjnum,rulerealobjnum;
	char ruleallobjname[4][50];
	float *previousoutputobj, *currentoutputobj;
	float time;
	
	previousoutputobj=(float *)malloc(allobjnum*sizeof(float));
	currentoutputobj=(float *)malloc(allobjnum*sizeof(float));
	
	//printf("in000\n");
	ruleno=0;
	propensity=(float *)malloc(totalrulesnum*sizeof(float));
	step=0;
	time=0.0;
	//printf("in*******\n");



	//printf("allobjnum=%d  \n",allobjnum,maxsteps);
       
	for(i=0;i<allobjnum;i++)
		for(j=0;j<timesteps;j++)
			outputobj[i][j]=0;
	for(i=0;i<allobjnum;i++)
		outputobj[i][0]=allobjinit[i];
	for(i=0;i<allobjnum;i++)
	{
		currentoutputobj[i]=allobjinit[i];
		previousoutputobj[i]=allobjinit[i];	
	}
	//for(i=0;i<allobjnum;i++)
	//	printf("obj[%d]:%s    %f\n",i,allobjname[i],allobjinit[i]);
	//getchar();

	kk=1;
	/*if((fp=fopen("result.txt","wt"))==NULL)
	{
		printf("Cannot open output file !\n");
		exit(1);
	}
	fprintf(fp,"allobjnum=%d \n",allobjnum);
	strcpy(st,"step  time   deltat  ");
       	for(i=0;i<10;i++)
	{
		sprintf(str1,"%s  ",allobjname[i]);
		strcat(st,str1);
	}
	fprintf(fp,"%s",st);
	*/
	while(time<=maxtime && step<maxsteps)
	{
		for(i=0;i<totalrulesnum;i++)
		{

			if(strcmp(totalrules[i].kcscale,"linear")==0)
				tempkc=totalrules[i].kcvalue;
			if(strcmp(totalrules[i].kcscale,"logarithmic")==0)
				tempkc=pow((float)10.0, (float)totalrules[i].kcvalue);
			if(totalrules[i].lonum==1)
				for(j=0;j<allobjnum;j++)
				{
					if(strcmp(totalrules[i].lowithvalue[0],allobjname[j])==0)
					{
						propensity[i]=tempkc*currentoutputobj[j];
						break;
					}
				}
			if(totalrules[i].lonum==2)
			{
				if(strcmp(totalrules[i].lowithvalue[0],totalrules[i].lowithvalue[1])==0)
					for(j=0;j<allobjnum;j++)
					{
						if(strcmp(totalrules[i].lowithvalue[0],allobjname[j])==0)
						{
							propensity[i]=tempkc*(currentoutputobj[j]*(currentoutputobj[j]-1)/2.0);
							break;
						}
					}
				if(strcmp(totalrules[i].lowithvalue[0],totalrules[i].lowithvalue[1])!=0)
				{
					for(j=0;j<allobjnum;j++)
					{
						if(strcmp(totalrules[i].lowithvalue[0],allobjname[j])==0)
							break;
					}
					for(k=0;k<allobjnum;k++)
					{
						if(strcmp(totalrules[i].lowithvalue[1],allobjname[k])==0)
							break;
					}
					propensity[i]=tempkc*currentoutputobj[j]*currentoutputobj[k];
				}
			}
		}
		sum=0;
		for(i=0;i<totalrulesnum;i++)
			sum=sum+propensity[i];
		totalprop=sum;
		if(totalprop==0)
		{
			free(propensity);
			return(maxsteps);
		}
		do{
			ran1=randf();
		}while (ran1==0);
		do{
			ran2=randf();
		}while(ran2==0);
//		printf("in111\n");
		deltat=1/totalprop*log(1/ran1);
		temp=ran2*totalprop;
		sum=0;
		for(i=0;i<totalrulesnum;i++)
		{
			sum=sum+propensity[i];
			if(sum>temp) 
			{
				ruleno=i;
				break;
			}
		}
		step++;
		if(step==maxsteps-1)
		{
			free(propensity);			
			return(maxsteps);
		}
//		printf("in222\n");
		for(i=0;i<allobjnum;i++)
			previousoutputobj[i]=currentoutputobj[i];
		ruleobjnum=totalrules[ruleno].lonum+totalrules[ruleno].ronum;
		j=0;
		strcpy(ruleallobjname[0],totalrules[ruleno].lowithvalue[0]);
		for(i=0;i<totalrules[ruleno].lonum;i++)
		{
			sameflag=0;
			for(k=0;k<j;k++)
				if(strcmp(totalrules[ruleno].lowithvalue[i],ruleallobjname[k])==0)
					sameflag=1;
			if(sameflag==0)
			{
				strcpy(ruleallobjname[j],totalrules[ruleno].lowithvalue[i]);
				j++;
			}
		}
		for(i=0;i<totalrules[ruleno].ronum;i++)
		{
			sameflag=0;
			for(k=0;k<j;k++)
				if(strcmp(totalrules[ruleno].rowithvalue[i],ruleallobjname[k])==0)
					sameflag=1;
			if(sameflag==0)
			{
				strcpy(ruleallobjname[j],totalrules[ruleno].rowithvalue[i]);
				j++;
			}
		}
//		printf("in333\n");
		rulerealobjnum=j;
		for(i=0;i<rulerealobjnum;i++)
		{
			numchange=0;
			for(j=0;j<totalrules[ruleno].lonum;j++)
				if(strcmp(ruleallobjname[i],totalrules[ruleno].lowithvalue[j])==0)
					numchange--;
			for(j=0;j<totalrules[ruleno].ronum;j++)
				if(strcmp(ruleallobjname[i],totalrules[ruleno].rowithvalue[j])==0)
					numchange++;
			for(j=0;j<allobjnum;j++)
				if(strcmp(ruleallobjname[i],allobjname[j])==0)
				{
					currentoutputobj[j]=previousoutputobj[j]+numchange;
					break;
				}
		}
		time=time+deltat;
		if(time>kk*interval)
		{
			for(i=0;i<allobjnum;i++)
				outputobj[i][kk]=previousoutputobj[i];
			kk++;		
		}
		if(time==kk*interval)
		{
			for(i=0;i<allobjnum;i++)
				outputobj[i][kk]=currentoutputobj[i];
			kk++;		
		}
	//	fprintf(fp,"\n%d  %.10f  %.10f",step,time,deltat);
//printf("\nstep=%d  time=%.10f deltat=%.10f",step,time,deltat);
	}
	//fclose(fp); 
	free(propensity);
	free(previousoutputobj);
	free(currentoutputobj);
	return(step);
}

